<?php

namespace Service;

use Service\ShellScriptCommands;
use Util\InstanceProvider;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class CreateTestEnvironmentScript {

    /** @var EntityManager $entityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function createScript($productRepositories, $currentRow, $branchToHomologate, $pathToTest, $productFiles,
                                 $scriptBeforeProductFiles, $scriptAfterProductFiles, $repositoriesWithNewPath, $subReleaseName,
                                 $configurationFiles)
    {
        $script = ShellScriptCommands::getLogInfo('Iniciando o processo');
        $script .= ShellScriptCommands::getLogInfo('Obtendo diretório local');
        $script .= 'LOCAL_PATH=$PWD' . PHP_EOL . PHP_EOL;

        $mainDirectory = $this->getMainDirectory($productRepositories);
        $script .= ShellScriptCommands::getComment('Clonando repositórios / Referenciando tags e branch principal / Atualizando pasta base ');
        foreach ($productRepositories as &$pr) {
            $pr['DIRECTORY'] = strstr($pr['PATH'], $mainDirectory);
            $script .= 'if [ -e "./' . $pr['DIRECTORY'] . '" ]' . PHP_EOL;
            $script .= 'then ' . PHP_EOL;

            $script .= ShellScriptCommands::tab() . ShellScriptCommands::getComment('Buscando referências remotas');

            $script .= ShellScriptCommands::tab() . ShellScriptCommands::cd($pr['DIRECTORY']);
            $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo('Buscando referências remotas | comando: git fetch --all | url => \\"' . $pr['URL'] .'\\"');
            $script .= ShellScriptCommands::tab() . ShellScriptCommands::gitFetch('--all');

            $script .= $this->resetAllInRepositories($pr, $branchToHomologate);
            $script .= $this->pullBranchInRepositories($branchToHomologate);
            $script .= ShellScriptCommands::tab() . $this->cdLocalPath();

            $script .= 'else ' . PHP_EOL;
            $script .= $this->validatePathRepositoriesRelease($pr);
            $script .= 'fi'  . PHP_EOL . PHP_EOL;
        }
        $pathToCopy = strstr($pathToTest, $mainDirectory);

        $script .= ShellScriptCommands::getComment('Criando diretório de teste');
        $script .= "rm -rf {$pathToCopy}"  . PHP_EOL;
        $script .= ShellScriptCommands::getLogInfo("Copiando diretório: {$mainDirectory} => {$pathToCopy}");
        $script .= "cp -rf {$mainDirectory} {$pathToCopy}"  . ShellScriptCommands::orMsgErrorAndExit("Copiando diretório {$mainDirectory} => $pathToCopy") . PHP_EOL . PHP_EOL;

        $script .= ShellScriptCommands::getComment('Movendo arquivos de script');
        $script .= ShellScriptCommands::getLogInfo('Movendo arquivos de script...') . PHP_EOL;
        $script .= $this->createPathAndMoveFile($productFiles, $pathToCopy);

        if(!empty($scriptBeforeProductFiles)) {
            $script .= ShellScriptCommands::getLogInfo('Executando scripts before test...') . PHP_EOL;
            $script .= $this->executeScripts($scriptBeforeProductFiles, $pathToCopy);
        }

        $script .= ShellScriptCommands::getLogInfo('Atualizando branches...') . PHP_EOL;
        $script .= $this->createBranches($repositoriesWithNewPath, $subReleaseName, $branchToHomologate, $mainDirectory);

        if(!empty($scriptAfterProductFiles)) {
            $script .= ShellScriptCommands::getLogInfo('Executando scripts after test...') . PHP_EOL;
            $script .= $this->executeScripts($scriptAfterProductFiles, $pathToCopy);
        }

        $script .= ShellScriptCommands::getLogInfo("Compactando pasta ({$pathToCopy}) em tar.gz...") . PHP_EOL;
        $script .= "cd {$pathToCopy} && tar -czf ../{$pathToCopy}.tar.gz ." . ShellScriptCommands::orMsgErrorAndExit("Erro ao tentar criar um arquivo tar.gz do diretório {$pathToCopy}");

        return $script;
    }

    private function validatePathRepositoriesRelease($productRepository)
    {
        $script = '';
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo('Criando diretório \\"' . $productRepository['DIRECTORY'] .'\\"');
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::createPath($productRepository['DIRECTORY']);
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo('Clonando url \\"' . $productRepository['URL'] .'\\"');
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::gitClone($productRepository['URL'], $productRepository['DIRECTORY']);
        return $script;
    }

    private function pullTagsInRepositories($productRepository)
    {
        $script = '';
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::cd($productRepository['DIRECTORY']);
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo('Buscando referência das tags | comando: git fetch --tags | url => \\"' . $productRepository['URL'] .'\\"');
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::gitFetchTags();
        $script .= ShellScriptCommands::tab() . $this->cdLocalPath();
        return $script;
    }

    private function resetAllInRepositories($productRepository, $branchToHomologate)
    {
        $script = '';
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo("Referenciando/criando branch local à uma remota. Branch => {$branchToHomologate}");
        $script .= ShellScriptCommands::tab() . $this->branchSpecificRemote($branchToHomologate);
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo("Comando => git checkout {$branchToHomologate}");
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::gitCheckout($branchToHomologate);
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo("Comando => git reset --hard | branch => {$branchToHomologate}");
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::gitReset('--hard');
        return $script;
    }

    private function pullBranchInRepositories($branchToHomologate)
    {
        $script = '';
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::getLogInfo("Comando => git pull origin {$branchToHomologate}");
        $script .= ShellScriptCommands::tab() . ShellScriptCommands::gitPull($branchToHomologate);
        return $script;
    }

    private function getMainDirectory($productRepositories)
    {
        $mainDirectory = '';
        foreach ($productRepositories as $pr) {
            if($pr['IS_MASTER'] == 'Y') {
                $mainDirectory = substr(strrchr($pr['PATH'], '/'), 1);
            }
        }
        return $mainDirectory;
    }

    private function getMainProduct($productRepositories)
    {
        $mainProduct = array();
        foreach ($productRepositories as $pr) {
            if($pr['IS_MASTER'] == 'Y') {
                $mainProduct['MAIN_DIRECTORY'] = substr(strrchr($pr['PATH'], '/'), 1);
                $mainProduct['URL'] = $pr['URL'];
                $mainProduct['PATH'] = $pr['PATH'];
            }
        }
        return $mainProduct;
    }

    private function cdLocalPath()
    {
        return 'cd $LOCAL_PATH' . PHP_EOL;
    }

    private function branchSpecificRemote($branchName)
    {
        return "git branch {$branchName} origin/{$branchName}" . PHP_EOL;
    }

    private function createPathAndMoveFile($files, $pathToCopy)
    {
        $script = '';
        foreach ($files as $file) {
            $filePath = $pathToCopy . $file['RELATIVE_PATH'];
            $productFileName = $file['FILE_NAME'];
            $script .= 'if [ -e "./' . $filePath . '" ]' . PHP_EOL;
            $script .= 'then' . PHP_EOL;
            $script .= ShellScriptCommands::tab() . ShellScriptCommands::createPath($filePath);
            $script .= ShellScriptCommands::tab() . ShellScriptCommands::copyFile('scripts/' . $productFileName, $filePath);
            $script .= ShellScriptCommands::tab() . "chmod 775 {$filePath}/{$productFileName}" . PHP_EOL;
            $script .= 'else' . PHP_EOL;
            $script .= ShellScriptCommands::tab() . ShellScriptCommands::copyFile('scripts/' . $productFileName, $filePath);
            $script .= ShellScriptCommands::tab() . "chmod 775 {$filePath}/{$productFileName}" . PHP_EOL;
            $script .= 'fi' . PHP_EOL . PHP_EOL;
        }
        return $script;
    }

    private function executeScripts($scriptBeforeProductFiles, $pathToCopy, $noTreatment = false)
    {
        $script = '';
        foreach ($scriptBeforeProductFiles as $file) {
            $filePath = $pathToCopy . $file['RELATIVE_PATH'];
            $script .= ShellScriptCommands::cd($filePath);
            if($noTreatment){
                $script .= ShellScriptCommands::executeShellScriptNoTreatment($file['FILE_NAME']);
            } else {
                $script .= ShellScriptCommands::executeShellScript($file['FILE_NAME']);
            }
            $script .= $this->cdLocalPath() . PHP_EOL;
        }
        return $script;
    }

    private function createBranches($repositoriesWithNewPath, $subReleaseName, $branchToHomologate, $mainDirectory)
    {
        $script = '';
        foreach ($repositoriesWithNewPath as $repository) {
            $script .= ShellScriptCommands::cd(strstr($repository['PATH'], $mainDirectory));
            $script .= ShellScriptCommands::gitCheckout($branchToHomologate);
            $script .= ShellScriptCommands::gitBranch($subReleaseName);
            $script .= ShellScriptCommands::gitCheckout($subReleaseName);
            foreach ($repository['BRANCHES'] as $branch) {
                $script .= ShellScriptCommands::gitPull($branch);
            }
            $script .= $this->cdLocalPath() . PHP_EOL;
        }
        return $script;
    }

    public function getReadmeFileText()
    {
        return "- O script foi desenvolvido para ser executado em ambiente linux.
- A execução do script no cloud9 pode (e deve) demorar muito. Recomendável utilizar a própria máquina.
- Não altere a estrutura dos arquivos que foram extraídos.

- Verifique se os scripts cadastrados no DevOps possuem alguma relação direta com o servidor de teste. Como a execução será local, isso não irá funcionar.
    - Exemplo: A cópia dos módulos de LOGIN e ACL deverá apontar para um diretório em seu ambiente local.

- Caso necessário, lembre-se de exportar o proxy antes de iniciar o processo. Nas máquinas dos desenvolvedores, geralmente é necessário. Linhas de comando:
    - Exemplo: export HTTP_PROXY=\"http://192.168.122.121:3128\" && HTTPS_PROXY=\"http://192.168.122.121:3128\"
    
- Já no diretório de teste, onde os arquivos foram extraídos, dê permisão para o arquivo testEnvironment.sh e o execute.
    - Exemplo de comando para permissão: chmod a+x testEnvironment.sh
    - Exemplo de comando para executar: ./testEnvironment.sh

- No final, se tudo der certo, será gerado um arquivo compactado, no formato tar.gz, com o mesmo nome da pasta que foi gerada para a Sub-release.
    - Esse passo pode demorar alguns instantes, visto que compactar um produto e no mínimo dois módulos são muitos arquivos (3 vendor's, 3 bower's, etc...).
    - Utilize a action \"Upload Test Environment\" para subir seu ambiente compactado para o servidor e gerar as tarefas de teste.

- É recomendável utilizar sempre o mesmo diretório para gerar os teste. Dessa forma, a pasta base que foi gerada será reutilizada para agilizar o processo, evitando que o clone dos repositórios seja realizado todas as vezes.";
    }
}