<?php

namespace Service;

use Doctrine\ORM\Mapping\Entity;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Devops\Entities as DevOpsEntities;
use Zeedhi\DevOps\Util\Environment;
use Util\Messages;

class ProductVersion {
    
     


    public function findLTSProductVersion($productId, $conditions, $page, $pageSize) {
        $productVersionRepository = InstanceProvider::getRepository()->getProductVersion();
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $filters = Utilities::createQueryFiltered($conditions);
        $limit   = $page * $pageSize;
        $offset  = ($page - 1) * $pageSize;
        return $productVersionRepository->findLtsVersionWithoutBuildWithZeroVersion($productId, $organizationId, $limit, $offset, $filters);
    }
    
    public function minimumProductVersion($version) {
        $productVersionProductsRepository = InstanceProvider::getRepository()->getProductVersionProducts();
        return $productVersionProductsRepository->getProductVersionsProducts($version);
        
    }
    
    public function minimumVersion($product) {
        
        $productVersionRepository = InstanceProvider::getRepository()->getProductVersion();
        return $productVersionRepository->findProductVersionWithConcatVersionByProduct($product, InstanceProvider::getLibEnvironment()->getCurrentOrganizationId());
    }
    
    public function saveProductVersionProducts($id, $productId, $productVersionId, $depProductVersionId) {
         $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
       
        try {
            $entity = null;
            if($id){
                $entity = InstanceProvider::getRepository()->getProductVersionProducts()->find($id);
            }
            $productVersionProducts = InstanceProvider::getFactory()->getProductVersionProducts()->save($entity, $productId, $productVersionId, $depProductVersionId);
            $entityManager->persist($productVersionProducts);
            $entityManager->flush();
            $connection->commit();
            return $productVersionProducts->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
       
        
    }
    
    public function removeProductVersionProducts($id) {
         $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            if($id){
                $entity = InstanceProvider::getRepository()->getProductVersionProducts()->find($id);
                $entityManager->remove($entity);
                $entityManager->flush();
                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
       
        
    }

    public function findModulesConfiguration($conditions)
    {
        $productVersionRepository = InstanceProvider::getRepository()->getProductVersion();
        $productVersionId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
        $productVersion = $productVersionRepository->find($productVersionId);
        $modulesConfiguration = $productVersion->getModulesConfiguration();
        $modules = [];
        if($modulesConfiguration) {
        	$modulesConfiguration = json_decode($modulesConfiguration, true);
        	foreach($modulesConfiguration as $key => $module) {
		        $onlyControl = isset($module['onlyControl']) ? $module['onlyControl'] : false;
		        if(!$onlyControl) {
		        	$product   = InstanceProvider::getRepository()->getProduct()->find($module['devOpsId']);
                    $isProduct = $key == 'product';
                    $modules[] = array (
                        'NAME'             => $product ? $product->getName() : $key,
                        'MAX_VERSION'      => isset($module['maxVersion']) ? $module['maxVersion'] : null,
                        'PARENT_DEVOPS_ID' => $isProduct ? null : $modulesConfiguration['product']['devOpsId'],
                        'DEVOPS_ID'        => $module['devOpsId']
                    );
		       }
    		}
        }
        return $modules;
    }

    /**
     * @param $rows
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function saveModulesConfiguration($rows)
    {
        $productVersionId = $rows[0]['PRODUCT_VERSION_ID'];
        $productVersionRepository = InstanceProvider::getRepository()->getProductVersion();

        /** @var \Zeedhi\DevOps\Model\Devops\Entities\ProductVersion $productVersion */
        $productVersion = $productVersionRepository->find($productVersionId);
        $modulesConfiguration = $productVersion->getModulesConfiguration();
        if($modulesConfiguration) {
            $entityManager = InstanceProvider::getEntityManager();

            $modulesConfiguration = json_decode($modulesConfiguration, true);
            foreach ($modulesConfiguration as &$module){
                foreach ($rows as $row){
                    if($row['DEVOPS_ID'] === $module['devOpsId']) {
                        $module['maxVersion'] = $row['MAX_VERSION'];
                    }
                }
            }

            $modulesConfiguration = json_encode($modulesConfiguration);
            $productVersion->setModulesConfiguration($modulesConfiguration);
            $entityManager->persist($productVersion);
            $entityManager->flush();
        }
    }

    public function getCurrentBuildFromVersion($productVersionId) {
        $currentVersionBuild = array (
            'VERSION_TAG' => null
        );

        if($productVersionId) {
            /** @var DevOpsEntities\ProductVersion $productVersion */
            $productVersion = InstanceProvider::getRepository()->getProductVersion()->find($productVersionId);
            $currentBuild   = InstanceProvider::getRepository()->getProductVersion()->findCurrentBuildFromVersion (
                $productVersion->getProduct()->getId(), InstanceProvider::getLibEnvironment()->getCurrentOrganizationId(),
                $productVersion->getVersion(), $productVersion->getRevision()
            )[0];
            $currentVersionBuild['VERSION_TAG'] = 'PROD_' . $productVersion->getProduct()->getId() . '_' . $currentBuild['VERSION'];
            $currentVersionBuild['VERSION']     = $currentBuild['PR_VERSION'];
            $currentVersionBuild['REVISION']    = $currentBuild['PR_REVISION'];
            $currentVersionBuild['BUILD']       = $currentBuild['PR_BUILD'];
        }

        return $currentVersionBuild;
    }

    public function findLTSProductVersionToRemove($productId, $filterDate) {
        if(empty($filterDate)) {
            return array();
        }
        return InstanceProvider::getRepository()->getProductVersion()->findLTSProductVersionToRemove($productId, $filterDate);
    }

    public function removeLTSFromVersions($rows) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
       
        try {
            
            $productVersionIds = Utilities::getArrayFromArrayAssociative($rows, 'ID');
            $productVersionIdsFilter = Utilities::getStringForArray($productVersionIds);
            $isNotHomologated = InstanceProvider::getRepository()->getProductVersion()->versionWithRelationshipWithHomologatedSubRelease($productVersionIdsFilter);
            
            if(empty($isNotHomologated)){
                foreach ($rows as $row) {
                    /** @var DevOpsEntities\ProductVersion $productVersion */
                    $productVersion = InstanceProvider::getRepository()->getProductVersion()->find($row['ID']);
                    $productVersion->setIsLts(DevOpsEntities\ProductVersion::IS_LTS_NO);
                    $entityManager->persist($productVersion);
                }
    
                $entityManager->flush();
                $connection->commit();
            } else {
                $msg = Messages::NO_PERMISSION_REMOVE_LTS_VERSIONS;
                foreach($isNotHomologated as $version) {
                    $msg .= '<li>' . $version['VERSION'] . '</li>';
                }
                throw new \Exception($msg);
            }
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function validateRemoveLTSVersion($productVersionId)
    {
        $isNotHomologated = InstanceProvider::getRepository()->getProductVersion()->versionWithRelationshipWithHomologatedSubRelease($productVersionId);
            
        if(!empty($isNotHomologated)){
            throw new \Exception(Messages::NO_PERMISSION_REMOVE_LTS_VERSION);
        }
    }
}
