<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Task extends EntityRepository {

	const ALLOWENDTASK_NEGATIVE = 'N';
	const CONTROL_TASK_STATUS_CONCLUDED = 'C';
  const CONTROL_TASK_STATUS_IN_DEVELOPMENT = 'D';


    /*
     * Phillipe Silva - 22/12/2015
     * A classe \Service\Allocation não existe no DevOps.
     * Com isso é necessário adicionar as strings fixas.
     */
    const RESOURCE_FOLLOWER = 'F';// \Service\Allocation::FOLLOWER;
    const RESOURCE_REPORTER = 'R';// \Service\Allocation::REPORTER;

	/**
	 * @param $taskId
	 * @return array
	 */
	public function getChildren($taskId){
        return $this->findBy(array('id' => Utilities::getAllTaskChildrenIdInArrayByCalc($taskId)));
	}/**
	 * @param $taskId
	 * @return array
	 */
	public function getAllChildren($taskId){
        return $this->findBy(array('id' => Utilities::getAllTaskChildrenIdInArray($taskId)));
	}

    public function getStartedTasksByUser($resourceId) {
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
                    $query = <<<SQL
                SELECT T.NAME AS TASK, P.NAME AS PROJECT
                FROM TASK T
                INNER JOIN TASK_STATUS TS ON TS.ID = T.TASK_STATUS
                INNER JOIN PROJECT P ON P.ID = T.PROJECT
                INNER JOIN ALLOCATION A ON T.ID = A.TASK
                LEFT  JOIN TASK_STATUS TA ON TA.ID = A.TASK_STATUS
                      JOIN PROJECT_TYPE PT ON(PT.ID = P.PROJECT_TYPE $filterTypeProject)
                WHERE A.RESOURCE_T = :resourceId
                AND T.STATUS = :activeTask
                AND TA.CONTROL = :deleteLogic
                AND P.STATUS = :activeProject
                AND T.IS_ISSUE = :notIssue
                AND A.TYPE IS NULL
SQL;

        $params = array(
            'resourceId' => $resourceId,
            'activeProject' => Entities\Project::IS_ACTIVE,
            'activeTask' => Entities\Task::ACTIVE,
            'notIssue' => Entities\Task::NOT_IS_ISSUE,
            'deleteLogic' => LogicalDeleted::STATUS_LOGICAL_DELETED,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

	}

    public function getStartedTaskPriority($priorityId) {

            $query = <<<SQL
                SELECT T.NAME
                FROM TASK T
                WHERE T.PRIORITY = :priorityId
SQL;

        $params = array(
            'priorityId' => $priorityId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

	}

    public function findTaskBranch($taskId) {
        $query = <<<SQL
    		SELECT T.NAME
    		  FROM TASK T
    	INNER JOIN ALLOCATION A ON A.TASK = T.ID
     	INNER JOIN TASK_BRANCHES TB ON TB.TASK = T.ID
    	   	 WHERE T.ID = :ID
    		   AND T.PERCENT_COMPLETED < 100
SQL;

	$params = array(
 			'ID' => $taskId,
 	);

	    return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
 	}

    public function findAllTaskByProject($projectId) {
        $query = <<<SQL
		    SELECT T.ID,T.NAME,T.PARENT
		      FROM TASK T
		     WHERE T.PROJECT = :project
		       AND T.STATUS  = 'A'
		       AND NOT (T.IS_PARENT = 'N' AND T.PERCENT_COMPLETED = 100)
SQL;

    	$params = array(
     			'project' => $projectId,
     	);
	    return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
 	}

	/**
	 * @param $organization
	 * @param $project
	 * @return array
	 */
	public function getTaskByProject($organization, $project) {
		$query = $this->getEntityManager()->createQueryBuilder()->select('u')
			->from('\Model\Task','u')
			->where('u.organization = ?1')
			->setParameter(1,$organization)
			->andWhere('u.project = ?2')
			->setParameter(2,$project);

		return $query->getQuery()->getArrayResult();
	}

    public function validateEntranceComment($issueId,$typeTask){
        $query = <<<SQL
            SELECT TTCT.ID ,
                   TTCT.TASK_TYPE_ID AS TASK_TYPE,
                   TTCT.COMMENT_TYPE_ID AS COMMENT_TYPE,
                   TTCT.IN_OUT_COMMENT ,
                   TTCT.ACTION_LABEL ,
                   TTCT.STATUS ,
                   TTCT.COMMENT_DESCRIPTION AS DESCRIPTION ,
                   TTCT.TASK_TYPE_TARGET,
                   COTY.NAME AS COMMENT_TYPE_NAME
              FROM TASK_TYPE_COMMENT_TYPE TTCT
         LEFT JOIN COMMENT_TYPE COTY ON (COTY.ID = TTCT.COMMENT_TYPE_ID )
         LEFT JOIN TASK_TYPE_COMMENT_TYPE_CLASSIF TTCC ON (TTCC.TASK_TYPE_COMMENT_TYPE_ID = TTCT.ID)
              JOIN TASK ISS ON(ISS.ID = :issueId)
             WHERE TTCT.TASK_TYPE_ID = :taskType
               AND (TTCC.CLASSIFICATION_ID = ISS.CLASSIFICATION OR TTCC.CLASSIFICATION_ID IS NULL)
               AND TTCT.IN_OUT_COMMENT = :inComment

SQL;
        $params = array(
            'taskType' => $typeTask,
            'inComment' => Entities\TaskTypeCommentType::IN_COMMENT,
            'issueId' => $issueId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
}

	/**
	 * @param $userId
	 * @return array
	 */
	public function getTasksByUser($userId) {
		$query = <<<SQL
			SELECT PROJECT.NAME AS PROJECT_NAME, TASK.ID, TASK.PARENT, TASK.NAME,
			       TO_CHAR(TASK.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS BEGIN_TASK,
			       TO_CHAR(TASK.ESTIMATED_END_DATE, 'DD/MM/YYYY')   AS END_TASK
			  FROM RESOURCE_T JOIN RESOURCE_PROJECT ON RESOURCE_T.ID = RESOURCE_PROJECT.RESOURCE_C
		INNER JOIN BUSINESS_PARTNER ON RESOURCE_T.BUSINESS_PARTNER = BUSINESS_PARTNER.ID
						AND BUSINESS_PARTNER.ORGANIZATION = RESOURCE_T.ORGANIZATION
	    INNER JOIN ALLOCATION ON RESOURCE_PROJECT.ID = ALLOCATION.RESOURCE_PROJECT
		INNER JOIN TASK ON ALLOCATION.TASK = TASK.ID
		INNER JOIN PROJECT ON PROJECT.ID = TASK.PROJECT
			 WHERE BUSINESS_PARTNER.ID = :USER_ID
			   AND TASK.STATUS <> 'D'
			   AND TASK.PERCENT_COMPLETED <> 100
SQL;
		$params = array(
			'USER_ID' => $userId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getParentTask($taskId) {
		$query = <<<SQL
			SELECT * FROM task T JOIN TASK TP on(T.ID = TP.PARENT) WHERE T.ID = :TASK_ID
SQL;
		$params = array(
			'TASK_ID' => $taskId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function findScheduleMeetingPlanning($projectId, $filter, $limit, $offset)
    {
        $query = <<<SQL
            SELECT T.ID, T.NAME, T.TASK_STATUS_ID, T.TASK_STATUS, T.PROJECT_NAME, T.COMMITMENT_DATE, T.TASK_ORDER,
                   (SELECT rtrim(xmlagg(xmlelement(e,RES.ID,',').extract('//text()') ORDER BY RES.ID).GetClobVal(),',')
                     FROM ALLOCATION ALO
                     JOIN RESOURCE_T RES ON RES.ID = ALO.RESOURCE_T AND RES.STATUS <> 'D'
                    WHERE ALO.TASK = T.ID ) AS MANAGER
             FROM ( SELECT  *
                      FROM ( SELECT TAS.ID, TAS.NAME, RES.ID AS MANAGER, TST.ID AS TASK_STATUS_ID, TST.LABEL AS TASK_STATUS,
                                    PRO.NAME AS PROJECT_NAME, TAS.COMMITMENT_DATE, TAL.LABEL AS LABEL, TAS.TASK_ORDER
                               FROM TASK TAS
                               JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                               JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
                          LEFT JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID
                          LEFT JOIN RESOURCE_T RES ON RES.ID = ALO.RESOURCE_T
                          LEFT JOIN TASK_LABEL TAL ON TAL.TASK = TAS.ID
                              WHERE PRO.ID = :PROJECT_ID
                                AND TAS.PARENT IS NOT NULL
                                AND TAS.STATUS <> 'D')
                     WHERE 1 = 1
                     $filter) T
                  GROUP BY T.ID, T.NAME, T.TASK_STATUS_ID, T.TASK_STATUS, T.PROJECT_NAME, T.COMMITMENT_DATE, T.TASK_ORDER
                  ORDER BY T.TASK_ORDER ASC NULLS FIRST
SQL;
        $params = array(
            'PROJECT_ID' => $projectId
        );
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        $schedules = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach ($schedules as &$schedule) {
            $schedule['MANAGER'] = Utilities::setSelectMultiInQuery($schedule['MANAGER'], true);
        }
        return $schedules;
    }

    public function findSchedulesIdsByMeeting($projectId)
    {
        $query = <<<SQL
            SELECT TAS.ID
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
             WHERE PRO.ID = :PROJECT_ID
               AND TAS.PARENT IS NOT NULL
               AND TAS.STATUS <> 'D'

SQL;
        $params = array(
            'PROJECT_ID' => $projectId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findTaskSchedulesIdsByMeeting($projectId)
    {
        $query = <<<SQL
            SELECT TAS.ID
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
             WHERE PRO.ID = :PROJECT_ID
               AND TAS.PARENT IS NOT NULL
               AND TAS.STATUS <> 'D'

SQL;
        $params = array(
            'PROJECT_ID' => $projectId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    public function findTaskMeetingPlannig($scheduleFilter, $taskScheduleFilter, $filter, $limit, $offset)
    {
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $query = <<<SQL
                 SELECT ID, TASK_NAME, PROJECT_ID, PROJECT_NAME, MODEL_ID, COMMITMENT_DATE, TASK_STATUS,
                       TASK_STATUS_LABEL, ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE, PERCENT_COMPLETED,
                       TASK_LABEL, TASK_CLASSIFICATION, TASK_CLASSIFICATION_LABEL, RESOURCE_ID
                  FROM (SELECT TAS.ID AS ID, TAS.NAME AS TASK_NAME, PRO.ID AS PROJECT_ID, PRO.NAME AS PROJECT_NAME,
                               PRL.ID AS MODEL_ID, TAS.COMMITMENT_DATE, TAS.TASK_STATUS, TST.LABEL AS TASK_STATUS_LABEL,
                               TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                               TO_CHAR(TAS.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, TAS.PERCENT_COMPLETED,
                               TLB.LABEL AS TASK_LABEL, TAS.CLASSIFICATION AS TASK_CLASSIFICATION,
                               CLA.LABEL AS TASK_CLASSIFICATION_LABEL, TAR.ID AS SCHEDULE, TAR.NAME AS SCHEDULE_NAME, ALO.RESOURCE_T AS RESOURCE_ID
                          FROM TASK TAS
                          JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                          JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS AND TST.IS_TASK = 'Y'
                     LEFT JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID
                     LEFT JOIN PROJECT_RELATIONSHIP PRL ON (PRO.ID = PRL.PROJECT AND PRL.TYPE = 'G')
                     LEFT JOIN TASK_RELATIONSHIP TRL ON (TRL.TASK = TAS.ID AND TRL.TYPE = 'W')
                     LEFT JOIN TASK TAR ON TAR.ID = TRL.TASK_RELATIONSHIP
                     LEFT JOIN TASK_LABEL TLB ON TLB.TASK = TAS.ID
                     LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
                          JOIN CLASSIFICATION CLA ON CLA.ID = TAS.CLASSIFICATION
                         WHERE (TAS.IS_ISSUE = 'N'
                         AND TAS.STATUS = 'A'
                         $scheduleFilter)
                         $taskScheduleFilter
                         $filterTypeProject)
                 WHERE 1 = 1
                 $filter
              GROUP BY ID, TASK_NAME, PROJECT_ID, PROJECT_NAME, MODEL_ID, COMMITMENT_DATE, TASK_STATUS,
                       TASK_STATUS_LABEL, ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE, PERCENT_COMPLETED,
                       TASK_LABEL, TASK_CLASSIFICATION, TASK_CLASSIFICATION_LABEL, RESOURCE_ID
              ORDER BY TASK_NAME
SQL;
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $tasks;
    }

    public function findIssueMeetingPlannig($scheduleFilter, $issueScheduleFilter, $filter, $limit, $offset)
    {
        $query = <<<SQL
            SELECT ID, NAME, FRAMING, PRODUCT_ID, PRODUCT, SPONSOR, CLASSIFICATION, CLASSIFICATION_LABEL, TASK_STATUS_ID, TASK_STATUS,
                   TASK_STATUS_LABEL, PRIORITY, DUE_DATE, REPORTER_BY_ID, REPORTER_BY, CREATED_AT, UPDATED_AT
              FROM (SELECT TAS.ID AS ID, TAS.NAME AS NAME, TTY.LABEL AS FRAMING, PRO.ID AS PRODUCT_ID, PRO.NAME AS PRODUCT,
                           RES.NAME AS SPONSOR, TAS.CLASSIFICATION AS CLASSIFICATION, CLA.LABEL AS CLASSIFICATION_LABEL,
                           TST.ID AS TASK_STATUS_ID, TST.CONTROL AS TASK_STATUS, TST.LABEL AS TASK_STATUS_LABEL, PRI.LABEL AS PRIORITY,
                           TO_CHAR(TAS.END_DATE, 'DD/MM/YYYY') AS DUE_DATE, TLB.LABEL AS LABEL, RSS.ID AS REPORTER_BY_ID,
                           RSS.NAME AS REPORTER_BY, RST.ID AS FOLLOWER_ID, RST.NAME AS FOLLOWERS, TAS.CREATED_AT, TAS.UPDATED_AT
                     FROM TASK TAS
                     JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                     JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
                     JOIN TASK_TYPE TTY ON TTY.ID = TST.TASK_TYPE
                LEFT JOIN TASK_LABEL TLB ON TLB.TASK = TAS.ID
                LEFT JOIN ALLOCATION ALO ON (ALO.TASK = TAS.ID AND ALO.TYPE = 'S')
                LEFT JOIN RESOURCE_T RES ON RES.ID = ALO.RESOURCE_T
                LEFT JOIN ALLOCATION ALC ON (ALC.TASK = TAS.ID AND ALC.TYPE = 'R')
                LEFT JOIN RESOURCE_T RSS ON RSS.ID = ALC.RESOURCE_T
                LEFT JOIN ALLOCATION ALT ON (ALT.TASK = TAS.ID AND ALC.TYPE = 'F')
                LEFT JOIN RESOURCE_T RST ON RST.ID = ALC.RESOURCE_T
                     JOIN CLASSIFICATION CLA ON CLA.ID = TAS.CLASSIFICATION
                LEFT JOIN PRIORITY PRI ON PRI.ID = TAS.PRIORITY
                    WHERE (TAS.IS_ISSUE = 'Y'
                      AND TAS.STATUS = 'A'
                        $scheduleFilter)
                        $issueScheduleFilter)
              WHERE 1 = 1
              $filter
              GROUP BY ID, NAME, FRAMING, PRODUCT_ID, PRODUCT, SPONSOR, CLASSIFICATION, CLASSIFICATION_LABEL, TASK_STATUS_ID, TASK_STATUS,
                       TASK_STATUS_LABEL, PRIORITY, DUE_DATE, REPORTER_BY_ID, REPORTER_BY, CREATED_AT, UPDATED_AT
              ORDER BY NAME
SQL;

        $params = array(
            'taskWithScheduleType' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE,
            'isIssue' => Entities\Task::IS_ISSUE
        );
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    /************************************************************
     * Gets all task status.
     * @param int $organizationId
     * @param string $allowEndTasks
     * @return array
     ***********************************************************/
	public function getAllStatusTask($organizationId, $allowEndTasks = 'Y', $filterString = '') {
		$query = <<<SQL
        SELECT 	ID , LABEL,CONTROL,
            (CASE CONTROL WHEN 'N' THEN 1
            WHEN 'D' THEN 2
            WHEN 'P' THEN 3
            WHEN 'C' THEN 4
            END) ORDER_BY
		FROM 	TASK_STATUS
		WHERE 	ORGANIZATION = :ORGANIZATION_ID
		AND 	USED_IN_TASK = 'Y'
		AND 	IS_PROJECT = 'N'
		$filterString
SQL;

		# If the project does not allow ended tasks, we don't want to get concluded status in the dataSet
        if($allowEndTasks == self::ALLOWENDTASK_NEGATIVE){
            $query .= "AND CONTROL <> '".self::CONTROL_TASK_STATUS_CONCLUDED."'";
        }
        $query .=
            <<<SQL
        ORDER BY ORDER_BY, LABEL
SQL;
		$params = array(
			'ORGANIZATION_ID' => $organizationId,
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getAllTypeTask($organizationId, $conditions = "") {
		$query = <<<SQL
			SELECT ID, LABEL AS NAME, ORDER_TO_FRAME, ALLOW_CLOSE_ISSUE, CONTROL, ORDER_TO_VIEW FROM TASK_TYPE
			WHERE ORGANIZATION = :ORGANIZATION_ID
			      $conditions
			ORDER BY ORDER_TO_VIEW
SQL;
		$params = array(
			'ORGANIZATION_ID' => $organizationId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getNextOrder($projectId) {
		$query = <<<SQL
			SELECT MAX(TASK_ORDER)+1 AS NEXTORDER FROM TASK WHERE PROJECT = :PROJECT_ID
SQL;
		$params = array(
			'PROJECT_ID' => $projectId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getParentTaskByProjectAndTask($organizationId, $projectId, $taskId, $selectedTasksQuery,$isParent = 'N', $filterstring = '') {


        $filterIsParent = '';
        if($isParent == 'Y'){
            $filterIsParent = "AND T.IS_PARENT = 'Y'";
        }
        $params = array(
            'organization' => $organizationId,
            'project' => $projectId,
            'task' => $taskId,
            'type' => Entities\TaskRelationship::RELATED_TASK_TYPE,
        );
        $query = <<<SQL
		 SELECT NVL(TA1.ID ,TA2.ID )AS ID
           FROM TASK_RELATIONSHIP TAS
      LEFT JOIN TASK TA1 ON(TA1.ID = TAS.TASK  AND TA1.PROJECT = :project)
      LEFT JOIN TASK TA2 ON(TA2.ID = TAS.TASK_RELATIONSHIP AND TA2.PROJECT = :project )
          WHERE TAS.TYPE = :type
            AND (TA1.ID IS NOT NULL OR TA2.ID IS NOT NULL)
SQL;
        $taskRelated = $this->getEntityManager()->getConnection()->fetchAll($query,$params);
        $filterTaskRelated = '';
        if(!empty($taskRelated)){
            $taskRelated = Utilities::getArrayFromArrayAssociative($taskRelated);
            $taskRelated = Utilities::getStringForArray($taskRelated);
            $filterTaskRelated = "AND T.ID NOT IN ($taskRelated)";
        }

		$query = <<<SQL
		 	SELECT *
		 	  FROM (SELECT T.ID,
        		           (CASE WHEN (PR.ID IS NOT NULL AND T.PARENT IS NULL) THEN 'Pauta' ELSE T.NAME END) AS NAME,
        		           T.PARENT, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS BEGIN_TASK, T.DURATION,
        		           TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS END_T, T.IS_PARENT,
        		           T.BEGIN_DATE AS BEGIN_DATE_TASK, T.END_DATE AS END_DATE_TASK
			          FROM TASK T
			     LEFT JOIN (SELECT PRO.ID
                              FROM PROJECT PRO
                              JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
                             WHERE PRO.ID = :project
                               AND PT.EXTERNALTYPE IN ('M','O')
                           ) PR ON (PR.ID = T.PROJECT)
                 LEFT JOIN TASK_STATUS TST ON (T.TASK_STATUS = TST.ID AND TST.CONTROL = 'B')
			         WHERE T.ORGANIZATION = :organization
			           AND T.PROJECT = :project
			           AND T.ID <> :task
                       AND TST.CONTROL IS NULL
			           $filterIsParent
			           AND T.STATUS <> 'D'
			           AND   NOT (T.IS_PARENT = 'N' AND T.PERCENT_COMPLETED = 100)
			           $filterTaskRelated
			           AND NVL(T.IS_ISSUE,'N') <> 'Y'
SQL;
		$params = array(
			'organization' => $organizationId,
			'project' => $projectId,
			'task' => $taskId,
			'type' => Entities\TaskRelationship::RELATED_TASK_TYPE,
		);
        $selectedTasksQuery = "AND T.ID " . $selectedTasksQuery;
		return $this->getEntityManager()->getConnection()->fetchAll($query.$selectedTasksQuery.") T WHERE 1 = 1 ".$filterstring."
			ORDER BY T.NAME,T.BEGIN_DATE_TASK DESC, T.END_DATE_TASK DESC", $params);
	}

    public function validateAddTaskClassification($taskTypeId) {
        $query = <<<SQL
    		  SELECT TAS.ID
                FROM TASK_TYPE TAT
                JOIN TASK_STATUS TAS ON (TAS.TASK_TYPE  = TAT.ID)
               WHERE TAT.ID = :ID
                 AND TAS.ID IS NOT NULL
SQL;

        $params = array(
            'ID' => $taskTypeId,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

	public function getPrecursorTaskByProjectAndTask($organizationId, $projectId, $taskId) {
		 $query = <<<SQL
		 	SELECT 	T.ID, T.NAME, T.PARENT, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS BEGIN_TASK, T.PREVIOUS_TIME/3600 as PREVIOUS_TIME,
		 			TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS END_TASK, T.DURATION,
		 			(SELECT  LISTAGG(RES.NAME, ', ') WITHIN GROUP (ORDER BY RES.NAME ASC) AS BUSINESS_PARTNER_NAME FROM  RESOURCE_T RES
						INNER JOIN ALLOCATION ALO ON (RES.ID = ALO.RESOURCE_T)
						WHERE ALO.TASK = T.ID) AS RESOURCES
	 	  	FROM TASK T
	   LEFT JOIN TASK_STATUS TST ON (T.TASK_STATUS = TST.ID AND TST.CONTROL = 'B')
			WHERE 	T.ORGANIZATION = :organization
			AND 	T.PROJECT = :project
			AND 	T.ID <> :task
			AND 	T.STATUS <> 'D'
			AND     TST.CONTROL IS NULL
			AND	 NVL(T.IS_ISSUE,'N') <> 'Y'
			ORDER BY T.NAME
SQL;
		$params = array(
			'organization' => $organizationId,
			'project' => $projectId,
			'task' => $taskId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getDateOfUserTasksFromToday($userId) {
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$query = <<<SQL
			SELECT TASK.ESTIMATED_BEGIN_DATE
			  FROM (
                    SELECT TO_CHAR(TASK.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                    TO_CHAR(TASK.ESTIMATED_BEGIN_DATE,'YYYYMMDD') AS ORDER_BY
                    FROM TASK
                    INNER JOIN ALLOCATION ON ALLOCATION.TASK = TASK.ID
                    INNER JOIN RESOURCE_T ON ALLOCATION.RESOURCE_T = RESOURCE_T.ID
                    LEFT JOIN 	PROJECT PRO           ON PRO.ID = TASK.PROJECT
                    LEFT JOIN   PROJECT_TYPE PT       ON (PT.ID = PRO.PROJECT_TYPE $filterTypeProject)
                    WHERE RESOURCE_T.ID = :USER_ID
                      AND TASK.ESTIMATED_BEGIN_DATE IS NOT NULL
                      AND TASK.ESTIMATED_BEGIN_DATE >= TO_DATE(:TODAY || '00:00', 'DD/MM/YYYY HH24:MI')
                      AND TASK.STATUS = 'A'
                      AND NVL(TASK.IS_ISSUE,'N') <> 'Y')TASK
			GROUP BY TASK.ESTIMATED_BEGIN_DATE,TASK.ORDER_BY
			ORDER BY TASK.ORDER_BY
SQL;
		$today = new \DateTime();
		$todayString =  $today->format('d/m/Y');
		$params = array(
			':USER_ID' => $userId,
			':TODAY' => $today->format('d/m/Y')
		);
		$result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        $hasToday = false;
        foreach ($result as $taskDate) {
            if($taskDate['ESTIMATED_BEGIN_DATE'] == $todayString){
                $hasToday = true;
            }
        }
        if(!$hasToday){
			$dateToday = array('ESTIMATED_BEGIN_DATE' => $todayString);
            array_unshift($result,$dateToday);
		}
		return $result;
	}

    /**
     * Gets tasks of an user when he clicks on '+' action,
     * on appropriation's screen.
     * @param int $userId
     * @param int $projectId
     * @return array
     */
	public function getDelayedTasksByUserOnDemand($userId,$projectId) {
	    
		$query = <<<SQL
		SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TSA.CONTROL AS TASK_STATUS, TSA.LABEL AS TASK_STATUS_LABEL, T.DESCRIPTION,
                        TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                        T.TIME_PLANNED AS DURATION, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE, T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
                        P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.ID AS TASK_ID,
                        PT.USETIMEPLANNING AS USE_TIME_PLANNING, TSP.TASK_CAN_START AS TASK_CAN_START, TP.NAME AS PARENT_NAME, PT.ALLOWENDTASK,
                        TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP,CLA.LABEL AS CLASSIFICATION_NAME,
                        CASE WHEN (SELECT TASK.ID
                                  FROM TASK_TYPE TAST
                            RIGHT JOIN TASK_STATUS TSTT ON TSTT.TASK_TYPE = TAST.ID
                            RIGHT JOIN TASK ON TASK.TASK_STATUS = TSTT.ID
                            INNER JOIN PROJECT PROJ ON PROJ.ID = TASK.PROJECT
                             LEFT JOIN PROJECT_TYPE PROT ON PROT.ID = PROJ.PROJECT_TYPE
                            INNER JOIN TASK_STATUS TSTP ON TSTP.ID = PROJ.PROJECT_STATUS
                             LEFT JOIN CLASSIFICATION CLAS ON CLAS.ID = TASK.CLASSIFICATION
                                 WHERE TASK.ID = T.ID
                                   AND (TASK.IS_PARENT = 'Y'
                                    OR (PROT.USETIMEPLANNING = 'Y' AND TASK.TIME_PLANNED = 0)
                                    OR (TSTP.CONTROL = 'O' AND CLAS.CONTROL IS NULL)
                             OR EXISTS (SELECT TTAS.ID
                                          FROM TASK_RELATIONSHIP TREL
                                     LEFT JOIN TASK TTAS ON TTAS.ID = TREL.TASK_RELATIONSHIP
                                          JOIN TASK TASQ ON TASQ.ID = TREL.TASK
                                         WHERE (TREL.TYPE IN ('C', 'G', 'E') AND TREL.TASK = TASK.ID AND TTAS.PERCENT_COMPLETED < 100)
                                            OR (TREL.TASK_RELATIONSHIP = TASK.ID AND TREL.TYPE = 'M' AND TASQ.PERCENT_COMPLETED < 100)
                                       )
                             OR EXISTS (SELECT TAT.ID
                                          FROM TASK TAS
                                          JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                          JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
                                          JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                          JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                          JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                          JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
                                          JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                          JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
                                         WHERE TAS.ID = TASK.ID
                                           AND ISS.STATUS = 'A'
                                           AND TAT.STATUS = 'A'
                                           AND TAS.STATUS = 'A'
                                           AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                           AND TAT.PERCENT_COMPLETED < 100
                                           AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
                                           AND TAS.CLASSIFICATION NOT IN (SELECT TTC.CLASSIFICATION_ID
                                                                            FROM TASK ISS
                                                                            JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                                            JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                                           WHERE ISS.ID = TAR.TASK))
                                                                            )) IS NULL THEN 'N'
                                                                                       ELSE 'Y'
                                                                                     END AS IS_ALLOW_PLAY, PT.USEGITAUTOMATICALLY
                   FROM TASK_TYPE           TT
             RIGHT JOIN TASK_STATUS         TS  ON TT.ID             = TS.TASK_TYPE
             RIGHT JOIN TASK                T   ON TS.ID             = T.TASK_STATUS
             INNER JOIN ALLOCATION          A   ON A.TASK            = T.ID
             INNER JOIN RESOURCE_T          RT  ON A.RESOURCE_T      = RT.ID
             INNER JOIN PROJECT             P   ON T.PROJECT         = P.ID
              LEFT JOIN	PROJECT_TYPE        PT  ON PT.ID             = P.PROJECT_TYPE
              LEFT JOIN TASK_STATUS         TSP ON (P.PROJECT_STATUS = TSP.ID)
              LEFT JOIN TASK_STATUS         TSA ON (TSA.ID           = A.TASK_STATUS)
              LEFT JOIN TASK                TP  ON (T.PARENT         = TP.ID)
              LEFT JOIN TASK_RELATIONSHIP   TR  ON (T.ID             = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
              LEFT JOIN TASK                TTT ON (TTT.ID           = TR.TASK)
              LEFT JOIN CLASSIFICATION      CLA ON (CLA.ID           = T.CLASSIFICATION)
             INNER JOIN	TASK_STATUS         TSP ON P.PROJECT_STATUS  = TSP.ID
                  WHERE RT.ID = :USER_ID
                    AND P.ID = :PROJECT_ID
                    AND TO_DATE(:TASK_DATE, 'DD/MM/YYYY') > T.ESTIMATED_BEGIN_DATE
                    AND T.PERCENT_COMPLETED < 100
                    AND P.STATUS = 'A'
                    AND T.STATUS = 'A'
                    AND T.IS_ISSUE = 'N'
                    AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
               ORDER BY T.PARENT ASC NULLS FIRST
SQL;
		$today = new \DateTime();
		$params = array(
			':USER_ID' => $userId,
			':PROJECT_ID' => $projectId,
			':TASK_DATE' => $today->modify('+1 day')->format('d/m/Y')
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
			if($task['USE_TIME_PLANNING'] == 'N'){
				$task['ESTIMATED_BEGIN_DATE'] = '';
				$task['ESTIMATED_END_DATE'] = '';
			}
			if($task['ALLOWENDTASK'] == 'N'){
				$task['PERCENT_COMPLETED'] = '';
			}
        }

		return $tasks;
	}
	
	public function getUnatendedDelayedTasksByCurrentUserOnDemand($projectId) {
	    
		$query = <<<SQL
		SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TSA.CONTROL AS TASK_STATUS, TSA.LABEL AS TASK_STATUS_LABEL, T.DESCRIPTION,
                        TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                        T.TIME_PLANNED AS DURATION, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE, T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
                        P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.ID AS TASK_ID,
                        PT.USETIMEPLANNING AS USE_TIME_PLANNING, TSP.TASK_CAN_START AS TASK_CAN_START, TP.NAME AS PARENT_NAME, PT.ALLOWENDTASK,
                        TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP,CLA.LABEL AS CLASSIFICATION_NAME,
                        CASE WHEN (SELECT TASK.ID
                                  FROM TASK_TYPE TAST
                            RIGHT JOIN TASK_STATUS TSTT ON TSTT.TASK_TYPE = TAST.ID
                            RIGHT JOIN TASK ON TASK.TASK_STATUS = TSTT.ID
                            INNER JOIN PROJECT PROJ ON PROJ.ID = TASK.PROJECT
                             LEFT JOIN PROJECT_TYPE PROT ON PROT.ID = PROJ.PROJECT_TYPE
                            INNER JOIN TASK_STATUS TSTP ON TSTP.ID = PROJ.PROJECT_STATUS
                             LEFT JOIN CLASSIFICATION CLAS ON CLAS.ID = TASK.CLASSIFICATION
                                 WHERE TASK.ID = T.ID
                                   AND (TASK.IS_PARENT = 'Y'
                                    OR (PROT.USETIMEPLANNING = 'Y' AND TASK.TIME_PLANNED = 0)
                                    OR (TSTP.CONTROL = 'O' AND CLAS.CONTROL IS NULL)
                             OR EXISTS (SELECT TTAS.ID
                                          FROM TASK_RELATIONSHIP TREL
                                     LEFT JOIN TASK TTAS ON TTAS.ID = TREL.TASK_RELATIONSHIP
                                          JOIN TASK TASQ ON TASQ.ID = TREL.TASK
                                         WHERE (TREL.TYPE IN ('C', 'G', 'E') AND TREL.TASK = TASK.ID AND TTAS.PERCENT_COMPLETED < 100)
                                            OR (TREL.TASK_RELATIONSHIP = TASK.ID AND TREL.TYPE = 'M' AND TASQ.PERCENT_COMPLETED < 100)
                                       )
                             OR EXISTS (SELECT TAT.ID
                                          FROM TASK TAS
                                          JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                          JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
                                          JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                          JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                          JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                          JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
                                          JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                          JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
                                         WHERE TAS.ID = TASK.ID
                                           AND ISS.STATUS = 'A'
                                           AND TAT.STATUS = 'A'
                                           AND TAS.STATUS = 'A'
                                           AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                           AND TAT.PERCENT_COMPLETED < 100
                                           AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
                                           AND TAS.CLASSIFICATION NOT IN (SELECT TTC.CLASSIFICATION_ID
                                                                            FROM TASK ISS
                                                                            JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                                            JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                                           WHERE ISS.ID = TAR.TASK))
                                                                            )) IS NULL THEN 'N'
                                                                                       ELSE 'Y'
                                                                                     END AS IS_ALLOW_PLAY, PT.USEGITAUTOMATICALLY
                   FROM TASK_TYPE           TT
             RIGHT JOIN TASK_STATUS         TS  ON TT.ID             = TS.TASK_TYPE
             RIGHT JOIN TASK                T   ON TS.ID             = T.TASK_STATUS
             LEFT JOIN ALLOCATION          A   ON A.TASK            = T.ID
             LEFT JOIN RESOURCE_T          RT  ON A.RESOURCE_T      = RT.ID
             INNER JOIN PROJECT             P   ON T.PROJECT         = P.ID
              LEFT JOIN	PROJECT_TYPE        PT  ON PT.ID             = P.PROJECT_TYPE
              LEFT JOIN TASK_STATUS         TSP ON (P.PROJECT_STATUS = TSP.ID)
              LEFT JOIN TASK_STATUS         TSA ON (TSA.ID           = A.TASK_STATUS)
              LEFT JOIN TASK                TP  ON (T.PARENT         = TP.ID)
              LEFT JOIN TASK_RELATIONSHIP   TR  ON (T.ID             = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
              LEFT JOIN TASK                TTT ON (TTT.ID           = TR.TASK)
              LEFT JOIN CLASSIFICATION      CLA ON (CLA.ID           = T.CLASSIFICATION)
             INNER JOIN	TASK_STATUS         TSP ON P.PROJECT_STATUS  = TSP.ID
                  WHERE RT.ID IS NULL
                    AND P.ID = :PROJECT_ID
                    AND TO_DATE(:TASK_DATE, 'DD/MM/YYYY') > T.ESTIMATED_BEGIN_DATE
                    AND T.PERCENT_COMPLETED < 100
                    AND P.STATUS = 'A'
                    AND T.STATUS = 'A'
                    AND T.IS_ISSUE = 'N'
                    AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
               ORDER BY T.PARENT ASC NULLS FIRST
SQL;
		$today = new \DateTime();
		$params = array(
			':PROJECT_ID' => $projectId,
			':TASK_DATE' => $today->modify('+1 day')->format('d/m/Y')
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
			if($task['USE_TIME_PLANNING'] == 'N'){
				$task['ESTIMATED_BEGIN_DATE'] = '';
				$task['ESTIMATED_END_DATE'] = '';
			}
			if($task['ALLOWENDTASK'] == 'N'){
				$task['PERCENT_COMPLETED'] = '';
			}
        }

		return $tasks;
	}

    public function findScheduleForm($taskId,$meetingId){

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT');
        $typeLabelIssue = Entities\TaskLabel::TYPE_ISSUE;
        $typeLabelTask = Entities\TaskLabel::TYPE_TASK;
        $taskWithScheduleType = Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE;

        $typeReporter  = Entities\Allocation::TYPE_REPORTER;
        $typeFollower  = Entities\Allocation::TYPE_FOLLOWER;
        $typeSponsor   = Entities\Allocation::TYPE_SPONSOR;

        $query = <<<SQL
			SELECT (SELECT LISTAGG(TL.LABEL, ',') WITHIN GROUP (ORDER BY TL.LABEL)
					 FROM TASK_LABEL TL
					 WHERE TL.TASK = TAS.TASK_ID ) as LABELS,
					 (SELECT LISTAGG(ALO.RESOURCE_T, ',') WITHIN GROUP (ORDER BY ALO.RESOURCE_T)
					 FROM ALLOCATION ALO
					 WHERE ALO.TASK = TAS.TASK_ID
					 AND AlO.TYPE = :TYPE_FOLLOWER ) as FOLLOWERS,
					 TAS.TYPE_LABEL AS TYPE_LABEL_VIEW,
					 TAS.*
					 FROM (SELECT  TAS.ID AS TASK_ID, TAS.PRIORITY, TAS.PROJECT AS PROJECT_ID, TAS.NAME ,TAS.DESCRIPTION,
                            TAS.CLASSIFICATION,CLA.LABEL AS CLASSIFICATION_LABEL,TAS.PARENT, TO_CHAR(TAS.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
                             TO_CHAR(TAS.COMMITMENT_DATE, 'DD/MM/YYYY') AS COMMITMENT_DATE,CRY.NAME AS CREATED_BY,
                            (CASE WHEN PT.ID IS NULL THEN TPA.NAME ELSE 'Pauta' END) AS PARENT_NAME,
                            LISTAGG(RES.ID, ',') WITHIN GROUP (ORDER BY RES.ID) AS RESOURCES,TAS.TASK_STATUS,
                            TST.LABEL AS TASK_STATUS_NAME,(CASE WHEN PT.ID IS NULL THEN 'N' ELSE 'Y' END) AS IS_SCHEDULE,
                            (CASE WHEN TAS.IS_ISSUE = 'Y' THEN '$typeLabelIssue' WHEN PT.ID IS NOT NULL THEN 'P' ELSE '$typeLabelTask' END) AS TYPE_LABEL,
                            (SELECT  RTRIM(XMLAGG(XMLELEMENT(E,TRE.ID,',').EXTRACT('//text()') ORDER BY TRE.ID).GETCLOBVAL(),',') AS SCHEDULE_RELATIONSHIP
                                 FROM  TASK_RELATIONSHIP TAR
                                 JOIN  TASK TRE   ON (TRE.ID = TAR.TASK_RELATIONSHIP AND TRE.PROJECT =  :MEETING_ID)
                                WHERE  TAR.TYPE = :TASK_WITH_SCHEDULE_TYPE AND TAR.TASK = :TASK_ID ) AS SCHEDULE_RELATIONSHIP,
                             RER.NAME AS REPORTER_NAME,
                             REO.NAME AS SPONSOR_NAME,PRO.NAME AS PROJECT_NAME,PRO.NAME AS PRODUCT_NAME, PRI.LABEL AS PRIORITY_NAME
                     FROM   TASK TAS
                LEFT JOIN 	ALLOCATION ALO		  ON ALO.TASK = TAS.ID
                LEFT JOIN 	RESOURCE_T RES 		  ON RES.ID = ALO.RESOURCE_T
                LEFT JOIN 	ALLOCATION ALR		  ON (ALR.TASK = TAS.ID AND ALR.TYPE = :TYPE_REPORTER )
                LEFT JOIN 	RESOURCE_T RER 		  ON (RER.ID = ALR.RESOURCE_T)
                LEFT JOIN 	ALLOCATION ALS		  ON (ALS.TASK = TAS.ID AND ALS.TYPE = :TYPE_SPONSOR)
                LEFT JOIN 	RESOURCE_T REO 		  ON (REO.ID = ALS.RESOURCE_T)
                LEFT JOIN 	RESOURCE_T CRY 		  ON (CRY.ID = TAS.CREATED_BY)
                LEFT JOIN 	TASK TPA 			  ON TPA.ID = TAS.PARENT
                LEFT JOIN 	CLASSIFICATION CLA    ON CLA.ID = TAS.CLASSIFICATION
                LEFT JOIN 	TASK_STATUS TST       ON TST.ID = TAS.TASK_STATUS
                LEFT JOIN 	PROJECT PRO           ON PRO.ID = TAS.PROJECT
                LEFT JOIN   PROJECT_TYPE PT       ON (PT.ID = PRO.PROJECT_TYPE $filterTypeProject)
                LEFT JOIN   PRIORITY PRI          ON TAS.PRIORITY = PRI.ID
                    WHERE   TAS.ID = :TASK_ID
                 GROUP BY   TAS.ID, TAS.PRIORITY, TAS.NAME, TAS.PROJECT, TAS.DESCRIPTION, TAS.CLASSIFICATION,PRO.NAME,
                            TAS.PARENT, CLA.LABEL, TPA.NAME, TAS.TASK_STATUS, TST.LABEL,PT.ID,TAS.IS_ISSUE,
                            RER.NAME, REO.NAME, TAS.COMMITMENT_DATE,TAS.CREATED_AT,CRY.NAME, PRI.LABEL )TAS
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'MEETING_ID' => $meetingId,
            'TYPE_REPORTER' => $typeReporter,
            'TYPE_SPONSOR' => $typeSponsor,
            'TYPE_FOLLOWER' => $typeFollower,
            'TASK_WITH_SCHEDULE_TYPE' => $taskWithScheduleType,
        );
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach ($tasks as &$task) {
            $task['SCHEDULE_RELATIONSHIP'] = explode(',', $task['SCHEDULE_RELATIONSHIP']);
            $task['RESOURCES'] = Utilities::setSelectMultiInQuery($task['RESOURCES'],false,',',true);
            $task['FOLLOWERS'] = Utilities::setSelectMultiInQuery($task['FOLLOWERS'],true);
        }
        return $tasks;
    }

    public function findTaskListSchedule($taskId, $conditions = ""){

        $query = <<<SQL
			SELECT  *
			  FROM (SELECT  TRE.ID, TRE.NAME,TAS.ID AS SCHEDULE_ID,TAR.TASK_ORDER, TST.ID AS TASK_STATUS_ID,
			        TST.LABEL AS TASK_STATUS,TO_CHAR(TRE.COMMITMENT_DATE, 'DD/MM/YYYY') AS COMMITMENT_DATE,
                    RTRIM(XMLAGG(XMLELEMENT(E,RES.NAME,',').EXTRACT('//text()') ORDER BY RES.NAME).GETCLOBVAL(),',') AS RESOURCES
                      FROM  TASK TAS
                      JOIN  TASK_RELATIONSHIP TAR ON ( TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = :taskWithScheduleType)
                      JOIN  TASK TRE              ON (TAR.TASK = TRE.ID)
                 LEFT JOIN 	TASK_STATUS TST       ON TST.ID = TRE.TASK_STATUS
                 LEFT JOIN 	ALLOCATION ALO        ON ALO.TASK = TRE.ID
                 LEFT JOIN 	RESOURCE_T  RES       ON RES.ID = ALO.RESOURCE_T
                     WHERE  TRE.IS_ISSUE = :notIsIssue
                       AND  TAS.ID = :TASK_ID
                       AND  TAS.STATUS = :isActive
                 GROUP BY   TRE.ID, TRE.NAME,TST.LABEL,TRE.COMMITMENT_DATE,TAS.ID,TAR.TASK_ORDER, TST.ID
                 ORDER BY TASK_ORDER ASC NULLS FIRST)TAS
           WHERE 1=1
                 $conditions
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'taskWithScheduleType' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE,
            'isActive' => Entities\Project::IS_ACTIVE,
            'notIsIssue' => Entities\Task::NOT_IS_ISSUE,
        );
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach($tasks as &$task){
            $task['TASK_ORDER'] = intval($task['TASK_ORDER']);
        }
        return $tasks;
    }

    public function findIssueListSchedule($taskId, $conditions = "", $limit, $offset){
        $query = <<<SQL
		  SELECT *
	        FROM (SELECT TRE.ID, TRE.NAME,TAS.ID AS SCHEDULE_ID,TAR.TASK_ORDER, TST.ID AS TASK_STATUS_ID,
                	     TST.LABEL AS TASK_STATUS,TO_CHAR(TRE.COMMITMENT_DATE, 'DD/MM/YYYY') AS COMMITMENT_DATE,
                	     PRO.NAME AS PROJECT_NAME
                    FROM TASK TAS
                    JOIN TASK_RELATIONSHIP TAR ON ( TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = :taskWithScheduleType)
                    JOIN TASK TRE              ON (TAR.TASK = TRE.ID)
               LEFT JOIN TASK_STATUS TST       ON TST.ID = TRE.TASK_STATUS
               LEFT JOIN PROJECT PRO           ON PRO.ID = TRE.PROJECT
                   WHERE TRE.IS_ISSUE = :isIssue
                     AND TAS.ID = :TASK_ID
                     AND TAS.STATUS = :isActive
                GROUP BY TRE.ID, TRE.NAME,TST.LABEL,TRE.COMMITMENT_DATE,TAS.ID,TAR.TASK_ORDER,PRO.NAME, TST.ID
                ORDER BY TASK_ORDER ASC NULLS FIRST)TAS
          WHERE 1=1
                $conditions
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'taskWithScheduleType' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE,
            'isActive' => Entities\Project::IS_ACTIVE,
            'isIssue' => Entities\Task::IS_ISSUE,
        );
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach($tasks as &$task){
            $task['TASK_ORDER'] = intval($task['TASK_ORDER']);
        }
        return $tasks;
    }

   public function findIssueListScheduleFromProject($issueProjectIds, $taskId, $conditions = "", $limit, $offset,$meetingProjectId){

       $projectFilter = empty($issueProjectIds)? ' AND PRO.ID IN (     SELECT MP.PROJECT_ID
                                                                     FROM PROJECT P
                                                                     JOIN MEETING M ON M.PROJECT_ID = P.ID
                                                                     JOIN MEETING_PROJECT MP ON MP.MEETING_ID = M.ID
                                                                WHERE P.ID = :MEETING_PROJECT_ID)'
           : ' AND PRO.ID IN (:ISSUE_PROJECT_IDS)';

        $query = <<<SQL
		    SELECT * FROM (   SELECT ISS.ID, ISS.NAME, PRO.NAME AS PROJECT_NAME, TST.LABEL AS TASK_STATUS_NAME
                                FROM TASK ISS
                                JOIN PROJECT PRO ON (PRO.ID = ISS.PROJECT)
                                JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                               WHERE ISS.STATUS = :isActive
                                 $projectFilter
                                 AND ISS.ID NOT IN (  SELECT TAS.ID
                                                        FROM (SELECT TRE.ID, TRE.NAME,TAS.ID AS SCHEDULE_ID,TAR.TASK_ORDER, TST.ID AS TASK_STATUS_ID,
                                                                     TST.LABEL AS TASK_STATUS,TO_CHAR(TRE.COMMITMENT_DATE, 'DD/MM/YYYY') AS COMMITMENT_DATE,
                                                                     PRO.NAME AS PROJECT_NAME
                                                                FROM TASK TAS
                                                                JOIN TASK_RELATIONSHIP TAR ON ( TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = :taskWithScheduleType)
                                                                JOIN TASK TRE              ON (TAR.TASK = TRE.ID)
                                                           LEFT JOIN TASK_STATUS TST       ON TST.ID = TRE.TASK_STATUS
                                                           LEFT JOIN PROJECT PRO           ON PRO.ID = TRE.PROJECT
                                                               WHERE TRE.IS_ISSUE = :isIssue
                                                                 AND TAS.ID = :TASK_ID
                                                                 AND TAS.STATUS = :isActive
                                                            GROUP BY TRE.ID, TRE.NAME,TST.LABEL,TRE.COMMITMENT_DATE,TAS.ID,TAR.TASK_ORDER,PRO.NAME, TST.ID
                                                            ORDER BY TASK_ORDER ASC NULLS FIRST)TAS
                                                      )
                           )
             WHERE 1 = 1
            $conditions
SQL;
       $params = array(
            'TASK_ID' => $taskId,
            'MEETING_PROJECT_ID' => $meetingProjectId,
            'ISSUE_PROJECT_IDS' => $issueProjectIds,
            'isIssue' => Entities\Task::IS_ISSUE,
            'taskWithScheduleType' => Entities\TaskRelationship::TASK_WITH_SCHEDULE_TYPE,
            'isActive' => Entities\Task::ACTIVE
        );
       $paramTypes = array(
            'ISSUE_PROJECT_IDS' => Connection::PARAM_STR_ARRAY
        );
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        $issues = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        return $issues;
    }

    public function findScheduleByMeeting($projectId){

        $query = <<<SQL
		   SELECT TAS.*
             FROM (SELECT  TAS.ID, TAS.NAME
                     FROM   TASK TAS
                    WHERE   TAS.PROJECT = :PROJECT_ID
                      AND TAS.STATUS = 'A'
                      AND TAS.PARENT IS NOT NULL)TAS
         ORDER BY TAS.NAME
SQL;
        $params = array(
            'PROJECT_ID' => $projectId
        );
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        return $tasks;
    }

    public function getDelayedTasksByUser($userId) {

        $today = new \DateTime();
        $todayString = $today->modify('+1 day')->format('d/m/Y');
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $query = <<<SQL
			SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
                   TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                   T.TIME_PLANNED AS DURATION, P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                   T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.DESCRIPTION,
                   TS.TASK_CAN_START AS TASK_CAN_START, PT.ALLOWENDTASK, TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP
              FROM TASK_TYPE TT
        RIGHT JOIN TASK_STATUS TS 		ON TT.ID            = TS.TASK_TYPE
        RIGHT JOIN TASK T 				ON TS.ID            = T.TASK_STATUS
        INNER JOIN ALLOCATION A 		ON A.TASK           = T.ID
        INNER JOIN RESOURCE_T RT 		ON A.RESOURCE_T     = RT.ID
        INNER JOIN RESOURCE_PROJECT RP 	ON RP.RESOURCE_C    = RT.ID
        INNER JOIN PROJECT P 			ON RP.PROJECT       = P.ID AND P.ID = T.PROJECT
        INNER JOIN PROJECT_TYPE PT 		ON P.PROJECT_TYPE   = PT.ID
        INNER JOIN TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
         LEFT JOIN REPOSITORY R 		ON R.PROJECT        = P.ID
         LEFT JOIN TASK_RELATIONSHIP TR ON (T.ID    = TR.TASK_RELATIONSHIP AND TR.TYPE = 'R')
         LEFT JOIN TASK TTT             ON (TTT.ID  = TR.TASK)
         LEFT JOIN CLASSIFICATION CLA   ON (CLA.ID  = T.CLASSIFICATION)
             WHERE RT.ID = $userId
               AND TO_DATE('$todayString', 'DD/MM/YYYY') > T.ESTIMATED_BEGIN_DATE
               AND T.PERCENT_COMPLETED < 100
               AND P.STATUS = 'A'
               AND T.STATUS = 'A'
               AND T.IS_ISSUE = 'N'
               AND TS.CONTROL <> 'C'
               $filterTypeProject
               AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
          ORDER BY T.PARENT ASC NULLS FIRST
SQL;
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query);
        foreach($tasks as &$task){
            $task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
        }
        return $tasks;
    }
    
    
    public function getDelayedUnatendedTasks($userId) {
        
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
                

        $today = new \DateTime();
        $todayString = $today->modify('+1 day')->format('d/m/Y');
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $query = <<<SQL
			SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
                   TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                   T.TIME_PLANNED AS DURATION, P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                   T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.DESCRIPTION,
                   TS.TASK_CAN_START AS TASK_CAN_START, PT.ALLOWENDTASK, TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP
              FROM TASK_TYPE TT
        RIGHT JOIN TASK_STATUS TS 		ON TT.ID            = TS.TASK_TYPE
        RIGHT JOIN TASK T 				ON TS.ID            = T.TASK_STATUS
        LEFT JOIN ALLOCATION A 		ON A.TASK           = T.ID
        LEFT JOIN RESOURCE_T RT 		ON A.RESOURCE_T     = RT.ID
        INNER JOIN RESOURCE_PROJECT RP 	ON RP.RESOURCE_C    = 5
        INNER JOIN PROJECT P 			ON RP.PROJECT       = P.ID AND P.ID = T.PROJECT
        INNER JOIN PROJECT_TYPE PT 		ON P.PROJECT_TYPE   = PT.ID
        INNER JOIN TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
         LEFT JOIN REPOSITORY R 		ON R.PROJECT        = P.ID
         LEFT JOIN TASK_RELATIONSHIP TR ON (T.ID    = TR.TASK_RELATIONSHIP AND TR.TYPE = 'R')
         LEFT JOIN TASK TTT             ON (TTT.ID  = TR.TASK)
         LEFT JOIN CLASSIFICATION CLA   ON (CLA.ID  = T.CLASSIFICATION)
             WHERE $stringProjectByManager
               AND RT.ID is null
                 AND TO_DATE('$todayString', 'DD/MM/YYYY') > T.ESTIMATED_BEGIN_DATE
               AND T.PERCENT_COMPLETED < 100
               AND P.STATUS = 'A'
               AND T.STATUS = 'A'
               AND T.IS_ISSUE = 'N'
               AND TS.CONTROL <> 'C'
               AND PT.EXTERNALTYPE IN ('B', 'E') AND PT.EXTERNALTYPE <> 'L'
               AND P.STATUS <> 'D'
               AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
          ORDER BY T.PARENT ASC NULLS FIRST
SQL;
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $tasks;
    }

	public function getProjectByTasks($tasksIds) {
		$query = <<<SQL
			SELECT DISTINCT PRO.ID,PRO.ID||' - '||PRO.NAME AS TASK_NAME,'true' as "_hasChild"
			FROM 			PROJECT PRO
            LEFT JOIN       PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
            LEFT JOIN       TASK_STATUS PS ON (PS.ID = PRO.PROJECT_STATUS)
			JOIN 			TASK TAS ON (TAS.PROJECT = PRO.ID )
			WHERE 			TAS.ID IN ($tasksIds)
			AND             PT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNALTYPE_NOT_IN)
			AND             PS.CONTROL NOT IN(:ARRAY_CONTROL_NOT_IN)
			ORDER BY        PRO.ID
SQL;
        $params = array(
            ':ARRAY_CONTROL_NOT_IN' => array(Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                                             Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
                                             Entities\TaskStatus::TASK_STATUS_CONTROL_NUMB),
            ':ARRAY_EXTERNALTYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE)

        );
        $paramTypes = array(
            ':ARRAY_CONTROL_NOT_IN' => Connection::PARAM_STR_ARRAY,
            ':ARRAY_EXTERNALTYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query,$params,$paramTypes);
	}

	public function getDelayedUnatendedTasksByDate( $dateTask, $finalTaskStatusId ) {
	     $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$query = <<<SQL
			SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
                    TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                    T.TIME_PLANNED AS DURATION, P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                    T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.DESCRIPTION,
                    PT.ALLOWENDTASK, TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP
              FROM TASK_TYPE TT
            RIGHT JOIN TASK_STATUS TS 		ON TT.ID            = TS.TASK_TYPE
            RIGHT JOIN TASK T 				ON TS.ID            = T.TASK_STATUS
            LEFT JOIN ALLOCATION A 		ON A.TASK           = T.ID
            LEFT JOIN RESOURCE_T RT 		ON A.RESOURCE_T     = RT.ID
            INNER JOIN RESOURCE_PROJECT RP 	ON RP.RESOURCE_C    = 5
            INNER JOIN PROJECT P 			ON RP.PROJECT       = P.ID AND P.ID = T.PROJECT
            INNER JOIN PROJECT_TYPE PT 		ON P.PROJECT_TYPE   = PT.ID
            INNER JOIN TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
             LEFT JOIN REPOSITORY R 		ON R.PROJECT        = P.ID
             LEFT JOIN TASK_RELATIONSHIP TR ON (T.ID    = TR.TASK_RELATIONSHIP AND TR.TYPE = 'R')
             LEFT JOIN TASK TTT             ON (TTT.ID  = TR.TASK)
             LEFT JOIN CLASSIFICATION CLA   ON (CLA.ID  = T.CLASSIFICATION)
            WHERE 	$stringProjectByManager
            AND RT.ID IS NULL
            AND(
                TO_DATE(:TASK_DATE, 'DD/MM/YYYY') BETWEEN T.ESTIMATED_BEGIN_DATE
              AND T.ESTIMATED_END_DATE
                OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY')
                OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_END_DATE,'DD/MM/YYYY')
            )
            AND 	T.PERCENT_COMPLETED < 100
            AND 	P.STATUS = 'A'
            AND 	T.STATUS = 'A'
            AND 	T.IS_ISSUE = 'N'
            AND     TS.CONTROL <> 'C'
            $filterTypeProject
            AND 	TS.ID <> :FINAL_TASK_STATUS_ID
			AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
            ORDER BY T.PARENT ASC NULLS FIRST
SQL;
		$params = array(
			':USER_ID' => $userId,
			':TASK_DATE' => $dateTask,
            ':CONTROL_IN_HOMOLOGATION' => Entities\TaskStatus::TASK_STATUS_CONTROL_IN_HOMOLOGATION,
            ':typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            ':typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            ':statusActive' => Utilities::STATUS_ACTIVE,
			':FINAL_TASK_STATUS_ID' => $finalTaskStatusId
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
		}
		return $tasks;
	}
	
	public function getTasksByUserFromDate($userId, $dateTask, $finalTaskStatusId) {

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$query = <<<SQL
			SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
                    TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                    T.TIME_PLANNED AS DURATION, P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                    T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.DESCRIPTION,
                    PT.ALLOWENDTASK, TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP
            FROM 		TASK_TYPE TT
            RIGHT JOIN 	TASK_STATUS TS 			ON TT.ID = TS.TASK_TYPE
            RIGHT JOIN 	TASK T 					ON TS.ID = T.TASK_STATUS
            INNER JOIN 	ALLOCATION A 			ON A.TASK = T.ID
            INNER JOIN 	RESOURCE_T RT 			ON A.RESOURCE_T = RT.ID
            INNER JOIN 	RESOURCE_PROJECT RP 	ON RP.RESOURCE_C = RT.ID
            INNER JOIN 	PROJECT P 				ON RP.PROJECT = P.ID AND T.PROJECT = P.ID
            INNER JOIN	PROJECT_TYPE PT 		ON P.PROJECT_TYPE = PT.ID
            LEFT JOIN 	REPOSITORY R 			ON R.PROJECT = P.ID
            INNER JOIN	TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
            LEFT JOIN   TASK_RELATIONSHIP TR ON (TR.TASK_RELATIONSHIP = T.ID)
            LEFT JOIN   TASK TTT ON (TTT.ID = TR.TASK)
            LEFT JOIN   CLASSIFICATION CLA ON (CLA.ID = T.CLASSIFICATION)
            WHERE 	RT.ID = :USER_ID
            AND(
                TO_DATE(:TASK_DATE, 'DD/MM/YYYY') BETWEEN T.ESTIMATED_BEGIN_DATE
              AND T.ESTIMATED_END_DATE
                OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY')
                OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_END_DATE,'DD/MM/YYYY')
            )
            AND 	T.PERCENT_COMPLETED < 100
            AND 	P.STATUS = 'A'
            AND 	T.STATUS = 'A'
            AND 	T.IS_ISSUE = 'N'
            AND     TS.CONTROL <> 'C'
            $filterTypeProject
            AND 	TS.ID <> :FINAL_TASK_STATUS_ID
			AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
            ORDER BY T.PARENT ASC NULLS FIRST
SQL;
		$params = array(
			':USER_ID' => $userId,
			':TASK_DATE' => $dateTask,
            ':CONTROL_IN_HOMOLOGATION' => Entities\TaskStatus::TASK_STATUS_CONTROL_IN_HOMOLOGATION,
            ':typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            ':typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            ':statusActive' => Utilities::STATUS_ACTIVE,
			':FINAL_TASK_STATUS_ID' => $finalTaskStatusId
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
		}
		return $tasks;
	}
	
	
	public function getUnatendedTasksByUserFromDate( $dateTask, $finalTaskStatusId, $userId) {

        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$query = <<<SQL
			SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
                    TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
                    T.TIME_PLANNED AS DURATION, P.ID AS PROJECT_ID, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                    T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, T.DESCRIPTION,
                    PT.ALLOWENDTASK, TTT.RESPONSE AS TASK_RESPONSE, TR.TYPE AS TYPE_RELATIONSHIP
            FROM 		TASK_TYPE TT
            RIGHT JOIN 	TASK_STATUS TS 			ON TT.ID = TS.TASK_TYPE
            RIGHT JOIN 	TASK T 					ON TS.ID = T.TASK_STATUS
            LEFT  JOIN 	ALLOCATION A 			ON A.TASK = T.ID
            LEFT  JOIN 	RESOURCE_T RT 			ON A.RESOURCE_T = RT.ID
            INNER JOIN 	RESOURCE_PROJECT RP 	ON RP.RESOURCE_C = RT.ID
            INNER JOIN 	PROJECT P 				ON RP.PROJECT = P.ID AND T.PROJECT = P.ID
            INNER JOIN	PROJECT_TYPE PT 		ON P.PROJECT_TYPE = PT.ID
            LEFT JOIN 	REPOSITORY R 			ON R.PROJECT = P.ID
            INNER JOIN	TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
            LEFT JOIN   TASK_RELATIONSHIP TR ON (TR.TASK_RELATIONSHIP = T.ID)
            LEFT JOIN   TASK TTT ON (TTT.ID = TR.TASK)
            LEFT JOIN   CLASSIFICATION CLA ON (CLA.ID = T.CLASSIFICATION)
            WHERE $stringProjectByManager
            AND RT.ID IS NULL
            AND(
                TO_DATE(:TASK_DATE, 'DD/MM/YYYY') BETWEEN T.ESTIMATED_BEGIN_DATE
              AND T.ESTIMATED_END_DATE
                OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY')
                OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_END_DATE,'DD/MM/YYYY')
            )
            AND 	T.PERCENT_COMPLETED < 100
            AND 	P.STATUS = 'A'
            AND 	T.STATUS = 'A'
            AND 	T.IS_ISSUE = 'N'
            AND     TS.CONTROL <> 'C'
            AND PT.EXTERNALTYPE IN ('B','E') AND PT.EXTERNALTYPE <> 'L' 
            AND P.STATUS <> 'D'
            $filterTypeProject
            AND 	TS.ID <> :FINAL_TASK_STATUS_ID
			AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
            ORDER BY T.PARENT ASC NULLS FIRST
SQL;
		$params = array(
			':TASK_DATE' => $dateTask,
            ':CONTROL_IN_HOMOLOGATION' => Entities\TaskStatus::TASK_STATUS_CONTROL_IN_HOMOLOGATION,
            ':typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            ':typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            ':statusActive' => Utilities::STATUS_ACTIVE,
			':FINAL_TASK_STATUS_ID' => $finalTaskStatusId
		);
		
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
		}
		return $tasks;
	}
	
	public function getTasksByUserFromDateOnDemand($userId, $dateTask, $projectId) {
		$query = <<<SQL
			SELECT * FROM (SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
							TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
							T.TIME_PLANNED AS DURATION, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE, T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
							T.DESCRIPTION, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, P.ID AS PROJECT_ID,
							T.ID AS TASK_ID,PT.USETIMEPLANNING AS USE_TIME_PLANNING, PT.ALLOWENDTASK, TP.NAME AS PARENT_NAME, TTT.RESPONSE AS TASK_RESPONSE,
							TR.TYPE AS TYPE_RELATIONSHIP,CLA.LABEL AS CLASSIFICATION_NAME,
							DECODE (IAPT.ID, NULL, 'N', 'Y') AS IS_ALLOW_PLAY
			FROM 		TASK_TYPE TT
			RIGHT JOIN 	TASK_STATUS TS 		ON TT.ID = TS.TASK_TYPE
			RIGHT JOIN 	TASK T 				ON TS.ID = T.TASK_STATUS
			INNER JOIN 	ALLOCATION A 		ON A.TASK = T.ID
			INNER JOIN 	RESOURCE_T RT 		ON A.RESOURCE_T = RT.ID
			INNER JOIN 	PROJECT P 			ON T.PROJECT = P.ID
			LEFT JOIN   PROJECT_TYPE PT		ON PT.ID = P.PROJECT_TYPE
			LEFT JOIN   TASK TP		        ON TP.ID = T.PARENT
			LEFT JOIN   TASK_RELATIONSHIP TR ON (T.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
			LEFT JOIN   TASK TTT ON (TTT.ID = TR.TASK)
			LEFT JOIN   CLASSIFICATION CLA ON (CLA.ID = T.CLASSIFICATION)
            INNER JOIN	TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
             LEFT JOIN (SELECT TASK.ID
                          FROM TASK_TYPE TAST
                    RIGHT JOIN TASK_STATUS TSTT ON TSTT.TASK_TYPE = TAST.ID
                    RIGHT JOIN TASK ON TASK.TASK_STATUS = TSTT.ID
                    INNER JOIN PROJECT PROJ ON PROJ.ID = TASK.PROJECT
                     LEFT JOIN PROJECT_TYPE PROT ON PROT.ID = PROJ.PROJECT_TYPE
                    INNER JOIN TASK_STATUS TSTP ON TSTP.ID = PROJ.PROJECT_STATUS
                     LEFT JOIN CLASSIFICATION CLAS ON CLAS.ID = TASK.CLASSIFICATION
                         WHERE (TASK.IS_PARENT = 'Y'
                            OR (PROT.USETIMEPLANNING = 'Y' AND TASK.TIME_PLANNED = 0)
                            OR (TSTP.CONTROL = 'O' AND CLAS.CONTROL IS NULL)
                     OR EXISTS (SELECT TTAS.ID
                                  FROM TASK_RELATIONSHIP TREL
                             LEFT JOIN TASK TTAS ON TTAS.ID = TREL.TASK_RELATIONSHIP
                                  JOIN TASK TASQ ON TASQ.ID = TREL.TASK
                                 WHERE (TREL.TYPE IN ('C', 'G', 'E') AND TREL.TASK = TASK.ID AND TTAS.PERCENT_COMPLETED < 100)
                                    OR (TREL.TASK_RELATIONSHIP = TASK.ID AND TREL.TYPE = 'M' AND TASQ.PERCENT_COMPLETED < 100)
                               )
                     OR EXISTS (SELECT TAT.ID
                                  FROM TASK TAS
                                  JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                  JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
                                  JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                  JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                  JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                  JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
                                  JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                  JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
                                 WHERE TAS.ID = TASK.ID
                                   AND ISS.STATUS = 'A'
                                   AND TAT.STATUS = 'A'
                                   AND TAS.STATUS = 'A'
                                   AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                   AND TAT.PERCENT_COMPLETED < 100
                                   AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
                                   AND TAS.CLASSIFICATION NOT IN (SELECT TTC.CLASSIFICATION_ID
                                                                    FROM TASK ISS
                                                                    JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                                    JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                                   WHERE ISS.ID = TAR.TASK)))) IAPT ON IAPT.ID = T.ID
			WHERE 	RT.ID   = :USER_ID
			AND 	P.ID = :PROJECT_ID
			AND(
					TO_DATE(:TASK_DATE, 'DD/MM/YYYY') BETWEEN T.ESTIMATED_BEGIN_DATE
				AND T.ESTIMATED_END_DATE
				OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY')
				OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_END_DATE,'DD/MM/YYYY')
			)
			AND 	T.PERCENT_COMPLETED < 100
			AND 	P.STATUS = 'A'
			AND 	T.STATUS = 'A'
			AND 	T.IS_ISSUE = 'N'
			AND     TS.CONTROL <> 'C'
            AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <>'A' OR CLA.CONTROL IS NULL))
SQL;
		$params = array(
			':USER_ID' => $userId,
			':PROJECT_ID' => $projectId,
            ':typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            ':typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            ':statusActive' => Utilities::STATUS_ACTIVE,
			':TASK_DATE' => $dateTask
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query.") P
			ORDER BY P.ESTIMATED_BEGIN_DATE DESC, P.ESTIMATED_END_DATE DESC", $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
			if($task['USE_TIME_PLANNING'] == 'N'){
				$task['ESTIMATED_BEGIN_DATE'] = '';
				$task['ESTIMATED_END_DATE'] = '';
			}
			if($task['ALLOWENDTASK'] == 'N'){
				$task['PERCENT_COMPLETED'] = '';
			}
		}
		return $tasks;
	}
	
	public function getUnatendedTasksByUserFromDateOnDemand($dateTask, $projectId) {
		$query = <<<SQL
			SELECT * FROM (SELECT DISTINCT T.ID, RT.ID AS RESOURCE_T, TS.CONTROL AS TASK_STATUS, TS.LABEL AS TASK_STATUS_LABEL,
							TT.CONTROL AS TASK_TYPE, TT.LABEL AS TASK_TYPE_LABEL, T.PARENT, T.NAME AS TASK_NAME, P.NAME AS PROJECT_NAME,
							T.TIME_PLANNED AS DURATION, TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE, T.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
							T.DESCRIPTION, TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE, A.ID AS ALLOCATION, P.ID AS PROJECT_ID,
							T.ID AS TASK_ID,PT.USETIMEPLANNING AS USE_TIME_PLANNING, PT.ALLOWENDTASK, TP.NAME AS PARENT_NAME, TTT.RESPONSE AS TASK_RESPONSE,
							TR.TYPE AS TYPE_RELATIONSHIP,CLA.LABEL AS CLASSIFICATION_NAME,
							DECODE (IAPT.ID, NULL, 'N', 'Y') AS IS_ALLOW_PLAY
			FROM 		TASK_TYPE TT
			RIGHT JOIN 	TASK_STATUS TS 		ON TT.ID = TS.TASK_TYPE
			RIGHT JOIN 	TASK T 				ON TS.ID = T.TASK_STATUS
			LEFT JOIN 	ALLOCATION A 		ON A.TASK = T.ID
			LEFT JOIN 	RESOURCE_T RT 		ON A.RESOURCE_T = RT.ID
			INNER JOIN 	PROJECT P 			ON T.PROJECT = P.ID
			LEFT JOIN   PROJECT_TYPE PT		ON PT.ID = P.PROJECT_TYPEm
			LEFT JOIN   TASK TP		        ON TP.ID = T.PARENT
			LEFT JOIN   TASK_RELATIONSHIP TR ON (T.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
			LEFT JOIN   TASK TTT ON (TTT.ID = TR.TASK)
			LEFT JOIN   CLASSIFICATION CLA ON (CLA.ID = T.CLASSIFICATION)
            INNER JOIN	TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
             LEFT JOIN (SELECT TASK.ID
                          FROM TASK_TYPE TAST
                    RIGHT JOIN TASK_STATUS TSTT ON TSTT.TASK_TYPE = TAST.ID
                    RIGHT JOIN TASK ON TASK.TASK_STATUS = TSTT.ID
                    INNER JOIN PROJECT PROJ ON PROJ.ID = TASK.PROJECT
                     LEFT JOIN PROJECT_TYPE PROT ON PROT.ID = PROJ.PROJECT_TYPE
                    INNER JOIN TASK_STATUS TSTP ON TSTP.ID = PROJ.PROJECT_STATUS
                     LEFT JOIN CLASSIFICATION CLAS ON CLAS.ID = TASK.CLASSIFICATION
                         WHERE (TASK.IS_PARENT = 'Y'
                            OR (PROT.USETIMEPLANNING = 'Y' AND TASK.TIME_PLANNED = 0)
                            OR (TSTP.CONTROL = 'O' AND CLAS.CONTROL IS NULL)
                     OR EXISTS (SELECT TTAS.ID
                                  FROM TASK_RELATIONSHIP TREL
                             LEFT JOIN TASK TTAS ON TTAS.ID = TREL.TASK_RELATIONSHIP
                                  JOIN TASK TASQ ON TASQ.ID = TREL.TASK
                                 WHERE (TREL.TYPE IN ('C', 'G', 'E') AND TREL.TASK = TASK.ID AND TTAS.PERCENT_COMPLETED < 100)
                                    OR (TREL.TASK_RELATIONSHIP = TASK.ID AND TREL.TYPE = 'M' AND TASQ.PERCENT_COMPLETED < 100)
                               )
                     OR EXISTS (SELECT TAT.ID
                                  FROM TASK TAS
                                  JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                  JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
                                  JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                  JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                  JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                  JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
                                  JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                  JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
                                 WHERE TAS.ID = TASK.ID
                                   AND ISS.STATUS = 'A'
                                   AND TAT.STATUS = 'A'
                                   AND TAS.STATUS = 'A'
                                   AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                   AND TAT.PERCENT_COMPLETED < 100
                                   AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
                                   AND TAS.CLASSIFICATION NOT IN (SELECT TTC.CLASSIFICATION_ID
                                                                    FROM TASK ISS
                                                                    JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                                    JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                                   WHERE ISS.ID = TAR.TASK)))) IAPT ON IAPT.ID = T.ID
			WHERE 	RT.ID  IS NULL
			AND 	P.ID = :PROJECT_ID
			AND(
					TO_DATE(:TASK_DATE, 'DD/MM/YYYY') BETWEEN T.ESTIMATED_BEGIN_DATE
				AND T.ESTIMATED_END_DATE
				OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY')
				OR 	:TASK_DATE = TO_CHAR(T.ESTIMATED_END_DATE,'DD/MM/YYYY')
			)
			AND 	T.PERCENT_COMPLETED < 100
			AND 	P.STATUS = 'A'
			AND 	T.STATUS = 'A'
			AND 	T.IS_ISSUE = 'N'
			AND     TS.CONTROL <> 'C'
            AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <>'A' OR CLA.CONTROL IS NULL))
SQL;
		$params = array(
			':PROJECT_ID' => $projectId,
            ':typeLocked' => Entities\TaskTypeClassification::TYPE_LOCKED,
            ':typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            ':statusActive' => Utilities::STATUS_ACTIVE,
			':TASK_DATE' => $dateTask
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query.") P
			ORDER BY P.ESTIMATED_BEGIN_DATE DESC, P.ESTIMATED_END_DATE DESC", $params);
		foreach($tasks as &$task){
			$task['DURATION'] = Utilities::secondsToTime($task['DURATION']);
			if($task['USE_TIME_PLANNING'] == 'N'){
				$task['ESTIMATED_BEGIN_DATE'] = '';
				$task['ESTIMATED_END_DATE'] = '';
			}
			if($task['ALLOWENDTASK'] == 'N'){
				$task['PERCENT_COMPLETED'] = '';
			}
		}
		return $tasks;
	}

	public function getTaskByProjectGantt($projectId,$organizationId,$conditionsString = '') {
		$query = <<<SQL
      SELECT * FROM(
			SELECT  TA.ID AS TASK_ID,
                    TA.NAME AS TASK_NAME,
                    TA.DESCRIPTION,
                    TA.URL_TO_TEST,TS.CONTROL,
                    TS.LABEL AS TASK_STATUS_NAME,
                    TS.CONTROL AS TASK_STATUS_CONTROL,
                    TO_CHAR(CASE WHEN TS.CONTROL = 'B'
                                 THEN NULL
                                 WHEN APR.BEGIN_HOUR IS NOT NULL
                                 THEN APR.BEGIN_HOUR
                                 WHEN TA.BEGIN_DATE IS NOT NULL
                                 THEN TA.BEGIN_DATE
                                 WHEN TA.ESTIMATED_BEGIN_DATE IS NOT NULL AND TA.PERCENT_COMPLETED = 100
                                 THEN TA.ESTIMATED_BEGIN_DATE
                                 ELSE NULL
                            END  , 'DD/MM/YYYY') AS REAL_BEGIN,
                    TO_CHAR(CASE WHEN TS.CONTROL = 'B'
                                 THEN NULL
                                 WHEN (TA.END_DATE IS NULL AND APR.END_HOUR IS NOT NULL) OR APR.END_HOUR >= TA.END_DATE
                                 THEN APR.END_HOUR
                                 WHEN (APR.END_HOUR IS NULL AND TA.END_DATE IS NOT NULL) OR TA.END_DATE >= APR.END_HOUR
                                 THEN TA.END_DATE
                                 WHEN TA.ESTIMATED_END_DATE IS NOT NULL AND TA.PERCENT_COMPLETED = 100
                                 THEN TA.ESTIMATED_END_DATE
                                 ELSE null
                            END  , 'DD/MM/YYYY') AS REAL_END,
                    '#2c51e7' AS COLOR,
                    '#2c51e7' AS COLOR_ESTIMATED,
                    '#85AB2C' AS COLOR_REAL,
					TA.IS_PARENT,
					CASE WHEN TS.CONTROL = 'B' AND TAPA.ESTIMATED_END_DATE IS NULL
                         THEN TO_CHAR(PRO.END_DATE, 'DD/MM/YYYY')
                         WHEN TS.CONTROL = 'B'
                         THEN TO_CHAR(TAPA.ESTIMATED_END_DATE, 'DD/MM/YYYY')
                         ELSE TO_CHAR(TA.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY')
                    END AS ESTIMATED_BEGIN,
                    CASE WHEN TS.CONTROL = 'B' AND TAPA.ESTIMATED_END_DATE IS NULL
                         THEN TO_CHAR(PRO.END_DATE, 'DD/MM/YYYY')
                         WHEN TS.CONTROL = 'B'
                         THEN TO_CHAR(TAPA.ESTIMATED_END_DATE, 'DD/MM/YYYY')
                         ELSE TO_CHAR(TA.ESTIMATED_END_DATE, 'DD/MM/YYYY')
                    END AS ESTIMATED_END,
                    TA.CLASSIFICATION AS TASK_CLASSIFICATION,
					TA.PERCENT_COMPLETED AS COMPLETED,
                    TAPA.NAME AS PARENT_NAME,
                    TAPA.ID AS PARENT,
                    TATP.NAME AS PRECURSOR_NAME,
                    TATP.ID AS PREDECESSOR,
                    TA.TIME_PLANNED,
                    (SELECT RTRIM(XMLAGG(XMLELEMENT(E,T.ID,',').EXTRACT('//text()') ORDER BY T.ID).GETCLOBVAL(),',')FROM TASK T WHERE T.PARENT = TA.ID AND T.STATUS = 'A')AS CHILDREN
             FROM	TASK TA
        LEFT JOIN 	TASK_RELATIONSHIP TP ON (TA.ID = TP.TASK AND TP.TYPE = 'P')
        LEFT JOIN 	TASK_STATUS TS 			ON (TS.ID = TA.TASK_STATUS)
        LEFT JOIN 	TASK TATP ON (TATP.ID = TP.TASK_RELATIONSHIP)
        LEFT JOIN 	TASK TAPA ON (TAPA.ID = TA.PARENT)
        LEFT JOIN 	PROJECT PRO ON (PRO.ID = TA.PROJECT)
        LEFT JOIN 	PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
        LEFT JOIN 	CLASSIFICATION CLA ON (CLA.ID = TA.CLASSIFICATION)
        LEFT JOIN   (SELECT MAX(END_HOUR) AS END_HOUR, MIN(BEGIN_HOUR) AS BEGIN_HOUR, TASK
                      FROM APPROPRIATION
                      GROUP BY TASK) APR ON (APR.TASK = TA.ID)
        LEFT JOIN (
          SELECT ROWNUM AS ORDEM, T.ID
                  FROM TASK T
                  LEFT JOIN TASK_RELATIONSHIP TF ON (T.ID = TF.TASK AND TF.TYPE = 'P')
                  WHERE T.PROJECT = :project
                    AND	 T.STATUS <> 'D'
                    AND	 NVL(T.IS_ISSUE,'N') <> 'Y'
                  START WITH TF.TASK_RELATIONSHIP IS NULL
                     CONNECT BY PRIOR T.ID = TF.TASK_RELATIONSHIP) O ON (O.ID = TA.ID)
                    WHERE   TA.ORGANIZATION = :organization
                    AND	 TA.PROJECT = :project
                    AND	 TA.STATUS = 'A'
                    AND	 NVL(TA.IS_ISSUE,'N') <> 'Y'
                    AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL)
          ORDER BY TRUNC(CASE WHEN TS.CONTROL = 'B'
                              THEN TAPA.ESTIMATED_END_DATE
                              ELSE TA.ESTIMATED_BEGIN_DATE
                         END), O.ORDEM)
          WHERE  1 = 1
          $conditionsString

SQL;
		$params = array(
            'organization' => $organizationId,
            'project' => $projectId,
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}


    public function getScaleByProjectGantt($projectId,$organizationId) {
        $query = <<<SQL
      SELECT REAL_END-REAL_BEGIN AS MAX_REAL,ESTIMATED_END-ESTIMATED_BEGIN AS MAX_ESTIMATED FROM(
			SELECT
                    MIN(CASE WHEN APR.BEGIN_HOUR IS NOT NULL
                             THEN APR.BEGIN_HOUR
                             WHEN TA.BEGIN_DATE IS NOT NULL
                             THEN TA.BEGIN_DATE
                             WHEN TA.ESTIMATED_BEGIN_DATE IS NOT NULL AND TA.PERCENT_COMPLETED = 100
                             THEN TA.ESTIMATED_BEGIN_DATE
                             ELSE NULL
                        END) AS REAL_BEGIN,
                    MAX(CASE WHEN (TA.END_DATE IS NULL AND APR.END_HOUR IS NOT NULL) OR APR.END_HOUR >= TA.END_DATE
                             THEN APR.END_HOUR
                             WHEN (APR.END_HOUR IS NULL AND TA.END_DATE IS NOT NULL) OR TA.END_DATE >= APR.END_HOUR
                             THEN TA.END_DATE
                             WHEN TA.ESTIMATED_END_DATE IS NOT NULL AND TA.PERCENT_COMPLETED = 100
                             THEN TA.ESTIMATED_END_DATE
                             ELSE null
                        END ) AS REAL_END,
                    MIN(TA.ESTIMATED_BEGIN_DATE) AS ESTIMATED_BEGIN,
                    MAX(TA.ESTIMATED_END_DATE) AS ESTIMATED_END
             FROM	TASK TA
        LEFT JOIN 	TASK_STATUS TS 			ON (TS.ID = TA.TASK_STATUS)
        LEFT JOIN 	PROJECT PRO ON (PRO.ID = TA.PROJECT)
        LEFT JOIN 	CLASSIFICATION CLA ON (CLA.ID = TA.CLASSIFICATION)
         LEFT JOIN   (SELECT MAX(END_HOUR) AS END_HOUR, MIN(BEGIN_HOUR) AS BEGIN_HOUR, TASK
                      FROM APPROPRIATION
                      GROUP BY TASK) APR ON (APR.TASK = TA.ID)
            WHERE   TA.ORGANIZATION = :organization
            AND	 TA.PROJECT = :project
            AND	 TA.STATUS <> 'D'
            AND	 NVL(TA.IS_ISSUE,'N') <> 'Y'
            AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))

SQL;
        $params = array(
            'organization' => $organizationId,
            'project' => $projectId,
        );
        $return = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        if(!empty($return)){
            $return = $return[0];
            $return['MAX_REAL'] = intval($return['MAX_REAL']);
            $return['MAX_ESTIMATED'] = intval($return['MAX_ESTIMATED']);
           if($return['MAX_REAL'] > $return['MAX_ESTIMATED']){
               $return = $return['MAX_REAL'];
           }else{
               $return = $return['MAX_ESTIMATED'];
           }
        }else{
            $return = null;
        }

        return $return;
    }

    public function findMergeRequestByTask($taskIds) {
        $query = <<<SQL
            SELECT T.NAME
              FROM TASK_RELATIONSHIP TR
              JOIN TASK T ON TR.TASK = T.ID
             WHERE TYPE = 'M'
               AND TR.TASK IN ($taskIds)
                OR (TR.TASK_RELATIONSHIP IN ($taskIds) AND TYPE = 'G')
SQL;
		return $this->getEntityManager()->getConnection()->fetchAll($query);
	}

	public function getTaskByDate($date, $userId) {
		$query = <<<SQL
			SELECT 	T.NAME, T.ID, MAX(AP.END_HOUR) AS END_HOUR, TRUNC(SUM(AP.TOTAL)/3600, 2) AS TOTAL , T.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED,
					C.DAY_HOURS
			FROM 		RESOURCE_T RT
			INNER JOIN 	APPROPRIATION AP 	ON (RT.ID = AP.RESOURCE_T)
			INNER JOIN 	TASK T 				ON (AP.TASK = T.ID)
			LEFT JOIN 	CALENDAR C 			ON (C.ID = RT.CALENDAR)
			WHERE 	RT.ID = :USER_ID
			AND 	TO_CHAR(AP.BEGIN_HOUR, 'DD/MM/YYYY') = :DATE_CURRENT
			GROUP BY T.NAME, T.ID, T.PERCENT_COMPLETED, C.DAY_HOURS
SQL;
		$params = array('DATE_CURRENT' => $date, 'USER_ID' => $userId);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getUserTasksById($userId) {
		$query = <<<SQL
			SELECT T.ID AS ID, T.NAME AS NAME, D.LOCAL_DIRECTORY AS DIR, T.PROJECT AS PROJECT_ID, PR.REPOSITORY, R.URL
			FROM 		RESOURCE_T RT
			INNER JOIN 	ALLOCATION A 			ON RT.ID = A.RESOURCE_T
			INNER JOIN 	TASK T 					ON A.TASK = T.ID
			LEFT JOIN 	PROJECT_REPOSITORY PR 	ON T.PROJECT = PR.PROJECT
			INNER JOIN 	REPOSITORY R 			ON PR.REPOSITORY = R.ID
			LEFT JOIN 	DIRECTORY D 			ON D.REPOSITORY = R.ID
			WHERE 	RT.ID = :USER_ID
			AND 	T.STATUS = 'A'
SQL;
		$params = array(
			'USER_ID' => $userId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getUserTasksNameById($userId, $project) {
		$query = <<<SQL
		SELECT TASK.ID , TASK.NAME
		FROM TASK
		INNER JOIN ALLOCATION ON TASK.ID = ALLOCATION.TASK
		INNER JOIN RESOURCE_PROJECT ON ALLOCATION.RESOURCE_PROJECT = RESOURCE_PROJECT.ID
		INNER JOIN PROJECT ON RESOURCE_PROJECT.PROJECT = PROJECT.ID
		INNER JOIN RESOURCE_T ON RESOURCE_PROJECT.RESOURCE_C = RESOURCE_T.ID
		INNER JOIN BUSINESS_PARTNER ON BUSINESS_PARTNER.ID = RESOURCE_T.BUSINESS_PARTNER

		WHERE BUSINESS_PARTNER.ID = :USER_ID AND TASK.STATUS = 'A' AND PROJECT.ID = :PROJECT
SQL;
		$params = array(
			'USER_ID' => $userId,
			'PROJECT' =>$project
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getAllTaskStatus($organizationId) {
		$query = <<<SQL
		SELECT 	TASK_STATUS.ID,TASK_STATUS.NAME,
				TASK_STATUS.ID,TASK_STATUS.LABEL
		  FROM 	TASK_STATUS
		 WHERE 	TASK_STATUS.ORGANIZATION = :organization
		 AND 	TASK_STATUS.IS_TASK = 'Y'
SQL;
		$params = array(
			'organization' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getTaskDataToGenerateTask($relatedTasks){

        $query = <<<SQL
		SELECT TAS.ID,PRO.NAME AS PROJECT_NAME,TAS.NAME,TAS.PROJECT
          FROM TASK TAS
          JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
          WHERE TAS.ID IN (:TASKS)
SQL;
        $params = array(
            'TASKS' => $relatedTasks
        );
        $paramTypes = array(
            'TASKS' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

	public function getTaskData($taskId) {
		$query = "SELECT * FROM TASK T WHERE T.id = :taskId";
		$params = array(
			'taskId' => $taskId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getTaskAppropriations($taskId, $businessPartnerId) {
		$query = "SELECT ALLOCATION, TRUNC(TOTAL/3600)||':'|| SUBSTR(TO_CHAR(100 +MOD(TRUNC(TOTAL/60),60)),2,2) AS TOTAL,
 				TO_CHAR(END_HOUR, 'DD/MM/YYYY HH24:MI') AS END_HOUR,
 				TO_CHAR(BEGIN_HOUR,'DD/MM/YYYY HH24:MI') AS BEGIN_HOUR,
 				AP.DESCRIPTION, NOTE, NVL(COMPLETED, 0) AS PERCENT_COMPLETED, AP.ID
 				FROM APPROPRIATION AP
 				LEFT JOIN ALLOCATION AL ON AL.ID = AP.ALLOCATION
 				LEFT JOIN TASK T ON T.ID = AL.TASK
 				LEFT JOIN RESOURCE_PROJECT RP ON RP.ID = AL.RESOURCE_PROJECT
 				LEFT JOIN PROJECT P ON P.ID = T.PROJECT
 				LEFT JOIN RESOURCE_T RT ON RT.ID = RP.RESOURCE_C
 				WHERE T.ID = :taskId AND RT.BUSINESS_PARTNER = :businessPartnerId";

		$params = array(
			'businessPartnerId' => $businessPartnerId,
			'taskId' => $taskId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findIssueStatus($organizationId, $filterName = false, $filterOperator= false, $filterValue = false) {
        if($filterOperator == "LIKE") {
            $filterName = "UPPER("."TS.".$filterName.")";
            $filterValue = "UPPER('".$filterValue."')";
            $filterString = "AND $filterName $filterOperator $filterValue";
        } else {
            $filterString = "";
        }
        $query = <<<SQL
			SELECT DISTINCT TS.LABEL, TS.ID, TS.CONTROL,TT.CONTROL AS TASK_TYPE_CONTROL, TT.ID AS TASK_TYPE_ID
			FROM 		TASK_STATUS TS
			LEFT JOIN 	TASK_TYPE TT 		ON TT.ID = TS.TASK_TYPE
			WHERE 	TS.ORGANIZATION = :organizationId
			AND 	(TS.USED_IN_ISSUE = 'Y'
			OR 		TT.CONTROL = 'I'
			OR 		TT.CONTROL = 'T'
			OR 		TT.CONTROL = 'D')
            AND     TT.IS_PROJECT = 'N'
			$filterString
			ORDER BY TS.ID
SQL;

		$params = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function findIssueClassificationCP($nrOrg, $filterString = '', $classificationFilter = array(0)) {

        $query = <<<SQL
        SELECT *
          FROM (SELECT CLA.ID, CLA.LABEL,DECODE(CLA.CDASSUNTO,NULL,'Zeedhi','Zeedhi e Delphi') AS PLATAFORMA_NAME
                  FROM CLASSIFICATION CLA
                  JOIN CLASSIFICATION_PROJECT_TYPE CPT ON (CPT.CLASSIFICATION = CLA.ID AND CPT.TYPE = 'P')
                  JOIN PROJECT_TYPE PRT ON (PRT.ID = CPT.PROJECT_TYPE)
                 WHERE PRT.EXTERNALTYPE = 'P'
                   AND CLA.IS_TASK = 'Y'
                   AND CLA.CONTROL IS NULL
                   AND CLA.ORGANIZATION = :ORGANIZATION_ID
                   AND CLA.ID NOT IN (:CLASSIFICATION_FILTER)
              GROUP BY CLA.LABEL, CLA.ID, CLA.CDASSUNTO
              ORDER BY CLA.LABEL)
		 WHERE 1 = 1
			   $filterString
      ORDER BY LABEL
SQL;

        $params = array(
            'CLASSIFICATION_FILTER' => $classificationFilter,
            'ORGANIZATION_ID' => $nrOrg,
        );
        $paramTypes = array(
            'CLASSIFICATION_FILTER' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

	public function findIssueClassification($organizationId, $filter = '', $classificationFilter = array(0)) {

        $query = <<<SQL
        SELECT *
          FROM (SELECT CLA.ID, CLA.LABEL
                  FROM CLASSIFICATION CLA
                  JOIN CLASSIFICATION_PROJECT_TYPE CPT ON (CPT.CLASSIFICATION = CLA.ID AND CPT.TYPE = 'P')
                  JOIN PROJECT_TYPE PRT ON (PRT.ID = CPT.PROJECT_TYPE)
                 WHERE PRT.EXTERNALTYPE = 'P'
                   AND CLA.IS_TASK = 'Y'
                   AND CLA.CONTROL IS NULL
                   AND CLA.ORGANIZATION = :ORGANIZATION_ID
                   AND CLA.ID NOT IN (:CLASSIFICATION_FILTER)
              GROUP BY CLA.LABEL, CLA.ID
              ORDER BY CLA.LABEL)
		 WHERE 1 = 1
			   $filter
      ORDER BY LABEL
SQL;

		$params = array(
			'ORGANIZATION_ID'       => $organizationId,
			'CLASSIFICATION_FILTER' => $classificationFilter
		);
        $paramTypes = array(
            'CLASSIFICATION_FILTER' => Connection::PARAM_STR_ARRAY
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
	}

	public function findIssueType($organizationId, $boardId) {
		$query = <<<SQL
			SELECT IT.ID, IT.LABEL
			FROM TASK_TYPE IT
			WHERE 	IT.ORGANIZATION = :organizationId
			AND 	IT.PROJECT IN (:boardId)
			OR 		IT.CONTROL = 'D'
			OR 		IT.CONTROL = 'T'
			OR 		IT.CONTROL = 'I'
			ORDER BY IT.ID
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'boardId' => $boardId
		);
        $paramTypes = array(
            'boardId' => Connection::PARAM_STR_ARRAY
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
	}

	/**
	 * Builds "conditional queries" from specified filters.
	 * It's not generic and useful only for findKanbanIssues().
	 * @param $conditions
	 * @return string
	 */
	public function getFinalConditions($conditions, $paramsafterbug) {
		if($paramsafterbug) {
            $finalCondition = $paramsafterbug;
        } else{
            $finalCondition = "";
        }
		foreach ($conditions as $param) {
            if ($param['operator'] != 'MAPPED_LIKE_ALL'){
                if (is_array($param['value'])) {
                    $index = sizeof($param['value']);
                } else {
                    $index = 0;
                }
                $name = $param['columnName'];
                if (strcmp($param['columnName'], 'NAME') == 0) {
                    $string = true;
                    $finalCondition .= " AND LOWER(I.NAME) LIKE '";
                } else {
                    $string = false;
                }
                if (strcmp($param['columnName'], 'LABELS') == 0) {
                    $finalCondition .= " AND L.ID IN (";
                } elseif (strcmp($param['columnName'], 'ISSUE_CLASSIFICATION_NAME') == 0) {
                    $finalCondition .= " AND C.ID IN (";
                } elseif (strcmp($param['columnName'], 'ISSUE_STATUS_NAME') == 0) {
                    $finalCondition .= " AND TS.ID IN (";
                } elseif (strcmp($param['columnName'], 'REPORTER_NAME') == 0) {
                    $finalCondition .= " AND RA.RESOURCE_T IN (";
                } elseif (strcmp($param['columnName'], 'FOLLOWERS') == 0) {
                    $finalCondition .= " AND FA.RESOURCE_T IN (";
                } elseif ($param['columnName'] == 'SPONSOR') {
                    $finalCondition .= " AND SA.RESOURCE_T IN (";
                } elseif ($param['columnName'] == 'ISSUE_ID') {
                    $finalCondition .= " AND I.ID IN ( ";
                }elseif ($param['columnName'] == 'PRIORITY') {
                    $finalCondition .= " AND I.PRIORITY IN ( ";
                }
                if ($index > 0 && is_array($param['value'])) {
                    foreach ($param['value'] as &$value) {
                        $index--;
                        if ($param['columnName'] == 'NAME') {
                            $value = strtolower($value);
                        }
                        if (strcmp($param['columnName'], 'STATUS') == 0) {
                            $finalCondition .= " '" . $value . "'";
                        } else {
                            $finalCondition .= $value;
                        }
                        if ($index > 0) {
                            $finalCondition .= ",";
                        }
                    }
                } else {
                    if ($string) {
                        $finalCondition .= strtolower($param['value']);
                    } else {
                        $finalCondition .= "'" . $param['value'] . "'";
                    }
                }
                if (!$string) {
                    $finalCondition .= ")";
                } else {
                    $finalCondition .= "'";
                }
            }
		}
		return $finalCondition;
	}

    public function mappedLikeAllFixBug($conditions)
    {
        $finalCondition = "";
        foreach ($conditions as $param) {
            $name = $param['columnName'];
            if ($param['operator'] == 'MAPPED_LIKE_ALL') {
                foreach ($param['value'][0] as $index => $mappedLikeAllValues) {

                    if ($index == '_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $mappedLikeAllValues . "')";
                        } else {
                            $values = "UPPER('" . $mappedLikeAllValues . "')";
                        }

                        $columnList = explode('|', explode('#', $name)[0]);
                        $likes = array();
                        foreach ($columnList as $column) {

                            if (strcmp($column, 'NAME') == 0) {
                                $table = " I.NAME";
                            }
                             elseif (strcmp($column, 'ISSUE_CLASSIFICATION_NAME') == 0) {
                                $table = "C.LABEL";
                            } elseif (strcmp($column, 'ISSUE_STATUS_NAME') == 0) {
                                $table = "TS.label";
                            }elseif (strcmp($column, 'ISSUE_ID') == 0) {
                                $table = "I.ID";
                            }elseif (strcmp($column, 'DESCRIPTION') == 0) {
                                $table = "I.DESCRIPTION";
                            }
                                $likes[] = "UPPER($table) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $finalCondition .= " AND ( $filterLikeAll )";
                    } else {
                        $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                        if (!empty($tableAlias)) {
                            $finalCondition .= " AND $tableAlias.$index IN $stringValues ";
                        } else {
                            $finalCondition .= " AND $index IN $stringValues ";
                        }
                    }
                }
            }
        }
        return $finalCondition;
    }

    public function findBoardIssues($organizationId,$userId, $boardId, $conditions,$filterComments = '', $filterRelevance = '',
                                    $filterProductVersion = '', $filterStructureId = '', $filterProjectStatusIds = '') {
        $finalCondition = Utilities::createQueryFiltered($conditions, 'I');
        if(!empty($boardId)) {
            $productFilter = 'AND I.PROJECT IN (:boardId)';
        } else {
            $productFilter = 'AND ' . Utilities::getStringSelectAllProjectByUser($userId, 'I.PROJECT');
        }
        $query = <<<SQL
        SELECT I.ID,NAME,SPONSOR_NAME,ISSUE_CLASSIFICATION,ISSUE_CLASSIFICATION_NAME,ISSUE_STATUS,REPORTER_NAME,PRIORITY_NAME,END_DATE,PERCENT_COMPLETED,
                  DUE_DATE,RELEVANCE,ISSUE_STATUS_NAME,ISSUE_STATUS,DESCRIPTION,ALLOW_CLOSE_ISSUE,TYPE_TASK,TYPE_TASK_NAME,PRODUCT_NAME, I.RESPONSE, TASK_TYPE_CONTROL,
                  (  SELECT RTRIM(XMLAGG(XMLELEMENT(E,L.DESCRIPTION,',').EXTRACT('//text()') ORDER BY L.DESCRIPTION).GETCLOBVAL(),',') AS DESCRIPTION
                        FROM TASK_LABEL TL
                        JOIN (     SELECT LAB.ID,LAB.DESCRIPTION,LAB.PROJECT
                                     FROM LABEL LAB
                                LEFT JOIN PROJECT PRO ON(PRO.ID = LAB.PROJECT)
                                LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
                                    WHERE LAB.ORGANIZATION = :organizationId
                                      AND (LAB.RESOURCE_T = :userId OR LAB.RESOURCE_T IS NULL)
                                     ) L ON TL.LABEL = L.ID
                       WHERE TL.TASK = I.ID
                       AND (L.PROJECT IN (:boardId) OR L.PROJECT IS NULL)) AS LABELS,

                       (SELECT RTRIM(XMLAGG(XMLELEMENT(E,FRE.NAME,',').EXTRACT('//text()') ORDER BY FRE.NAME).GETCLOBVAL(),',') AS FOLLOWERS
                          FROM ALLOCATION FA
                          JOIN RESOURCE_T FRE ON (FRE.ID = FA.RESOURCE_T)
                         WHERE FA.TASK = I.ID
                           AND FA.TYPE = :follower) AS FOLLOWERS, RESOURCE_T, CREATION_DATE AS CREATED_AT
                FROM (
                    SELECT *
                      FROM (
                           SELECT I.PRIORITY, I.ID,I.NAME,SRE.ID AS SPONSOR,SRE.NAME AS SPONSOR_NAME,C.LABEL AS ISSUE_CLASSIFICATION_NAME,C.ID AS ISSUE_CLASSIFICATION,
                                  TS.LABEL AS ISSUE_STATUS_NAME, TS.ID AS ISSUE_STATUS,TL.LABEL AS LABELS,
                                  RRE.NAME AS REPORTER_NAME, RRE.ID AS REPORTER,PRI.LABEL AS PRIORITY_NAME,TT.ALLOW_CLOSE_ISSUE,FA.RESOURCE_T AS FOLLOWERS,
                                  TO_CHAR(I.END_DATE, 'DD/MM/YYYY') AS END_DATE,I.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
                                  TO_CHAR(I.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS DUE_DATE,I.DESCRIPTION,ALO.LAST_POST_VIEW,COM.CREATED_AT,
                                  (SELECT COUNT(*) FROM ALLOCATION A WHERE A.IS_RELEVANT = 'Y' AND A.TASK = I.ID) AS RELEVANCE,
                                  TT.ID AS TYPE_TASK,TT.LABEL AS TYPE_TASK_NAME, TT.CONTROL as TASK_TYPE_CONTROL,RA.RESOURCE_T AS RESOURCE_T,PRO.NAME AS PRODUCT_NAME,
                                  DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
                                  DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV, I.RESPONSE, I.CREATED_AT AS CREATION_DATE
                             FROM TASK I
                             JOIN TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
                             JOIN PROJECT PRO ON (PRO.ID = I.PROJECT)
                             JOIN TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
                        LEFT JOIN PRIORITY PRI ON(I.PRIORITY = PRI.ID)
                        LEFT JOIN CLASSIFICATION C ON C.ID = I.CLASSIFICATION
                        LEFT JOIN ALLOCATION RA ON (RA.TASK = I.ID  AND RA.TYPE = :reporter)
                        LEFT JOIN ALLOCATION FA ON (FA.TASK = I.ID  AND FA.TYPE = :follower)
                        LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
                        LEFT JOIN ALLOCATION SA ON (SA.TASK = I.ID AND SA.TYPE = :sponsor)
                        LEFT JOIN RESOURCE_T SRE ON (SRE.ID = SA.RESOURCE_T)
                        LEFT JOIN RESOURCE_T RRE ON (RRE.ID = RA.RESOURCE_T)
                        LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
                        LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
                        LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = I.PRODUCT_VERSION_ID
                        LEFT JOIN   PRODUCT_VERSION PROV ON PROV.ID = I.PRODUCT_VERSION_ID_DEV
                            WHERE I.ORGANIZATION = :organizationId
                              AND I.IS_ISSUE = :isIssue
                              $productFilter
                              $filterStructureId
                              $filterProjectStatusIds
                              AND I.STATUS = :isActive)I
                    WHERE 1 = 1
                    $filterComments
                    $filterRelevance
                    $filterProductVersion)I
                    WHERE 1 =1
                    $finalCondition
         GROUP BY ID,NAME,SPONSOR_NAME,ISSUE_CLASSIFICATION,ISSUE_CLASSIFICATION_NAME,ISSUE_STATUS,REPORTER_NAME,PRIORITY_NAME,END_DATE,PERCENT_COMPLETED,
                  DUE_DATE,RELEVANCE,ISSUE_STATUS_NAME,ISSUE_STATUS,DESCRIPTION,ALLOW_CLOSE_ISSUE,TYPE_TASK,TYPE_TASK_NAME,RESOURCE_T,PRODUCT_NAME, RESPONSE, TASK_TYPE_CONTROL, CREATION_DATE
         ORDER BY ID DESC
SQL;
        $params = array(
            'organizationId' => $organizationId,
            'boardId' => $boardId,
            'userId' => $userId,
            'isActive' => Entities\Task::ACTIVE,
            'isIssue' => Entities\Task::IS_ISSUE,
            'follower' => Entities\Allocation::TYPE_FOLLOWER,
            'reporter' => Entities\Allocation::TYPE_REPORTER,
            'sponsor' => Entities\Allocation::TYPE_SPONSOR,
        );
        $paramTypes = array('boardId' => Connection::PARAM_STR_ARRAY );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);

    }

	public function findKanbanIssues($organizationId,$userId, $boardId, $conditions,$filterComments = '',
                                        $filterRelevance = '', $filterTaskStatus = '', $filterProductVersion = '',
                                     $filterStructureId = '', $filterProjectStatusIds = '')
	{
	    if(!empty($boardId)) {
	        $productFilter = 'AND I.PROJECT IN (:boardId)';
        } else {
            $productFilter = 'AND ' . Utilities::getStringSelectAllProjectByUser($userId, 'I.PROJECT');
        }

        $finalCondition = '';
        if(count($conditions) > 0) {
            $paramsafterbug = $this->mappedLikeAllFixBug($conditions);
            $finalCondition = $this->getFinalConditions($conditions, $paramsafterbug);
        }
        $query = <<<SQL
		SELECT *
		 FROM (
			SELECT I.ID as "id",I.ID,I.NAME, C.LABEL,
			  I.ID AS ISSUE_ID, I.ID||' - '||I.NAME as "label",
              DECODE(I.CURRENT_TASK_STATUS_IN,NULL,NULL,(TRUNC(sysdate)-TRUNC(I.CURRENT_TASK_STATUS_IN))||' dia(s)') AS "lastUpdate",
			  TO_CHAR(I.ESTIMATED_END_DATE, 'DD/MM/YYYY') as "due-date1",
			  I.DESCRIPTION as "description",
			  './images/user.jpg' as "photo", I.PROJECT,
			  (SELECT COUNT(*) FROM ALLOCATION A WHERE A.IS_RELEVANT = 'Y' AND A.TASK = I.ID) AS RELEVANCE,
			  TT.ORDER_TO_VIEW as "column", TT.ID AS ISSUE_STATUS,TT.LABEL AS ISSUE_STATUS_NAME,C.LABEL as "category",
			  TT.LABEL as "columnLabel", TT.CONTROL as "columnType", TT.ORDER_TO_FRAME as ORDER_TO_FRAME,
			  TT.ALLOW_CLOSE_ISSUE as ALLOW_CLOSE_ISSUE, PRI.COLOR,RA.RESOURCE_T AS REPORTER,
			  I.CLASSIFICATION AS "issueClassification",
			  I.CLASSIFICATION AS ISSUE_CLASSIFICATION,
			  TT.ID AS FRAMING, PRV.ID AS PRODUCT_VERSION_ID, PROV.ID AS PRODUCT_VERSION_ID_DEV,
              DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
              DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV,
              DECODE(I.DESCRIPTION, null, 'N', 'Y') AS "kanbanItemStatus", I.PRIORITY, I.CREATED_AT
				FROM 		TASK I
				INNER JOIN 	TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
                INNER JOIN 	TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
				LEFT JOIN PRIORITY PRI ON(I.PRIORITY = PRI.ID)
				LEFT JOIN CLASSIFICATION C ON C.ID = I.CLASSIFICATION
				LEFT JOIN ALLOCATION RA ON RA.TASK = I.ID  AND RA.TYPE = 'R'
			  	LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
			  	LEFT JOIN LABEL L ON TL.LABEL = L.ID
			  	LEFT JOIN ALLOCATION FA ON FA.TASK = I.ID  AND FA.TYPE = 'F'
			  	LEFT JOIN ALLOCATION SA ON SA.TASK = I.ID AND SA.TYPE = 'S'
                LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
                LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
                LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = I.PRODUCT_VERSION_ID
                LEFT JOIN   PRODUCT_VERSION PROV ON PROV.ID = I.PRODUCT_VERSION_ID_DEV
                LEFT JOIN PROJECT PRO ON PRO.ID = I.PROJECT
				WHERE 	I.ORGANIZATION = :organizationId
				AND 	I.IS_ISSUE = 'Y'
				AND 	I.STATUS <> 'D'
				$productFilter
				$filterStructureId
                $filterComments
                $finalCondition
                $filterProjectStatusIds
                GROUP BY I.PRIORITY, I.ID,I.NAME,I.ESTIMATED_END_DATE,I.DESCRIPTION,TT.ID,TT.LABEL,TT.ORDER_TO_VIEW, TT.ORDER_TO_FRAME, TT.ALLOW_CLOSE_ISSUE, TT.CONTROL,TT.ID, PRI.COLOR,RA.RESOURCE_T,I.CLASSIFICATION, I.PROJECT, PRV.ID, PRV.VERSION, PRV.REVISION, PRV.BUILD,
                        PROV.ID, PROV.VERSION, PROV.REVISION, PROV.BUILD, C.LABEL, PRV.ID, PROV.ID, I.CURRENT_TASK_STATUS_IN, I.CREATED_AT
                ORDER BY I.ID DESC )I
        WHERE 1 = 1
              $filterRelevance
              $filterTaskStatus
              $filterProductVersion
SQL;
        $params = array(
			'organizationId' => $organizationId,
			'boardId' => $boardId,
			'userId' => $userId
		);
        $paramTypes = array(
            'boardId' => Connection::PARAM_STR_ARRAY
        );

        $issues = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        foreach ($issues as &$issue){
            $issue['kanbanItemStatus'] = $issue['kanbanItemStatus'] == 'Y' ? true : false;
            $issue['kanbanCaretStatus'] = false;
        }

        return $issues;
	}

	public function findMyIssuesUnread($organizationId, $userId, $filter, $limit, $offset) {
        $filter ? $conditions = Utilities::createQueryFiltered($filter) : $conditions="";
		$query = <<<SQL
          SELECT ISSUE_ID, ISSUE_NAME, LAST_COMMENT_DESCRIPTION,
                 substr(LAST_COMMENT_AUTHOR_FIRSTNAME,1,instr(LAST_COMMENT_AUTHOR_FIRSTNAME,' ',1,1)-1) AS LAST_COMMENT_AUTHOR_FIRSTNAME,
                 LAST_COMMENT_AUTHOR_ID, TO_CHAR(LAST_COMMENT_DATE, 'DD/MM/YYYY') AS LAST_COMMENT_DATE, PROJECT_NAME, PROJECT_ID
            FROM (
                  SELECT I.ISSUE_ID, I.ISSUE_NAME, I.LAST_COMMENT_DESCRIPTION, I.LAST_COMMENT_AUTHOR_FIRSTNAME,
                         I.LAST_COMMENT_AUTHOR_ID, I.LAST_COMMENT_DATE, I.PROJECT_NAME, I.PROJECT_ID
                    FROM (
                      SELECT ISS.ID AS ISSUE_ID, ISS.NAME AS ISSUE_NAME, ISS.STATUS AS STATUS, MAX(COM.ID) AS LAST_COMMENT_ID,
                             COM.DESCRIPTION AS LAST_COMMENT_DESCRIPTION, COM.CREATED_BY AS LAST_COMMENT_AUTHOR_ID, COM.CREATED_AT AS LAST_COMMENT_DATE,
                             RT.NAME AS LAST_COMMENT_AUTHOR_FIRSTNAME, PJT.NAME AS PROJECT_NAME, PJT.ID AS PROJECT_ID
                        FROM TASK ISS
                   LEFT JOIN (
                         SELECT COMM.TASK, COMM.DESCRIPTION, COMM.CREATED_AT, COMM.CREATED_BY, COMM.ID
                           FROM (
                             SELECT TASK, MAX(CREATED_AT) AS CREATED_AT
                               FROM COMMENTS
                              WHERE CREATED_BY <> :userId
                           GROUP BY TASK
                           ORDER BY CREATED_AT DESC)C
                      LEFT JOIN COMMENTS COMM ON COMM.TASK = C.TASK AND COMM.CREATED_AT = C.CREATED_AT
                                 ) COM              ON ISS.ID = COM.TASK
                       LEFT JOIN ALLOCATION ALO     ON ISS.ID = ALO.TASK AND ALO.RESOURCE_T = :userId
                       LEFT JOIN RESOURCE_T RT      ON RT.ID = COM.CREATED_BY
                       LEFT JOIN PROJECT PJT        ON PJT.ID = ISS.PROJECT
                       LEFT JOIN TASK_STATUS TS     ON TS.ID = ISS.TASK_STATUS
                       LEFT JOIN TASK_PROPERTIES TP ON TP.ORGANIZATION = ISS.ORGANIZATION
                           WHERE ISS.IS_ISSUE = :isIssue
                                 AND ISS.ORGANIZATION = :organizationId
                                 AND TS.ID <> TP.FINAL_TASK_STATUS
                                 AND ISS.CREATED_BY = :userId
                                 AND ISS.STATUS <> :deleteLogic
                                 AND (COM.CREATED_AT > ALO.LAST_POST_VIEW  OR ALO.LAST_POST_VIEW IS NULL)
                                 AND COM.CREATED_BY <> :userId
                        GROUP BY ISS.ID, ISS.NAME, ISS.STATUS, COM.DESCRIPTION, COM.CREATED_BY, COM.CREATED_AT, RT.NAME, PJT.NAME, PJT.ID) I
                WHERE 1=1
                      $conditions
                GROUP BY I.ISSUE_ID, I.ISSUE_NAME, I.LAST_COMMENT_DESCRIPTION, I.LAST_COMMENT_AUTHOR_ID, I.LAST_COMMENT_AUTHOR_FIRSTNAME, I.LAST_COMMENT_DATE,
                         I.PROJECT_NAME, I.PROJECT_ID)
SQL;

		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId,
			'deleteLogic' => LogicalDeleted::STATUS_LOGICAL_DELETED,
			'isIssue' => Entities\Task::IS_ISSUE
		);
		$query = Utilities::getPaginatedQuery($query, $limit, $offset);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function findMyProductsIssuesUnread($organizationId,$userId, $filter, $limit, $offset) {
	    $filter ? $conditions = Utilities::createQueryFiltered($filter) : $conditions="";
		$query = <<<SQL
    		SELECT C.DESCRIPTION AS LAST_COMMENT_DESCRIPTION, C.ID AS LAST_COMMENT_ID, C.CREATED_BY AS LAST_COMMENT_AUTHOR_ID,
    		       MIS.ISSUE_ID, MIS.ISSUE_NAME, MIS.PROJECT_NAME, MIS.PROJECT_ID, TO_CHAR(C.CREATED_AT, 'DD/MM/YYYY') AS LAST_COMMENT_DATE,
                   substr(MIS.LAST_COMMENT_AUTHOR_FULLNAME,1,instr(MIS.LAST_COMMENT_AUTHOR_FULLNAME,' ',1,1)-1) AS LAST_COMMENT_AUTHOR_FIRSTNAME
              FROM (
                    SELECT I.ISSUE_ID, I.ISSUE_NAME, MAX(I.LAST_COMMENT_ID) AS LAST_COMMENT_ID, RT.NAME AS LAST_COMMENT_AUTHOR_FULLNAME, I.PROJECT_NAME, I.PROJECT_ID
                     FROM (
                            SELECT  ISS.ID AS ISSUE_ID, ISS.NAME AS ISSUE_NAME, MAX(COM.ID) AS LAST_COMMENT_ID,
                                    PJT.NAME AS PROJECT_NAME, PJT.ID AS PROJECT_ID, COM.CREATED_BY AS LAST_COMMENT_AUTHOR_ID
                        	  FROM  TASK ISS
                        INNER JOIN 	TASK_STATUS TS 	   ON TS.ID = ISS.TASK_STATUS
                        INNER JOIN 	TASK_TYPE TT 	   ON TT.ID = TS.TASK_TYPE
                         LEFT JOIN  CLASSIFICATION C   ON C.ID = ISS.CLASSIFICATION
                         LEFT JOIN  ALLOCATION RA      ON (RA.TASK = ISS.ID AND RA.TYPE = :typeReporter)
                         LEFT JOIN  (
                                 SELECT COMM.TASK, COMM.DESCRIPTION, COMM.CREATED_AT, COMM.CREATED_BY, COMM.ID
                                   FROM (
                                     SELECT TASK, MAX(CREATED_AT) AS CREATED_AT
                                       FROM COMMENTS
                                       WHERE CREATED_BY <> :userId
                                   GROUP BY TASK
                                   ORDER BY CREATED_AT DESC)C
                              LEFT JOIN COMMENTS COMM ON COMM.TASK = C.TASK AND COMM.CREATED_AT = C.CREATED_AT
                                    ) COM              ON (ISS.ID = COM.TASK)
                         LEFT JOIN  ALLOCATION ALO     ON (ALO.TASK = ISS.ID AND ALO.RESOURCE_T = :userId)
                         LEFT JOIN  TASK_PROPERTIES TP ON TP.ORGANIZATION = ISS.ORGANIZATION
                         LEFT JOIN  PROJECT PJT        ON PJT.ID = ISS.PROJECT
                        	 WHERE  ISS.ORGANIZATION = :organizationId
                        			AND ISS.IS_ISSUE = :isIssue
                        			AND ISS.STATUS <> :deleteLogic
                        		    AND ISS.CREATED_BY <> :userId
                        		    AND TS.ID <> TP.FINAL_TASK_STATUS
                        			AND ISS.PROJECT IN   (SELECT P.ID
                                                            FROM PROJECT P
                                                      INNER JOIN RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
                                                           WHERE RP.RESOURCE_C = :userId
                                                                 AND P.STATUS <> :deleteLogic
                                                                 AND RP.TYPE = :managerType
                                                                 AND P.PRODUCT_TYPE NOT IN (:typeProductManagement))
                                    AND (COM.CREATED_AT > ALO.LAST_POST_VIEW  OR ALO.LAST_POST_VIEW IS NULL)
                          GROUP BY  ISS.ID, ISS.NAME, COM.DESCRIPTION, COM.CREATED_AT, COM.CREATED_BY, PJT.NAME, PJT.ID) I
                         LEFT JOIN  RESOURCE_T RT ON RT.ID = I.LAST_COMMENT_AUTHOR_ID
                             WHERE  1 = 1
                                    $conditions
                          GROUP BY  I.ISSUE_ID, I.ISSUE_NAME, I.PROJECT_NAME, I.PROJECT_ID, RT.NAME ) MIS
                         LEFT JOIN  COMMENTS C ON C.ID = MIS.LAST_COMMENT_ID
SQL;
        $params = array(
			'organizationId' => $organizationId,
			'userId' => $userId,
			'deleteLogic' => LogicalDeleted::STATUS_LOGICAL_DELETED,
			'isIssue' => Entities\Task::IS_ISSUE,
			'typeReporter' => Entities\Allocation::TYPE_REPORTER,
			'typeProductManagement' => Entities\ProjectType::TYPE_PRODUCT_CONTROL_MANAGEMENT,
			'managerType' => Entities\ResourceProject::MANAGER
		);
		$query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function totalCommentsUnread($organizationId, $userId) {
        $query = <<<SQL
           SELECT COUNT (*) AS VALUE
           FROM (
                SELECT  I.ID
                  FROM  TASK I
            INNER JOIN 	TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
            INNER JOIN 	TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
             LEFT JOIN  COMMENTS COM ON I.ID=COM.TASK
             LEFT JOIN  ALLOCATION ALO ON (I.ID=ALO.TASK AND ALO.RESOURCE_T = :userId)
             LEFT JOIN  PROJECT P ON P.ID = I.PROJECT
             LEFT JOIN  TASK_PROPERTIES TP ON TP.ORGANIZATION = I.ORGANIZATION
                 WHERE  P.ORGANIZATION = :organizationId
                        AND I.TASK_STATUS <> TP.FINAL_TASK_STATUS
                        AND I.STATUS <> :deleteLogic
                        AND I.IS_ISSUE = :isIssue
                        AND COM.CREATED_BY <> :userId
                        AND ((COM.CREATED_AT > ALO.LAST_POST_VIEW OR ALO.LAST_POST_VIEW IS NULL)
                              AND ((I.PROJECT IN (SELECT    P.ID
                                                    FROM    PROJECT P
                                                            INNER JOIN  RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
                                                   WHERE    RP.RESOURCE_C = :userId
                                                            AND   P.STATUS <> :deleteLogic
                                                            AND   RP.TYPE = :managerType
                                                            AND   P.PRODUCT_TYPE NOT IN (:typeProductManagement)
                                                  )) OR I.CREATED_BY = :userId))
                GROUP   BY I.ID)
SQL;
        $params = array(
			'organizationId' => $organizationId,
			'userId' => $userId,
			'isIssue' => Entities\Task::IS_ISSUE,
			'deleteLogic' => LogicalDeleted::STATUS_LOGICAL_DELETED,
			'typeProductManagement' => Entities\ProjectType::TYPE_PRODUCT_CONTROL_MANAGEMENT,
			'managerType' => Entities\ResourceProject::MANAGER
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function findProductsByManager($organizationId, $userId) {
        $query = <<<SQL
            SELECT  P.ID, P.NAME, P.ID AS BOARD_ID ,B.ID AS PROJECT_ISSUE_ID, P.BEGIN_DATE, P.END_DATE,
                    TS.LABEL AS PROJECT_STATUS_LABEL, TS.ID AS PROJECT_STATUS_ID,RES.NAME AS CREATED_BY,P.NOTIFY_ISSUE_OPENING,P.PRODUCT_TYPE,
                    TO_CHAR(P.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT
            FROM	PROJECT P
                    INNER JOIN  RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
                    INNER JOIN	PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID
                    INNER JOIN	PROJECT B ON B.PARENT = P.ID AND B.STATUS <> 'D'
                    INNER JOIN	PROJECT_TYPE BT ON BT.ID = B.PROJECT_TYPE AND BT.EXTERNALTYPE = 'I'
                    INNER JOIN TASK_STATUS TS ON TS.ID = P.PROJECT_STATUS
                    LEFT JOIN RESOURCE_T RES ON(RES.ID = P.CREATED_BY )
            WHERE   RP.RESOURCE_C = :userId
                    AND   P.STATUS <> 'D'
                    AND   RP.TYPE = 'M'
                    AND   P.ORGANIZATION = :organizationId
SQL;
        $params = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getAllIssues($organizationId, $userId, $conditions, $filterComments = '', $filterRelevance = '', $filterTaskStatus = '', $filterProductVersion = '') {
		$query = <<<SQL
			SELECT *
			 FROM (
			  SELECT I.ID as "id", I.ID||' - '||I.NAME as "label",
                     DECODE(I.CURRENT_TASK_STATUS_IN,NULL,NULL,(TRUNC(sysdate)-TRUNC(I.CURRENT_TASK_STATUS_IN))||' dia(s)') AS "lastUpdate", TO_CHAR(I.ESTIMATED_END_DATE, 'DD/MM/YYYY') as "due-date1",
			         I.DESCRIPTION as "description", './images/user.jpg' as "photo", TT.ID AS FRAMING, C.LABEL,
                     (SELECT COUNT(*) FROM ALLOCATION A WHERE A.IS_RELEVANT = 'Y' AND A.TASK = I.ID) AS RELEVANCE,
					 TT.ORDER_TO_VIEW as "column", TT.LABEL as "columnLabel", TT.ORDER_TO_FRAME as ORDER_TO_FRAME, PRI.COLOR,RA.RESOURCE_T AS REPORTER, C.LABEL as "category",
                     DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
                     DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV,
                     DECODE(I.DESCRIPTION, null, 'N', 'Y') AS "kanbanItemStatus"
				FROM TASK I
		  INNER JOIN TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
		  INNER JOIN TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
		   LEFT JOIN PRIORITY PRI ON (I.PRIORITY = PRI.ID)
		   LEFT JOIN CLASSIFICATION C ON C.ID = I.CLASSIFICATION
		   LEFT JOIN ALLOCATION RA ON RA.TASK = I.ID  AND RA.TYPE = 'R'
		   LEFT JOIN ALLOCATION SA ON SA.TASK = I.ID  AND SA.TYPE = 'S'
		   LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
		   LEFT JOIN LABEL L ON TL.LABEL = L.ID
		   LEFT JOIN ALLOCATION FA ON FA.TASK = I.ID  AND FA.TYPE = 'F'
		   LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
           LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
           LEFT JOIN PROJECT P ON (P.ID = I.PROJECT)
           LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = I.PRODUCT_VERSION_ID
           LEFT JOIN   PRODUCT_VERSION PROV ON PROV.ID = I.PRODUCT_VERSION_ID_DEV
		       WHERE I.ORGANIZATION = :organizationId
			         AND I.IS_ISSUE = 'Y'
				     AND I.STATUS <> 'D'
                     $filterComments
                     $conditions
            GROUP BY I.ID,I.NAME,I.ESTIMATED_END_DATE,I.DESCRIPTION,TT.ID,TT.LABEL,TT.ORDER_TO_VIEW, TT.ORDER_TO_FRAME,PRI.COLOR,RA.RESOURCE_T, TT.ID, PRV.ID, PRV.VERSION, PRV.REVISION, PRV.BUILD, PROV.ID, PROV.VERSION, PROV.REVISION, PROV.BUILD, C.LABEL, I.CURRENT_TASK_STATUS_IN
            ORDER BY I.ID DESC )I
        WHERE 1 = 1
              $filterRelevance
              $filterTaskStatus
              $filterProductVersion
SQL;

		$params = array(
			'organizationId' => $organizationId,
            'userId' => $userId
        );
        $issues = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach ($issues as &$issue){
            $issue['kanbanItemStatus'] = $issue['kanbanItemStatus'] == 'Y' ? true : false;
            $issue['kanbanCaretStatus'] = false;
        }
        return $issues;
	}
    public function getAllIssuesFromFilter($organizationId,$conditions = '') {
		$query = <<<SQL
			SELECT ID,NAME FROM(SELECT TAS.ID,TAS.NAME,PRO.NAME AS PROJECT
                     FROM TASK TAS
                     LEFT JOIN PROJECT PRO  ON(PRO.ID = TAS.PROJECT)
                    WHERE TAS.ORGANIZATION = :organizationId
                      AND TAS.STATUS <> 'D'
                      AND TAS.IS_ISSUE = 'Y')
                      WHERE 1=1
                      $conditions
                      GROUP BY ID,NAME
                      ORDER BY ID
SQL;

		$params = array(
			'organizationId' => $organizationId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

	}

    public function getFistStructureIdByManager($managerId){
		$sql = <<<SQL
			SELECT ST.ID
		  	FROM 		STRUCTURE ST
		  	INNER JOIN 	PERMISSION PE ON (PE.STRUCTURE = ST.ID)
		 	WHERE 	PE.RESOURCE_T = :manager
		   	AND 	ST.STATUS = 'A'
		   	AND 	PE.TYPE = 'M'
SQL;
		$params = array(
			'manager' => $managerId
		);

        return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
    }

    public function getAllStructureLessId($structuresId) {
        return Utilities::getAllStructureLessId($structuresId);
	}

    public function getAllStructureByManagerInString($userId)
	{
		$structure = $this->getFistStructureIdByManager($userId);
		$allStructure = $this->getAllStructureLessId($structure);
        $allStructuresArray = Utilities::getArrayFromArrayAssociative($allStructure);
        $allStructuresString = Utilities::getStringForArrayWithDefault($allStructuresArray,'0');
        return $allStructuresString;
	}

	public function getFistProjectIdByManager($managerId){
		$sql = <<<SQL
			SELECT P.ID
			FROM		PROJECT P
			INNER JOIN  RESOURCE_PROJECT RP ON (RP.PROJECT = P.ID)
			WHERE RP.RESOURCE_C = :manager
			AND   P.STATUS <> 'D'
			AND   RP.TYPE = 'M'
SQL;
		$params = array(
			'manager' => $managerId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
	}

	public function findMyKanbanIssues($organizationId, $userId, $filterComments = '', $conditions = '')
	{
		$query = <<<SQL
		SELECT ISSUE_ID, "id", "label", "lastUpdate", "due-date1", "description", "category", "photo", "column", "columnLabel", ORDER_TO_FRAME, ALLOW_CLOSE_ISSUE,
          PRODUCT, NAME, CLASSIFICATION, TASK_STATUS, COLOR, SPONSOR, REPORTER, CONTROL_TASK_TYPE, FRAMING, TT_LABEL, "kanbanItemStatus", VERSION, VERSION_DEV
        FROM(
            SELECT * FROM (
                SELECT I.ID AS ISSUE_ID, I.ID as "id", I.ID||' - '||I.NAME as "label",
                        DECODE(I.CURRENT_TASK_STATUS_IN,NULL,NULL,(TRUNC(sysdate)-TRUNC(I.CURRENT_TASK_STATUS_IN))||' dia(s)') AS "lastUpdate", TO_CHAR(I.ESTIMATED_END_DATE, 'DD/MM/YYYY') as "due-date1", I.DESCRIPTION as "description", CLA.LABEL as "category", './images/user.jpg' as "photo",
                        TT.ORDER_TO_VIEW as "column", TT.LABEL as "columnLabel", TT.ORDER_TO_FRAME as ORDER_TO_FRAME,
                        TT.ALLOW_CLOSE_ISSUE as ALLOW_CLOSE_ISSUE, I.PROJECT  as PRODUCT,I.NAME,I.CLASSIFICATION,TS.ID AS TASK_STATUS,PRI.COLOR, AL.RESOURCE_T AS SPONSOR,A.RESOURCE_T as REPORTER,
                        TT.CONTROL AS CONTROL_TASK_TYPE, TT.ID AS FRAMING, TT.LABEL AS TT_LABEL, TL.LABEL AS TL_LABEL, CLA.LABEL,
                        DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
                        DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV,
                        DECODE(I.DESCRIPTION, null, 'N', 'Y') AS "kanbanItemStatus"
                FROM 		TASK I
                INNER JOIN 	ALLOCATION A 	ON A.TASK = I.ID AND A.RESOURCE_T = :userId AND A.TYPE = 'R'
                INNER JOIN 	TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
                INNER JOIN 	TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
                INNER JOIN 	PROJECT P 		ON P.ID = I.PROJECT
                FULL JOIN 	ALLOCATION AL 	ON AL.TASK = I.ID AND AL.TYPE = 'S'
                LEFT JOIN PRIORITY PRI ON(I.PRIORITY = PRI.ID)
                LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
                LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
                LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
                LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = I.PRODUCT_VERSION_ID
                LEFT JOIN   PRODUCT_VERSION PROV ON PROV.ID = I.PRODUCT_VERSION_ID_DEV
                LEFT JOIN CLASSIFICATION CLA ON I.CLASSIFICATION = CLA.ID
                WHERE 	I.ORGANIZATION = :organizationId
                AND 	I.IS_ISSUE = 'Y'
                AND 	I.STATUS <> 'D'
                $filterComments
                GROUP BY I.ID , I.NAME , I.ESTIMATED_END_DATE, 'DD/MM/YYYY', I.DESCRIPTION,
                        TT.ID , TT.LABEL, ORDER_TO_FRAME, TT.ALLOW_CLOSE_ISSUE, I.PROJECT  ,I.NAME,I.CLASSIFICATION,TS.ID ,PRI.COLOR,
                        AL.RESOURCE_T,A.RESOURCE_T, TT.CONTROL, TT.ORDER_TO_VIEW, TL.LABEL, PRV.ID, PRV.VERSION, PRV.REVISION, PRV.BUILD,
                        PROV.ID, PROV.VERSION, PROV.REVISION, PROV.BUILD, CLA.LABEL, I.CURRENT_TASK_STATUS_IN )
        )
        WHERE 1=1
              $conditions
        GROUP BY ISSUE_ID, "id", "label", "lastUpdate", "due-date1", "description", "photo", "column", "columnLabel", "category", ORDER_TO_FRAME, ALLOW_CLOSE_ISSUE,
          PRODUCT, NAME, CLASSIFICATION, TASK_STATUS, COLOR, SPONSOR, REPORTER, CONTROL_TASK_TYPE, FRAMING, TT_LABEL, "kanbanItemStatus", VERSION, VERSION_DEV
SQL;

        $query .= " ORDER BY ISSUE_ID DESC";
		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);
		$issues = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach ($issues as &$issue){
            $issue['kanbanItemStatus'] = $issue['kanbanItemStatus'] == 'Y' ? true : false;
            $issue['kanbanCaretStatus'] = false;
        }
        return $issues;
	}

    public function validateFinishSpecificationTask($taskid)
    {
        $query = <<<SQL
            SELECT TS.ID
            FROM TASK T
                JOIN TASK_RELATIONSHIP  TAR     ON(TAR.TASK_RELATIONSHIP = T.ID AND TAR.TYPE = :issueType)
                JOIN TASK               TI      ON(TAR.TASK = TI.ID)
                JOIN TASK_RELATIONSHIP  TARI    ON(TARI.TASK = TI.ID AND TARI.TYPE = :issueType)
                JOIN TASK               TS      ON(TARI.TASK_RELATIONSHIP = TS.ID)
                JOIN CLASSIFICATION     CLA     ON(TS.CLASSIFICATION = CLA.ID)
            WHERE T.ID = :id
                AND (CLA.CONTROL NOT IN (:taskClassSpecification,:taskClassAccept) OR CLA.CONTROL IS NULL)
                AND TS.STATUS = :active
                AND TI.STATUS = :active
                AND TS.PERCENT_COMPLETED < 100
SQL;

        $params = array(
            'taskClassSpecification' => Entities\Classification::CONTROL_SPECIFICATION,
            'taskClassAccept' => Entities\Classification::CONTROL_CLIENT_ACCEPT,
            'active' => Entities\Task::ACTIVE,
            'usedInFraming' => Entities\Task::USED_IN_FRAMING,
            'issueType' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'id' => $taskid
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function validateSpecificationRequirements($organizationId)
    {
        $query = <<<SQL
              SELECT CTY.ID, CTY.ATTACHMENT_REQUIRED,CLA.ID AS CLASSIFICATION_ID,TAS.ID AS TASK_STATUS_ID,CTY.COMMENT_DESCRIPTION
                FROM COMMENT_TYPE CTY
           LEFT JOIN CLASSIFICATION CLA ON (CLA.CONTROL = :controlClientAccept)
           LEFT JOIN TASK_STATUS TAS ON (TAS.CONTROL = :taskStatusControlClientAccept)
               WHERE CTY.CONTROL = :typeSpecification
                 AND CTY.ORGANIZATION_ID = :organizationId
SQL;
        $params = array(
            'organizationId' => $organizationId,
            'controlClientAccept' => Entities\Classification::CONTROL_CLIENT_ACCEPT,
            'typeSpecification' => Entities\CommentType::TYPE_SPECIFICATION,
            'taskStatusControlClientAccept' => Entities\TaskStatus::TASK_STATUS_CONTROL_CLIENT_ACCEPT,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

	public function findFollowedKanbanIssues($organizationId, $userId,$filterComments = '', $conditions = '')
	{
		$query = <<<SQL
		 SELECT CLASSIFICATION, COLOR, ISSUE_ID, NAME, ORDER_TO_FRAME, PRODUCT, REPORTER, SPONSOR, TASK_STATUS, FRAMING,
		 "column", "columnLabel", "description", "due-date1", "id", "label", "lastUpdate", "photo","category", "columnType"
		 FROM(
		 SELECT * FROM (
			SELECT 	I.ID AS ISSUE_ID, I.ID as "id", I.ID||' - '||I.NAME as "label",
                    DECODE(I.CURRENT_TASK_STATUS_IN,NULL,NULL,(TRUNC(sysdate)-TRUNC(I.CURRENT_TASK_STATUS_IN))||' dia(s)') AS "lastUpdate", TO_CHAR(I.ESTIMATED_END_DATE, 'DD/MM/YYYY') as "due-date1", I.DESCRIPTION as "description", CLA.LABEL as "category",'./images/user.jpg' as "photo",
					TT.ORDER_TO_VIEW as "column", TT.LABEL as "columnLabel", TT.CONTROL as "columnType",TT.ORDER_TO_FRAME as ORDER_TO_FRAME, TT.ID AS FRAMING,
					P.ID as PRODUCT,I.NAME,I.CLASSIFICATION,AR.RESOURCE_T AS REPORTER,PRI.COLOR,I.TASK_STATUS, AL.RESOURCE_T AS SPONSOR, TL.LABEL AS TL_LABEL,
                        DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
                        DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV
			FROM 		TASK I
			INNER JOIN 	ALLOCATION A 	ON A.TASK = I.ID AND A.RESOURCE_T = :userId AND A.TYPE LIKE 'F'
			INNER JOIN 	TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
			INNER JOIN 	TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
			INNER JOIN 	PROJECT P 		ON P.ID = I.PROJECT
			FULL JOIN	ALLOCATION AL 	ON AL.TASK = I.ID AND AL.TYPE = 'S'
			LEFT  JOIN 	ALLOCATION AR 	ON (AR.TASK = I.ID AND AR.TYPE LIKE 'R')
			LEFT JOIN PRIORITY PRI ON(I.PRIORITY = PRI.ID)
            LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
            LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
			LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
            LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = I.PRODUCT_VERSION_ID
            LEFT JOIN CLASSIFICATION CLA ON I.CLASSIFICATION = CLA.ID
                LEFT JOIN   PRODUCT_VERSION PROV ON PROV.ID = I.PRODUCT_VERSION_ID_DEV
			WHERE 	I.ORGANIZATION = :organizationId
			AND 	I.IS_ISSUE = 'Y'
			AND 	I.STATUS <> 'D'
			$filterComments
			GROUP BY(I.ID, I.NAME, I.ESTIMATED_END_DATE, I.DESCRIPTION, TT.ID, TT.LABEL, TT.CONTROL, ORDER_TO_FRAME, P.ID, I.NAME, I.CLASSIFICATION, AR.RESOURCE_T, PRI.COLOR, I.TASK_STATUS, AL.RESOURCE_T, TT.ORDER_TO_VIEW, TL.LABEL, PRV.ID, PRV.VERSION, PRV.REVISION, PRV.BUILD, PROV.ID, PROV.VERSION, PROV.REVISION, PROV.BUILD, CLA.LABEL, I.CURRENT_TASK_STATUS_IN)
      		ORDER BY I.ID DESC
			)
			WHERE 1=1
			$conditions
			)
			GROUP BY ISSUE_ID, CLASSIFICATION, COLOR, NAME, ORDER_TO_FRAME, PRODUCT, REPORTER, SPONSOR, TASK_STATUS, FRAMING,
		  "column", "columnLabel", "description", "due-date1", "id", "label", "lastUpdate", "photo", "category", "columnType"
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getTasksWithSameResource($taskIds) {
		$query = <<<SQL
		SELECT RTRIM(XMLAGG(XMLELEMENT(E,P.ID,',').EXTRACT('//text()') ORDER BY P.ID).GETCLOBVAL(),',') AS TASKS
		  FROM (SELECT TAS.ID,PRO.PROJECT_TYPE,TAS.NAME,
                       LISTAGG(ALO.RESOURCE_T, ',') WITHIN GROUP (ORDER BY ALO.RESOURCE_T)AS RESOURCES
                  FROM TASK TAS
                  JOIN ALLOCATION ALO ON (ALO.TASK = TAS.ID)
                  JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
                 WHERE TAS.ID IN (:taskIds)
               GROUP BY TAS.ID,PRO.PROJECT_TYPE,TAS.NAME)P
       GROUP BY P.RESOURCES,P.PROJECT_TYPE,P.NAME
SQL;
		$params = array(
			'taskIds' => $taskIds,
		);
        $paramTypes = array(
            'taskIds' => Connection::PARAM_STR_ARRAY
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
	}

	public function getRelatedMergeRelationshipIdByGeneratedTaskIds($taskIds) {
		$query = <<<SQL
		SELECT DISTINCT ID
          FROM TASK_RELATIONSHIP TA,
          (SELECT TASK
             FROM TASK_RELATIONSHIP
            WHERE TASK_RELATIONSHIP IN (SELECT TO_NUMBER(P.TASKS) AS TASK
                                          FROM (SELECT RTRIM(XMLAGG(XMLELEMENT(E,P.ID,',').EXTRACT('//text()') ORDER BY P.ID).GETCLOBVAL(),',') AS TASKS
                                                  FROM (SELECT TAS.ID,PRO.PROJECT_TYPE,TAS.NAME,
                                                               LISTAGG(ALO.RESOURCE_T, ',') WITHIN GROUP (ORDER BY ALO.RESOURCE_T)AS RESOURCES
                                                          FROM TASK TAS
                                                          JOIN ALLOCATION ALO ON (ALO.TASK = TAS.ID)
                                                          JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
                                                         WHERE TAS.ID IN (:TASK_IDS)
                                                         GROUP BY TAS.ID,PRO.PROJECT_TYPE,TAS.NAME)P
                                                   GROUP BY P.RESOURCES,P.PROJECT_TYPE,P.NAME) P
                                          WHERE INSTR(P.TASKS, ',') = 0)
              AND TYPE = :ERROR_MR_TYPE) TG
         WHERE (TA.TASK = TG.TASK OR TA.TASK_RELATIONSHIP = TG.TASK)
           AND TA.TYPE = :RELATED_TASK_TYPE
SQL;
		$params = array(
			'TASK_IDS' => $taskIds,
            'ERROR_MR_TYPE' => Entities\TaskRelationship::ERROR_MR_TYPE,
            'RELATED_TASK_TYPE' => Entities\TaskRelationship::RELATED_TASK_TYPE
		);
        $paramTypes = array(
            'TASK_IDS' => Connection::PARAM_STR_ARRAY
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
	}

	public function getAllTasksByProjectOnDemand($organizationId,$filters = '',$filterLabel = '', $filterIssue = '',
                                                 $project = false,$statusResource = false) {
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $filterLabelIssue = '';
        $filterStatusResource = '';
        if($filterLabel) {
            $filterLabelIssue .= " AND TAS.ID IN (SELECT TRE.TASK_RELATIONSHIP
                                                    FROM TASK_LABEL TLA
                                               LEFT JOIN TASK_RELATIONSHIP TRE ON (TRE.TASK = TLA.TASK AND NVL(TRE.TYPE, 'I') = 'I')
                                                   WHERE TLA.LABEL IN($filterLabel))";
        }
        if($filterIssue) {
            $filterLabelIssue .= " AND TAS.ID IN (SELECT TASK_RELATIONSHIP
                                                    FROM TASK_RELATIONSHIP
                                                   WHERE TASK_RELATIONSHIP = TAS.ID
                                                     AND NVL(TYPE, 'I') = 'I'
                                                     AND TASK IN ($filterIssue))";
        }
        if($statusResource) {
            $filterLabelIssue .= " AND RES.STATUS = '$statusResource'";
        }
        $query = <<<SQL
           SELECT TASK_NAME,PROJECT_NAME||TASK_NAME AS ORDER_BY, NAME_RESOURCE,MODEL_ID,RESOURCE_T,
                (SELECT RTRIM(XMLAGG(XMLELEMENT(E,RP.EMAIL,',').EXTRACT('//text()') ORDER BY RP.ID).GETCLOBVAL(),',')
                     FROM RESOURCE_T RP
                    INNER JOIN ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                    WHERE TASK_ID = AL.TASK) AS EMAIL_RESOURCE,
                  ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE,DURATION,SUM(TOTAL_APPROPRIATION) AS TOTAL_APPROPRIATION,RESOURCE_T,
                  LABEL AS TASK_STATUS_LABEL,TIME_PLANNED,PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,TASK_ID,PROJECT_ID, PROJECT_NAME
                  FROM(SELECT  TASK_NAME,NAME_RESOURCE,ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE,DURATION,MODEL_ID,RESOURCE_T,
                              TOTAL_APPROPRIATION,LABEL,PERCENT_COMPLETED,TIME_PLANNED,TASK_ID,PROJECT_ID,EMAIL_RESOURCE, PROJECT_NAME, NULL AS TASK_STATUS_LABEL
                       FROM (SELECT NULL AS TASK_STATUS_LABEL, TAS.ID AS TASK_ID,TAS.NAME AS TASK_NAME,RES.NAME AS NAME_RESOURCE,TLA.LABEL AS TASK_LABEL,
                                  TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'YYYYMMDD') AS ORDER_FRONT, RES.EMAIL AS EMAIL_RESOURCE,
                                  TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                                  TO_CHAR(TAS.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE,
                                  TAS.DURATION,TAS.TOTAL_APPROPRIATION AS TOTAL_APPROPRIATION,TST.LABEL,TAS.PERCENT_COMPLETED,
                                  TAS.PROJECT_ID AS ID,TAS.PROJECT_ID, TAS.PROJECT_ID_FILTER, PRO.NAME AS PROJECT_NAME,TAS.STATUS,
                                  TST.ID AS TASK_STATUS, RES.ID AS RESOURCE_T,PER.STRUCTURE,
                                  DECODE(TK.ID, NULL, TAS.TIME_PLANNED, 0) AS TIME_PLANNED,
                                  (   SELECT PRM.ID
                                        FROM PROJECT PRO1
                                        JOIN PROJECT_RELATIONSHIP PRR ON (PRO1.ID = PRR.PROJECT AND PRR.TYPE = 'G')
                                        JOIN PROJECT PRM ON(PRM.ID = PRR.PROJECT_RELATIONSHIP)
                                        WHERE PRO1.ID = PRO.ID AND ROWNUM = 1) AS MODEL_ID,
                                  ALO.RESOURCE_T AS TASK_RESOURCE,PRE.RESOURCE_C AS MANAGER_RESOURCE,
                               TAS.ESTIMATED_BEGIN_DATE AS PERIOD,TAS.ESTIMATED_END_DATE AS PERIOD_END, TASK_CLASSIFICATION, TAS.CREATED_AT
                             FROM ( SELECT  TAS.ID AS TASK_ID,TAS.PROJECT AS PROJECT_ID_FILTER,TAS.ID,TAS.ORGANIZATION,TAS.NAME,TAS.ESTIMATED_BEGIN_DATE,TAS.ESTIMATED_END_DATE,
                                             TAS.PERCENT_COMPLETED,TAS.DURATION,TAS.TIME_PLANNED,TAS.PROJECT AS PROJECT_ID,TAS.STATUS, TAS.TASK_STATUS, TAS.CLASSIFICATION AS TASK_CLASSIFICATION,
                                             SUM(APR.TOTAL) AS TOTAL_APPROPRIATION, TAS.CREATED_AT
                                             FROM TASK TAS
                                                      LEFT JOIN APPROPRIATION APR ON(APR.TASK = TAS.ID)
                                                      WHERE TAS.ORGANIZATION = :organizationId
                                                      AND TAS.STATUS <> 'D'
                                                      AND NVL(TAS.IS_PARENT,'N') <> 'Y'
                                                      AND NVL(TAS.IS_ISSUE,'N') <> 'Y'
                                                      GROUP BY TAS.ID, TAS.ORGANIZATION, TAS.TIME_PLANNED, TAS.CALENDAR, TAS.PROJECT, TAS.NAME,
                                             TAS.ID,TAS.ORGANIZATION,TAS.NAME,TAS.ESTIMATED_BEGIN_DATE,TAS.ESTIMATED_END_DATE,
                                             TAS.PERCENT_COMPLETED,TAS.DURATION,TAS.PROJECT,TAS.STATUS, TAS.TASK_STATUS, TAS.CLASSIFICATION, TAS.CREATED_AT ) TAS
                        LEFT JOIN ALLOCATION ALO ON (ALO.TASK = TAS.ID)
                        LEFT JOIN RESOURCE_T RES ON(RES.ID = ALO.RESOURCE_T)
                        LEFT JOIN TASK_STATUS TST ON(TST.ID = ALO.TASK_STATUS)
                        LEFT JOIN PROJECT PRO ON(PRO.ID = TAS.PROJECT_ID)
                        LEFT JOIN RESOURCE_PROJECT PRE ON(PRO.ID = PRE.PROJECT)
                        LEFT JOIN TASK_LABEL TLA ON(TLA.TASK = TAS.ID)
                        LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
                        LEFT JOIN PERMISSION PER ON (PER.RESOURCE_T = RES.ID AND  NVL(PER.TYPE,'R') = 'R')
                        LEFT JOIN TASK_RELATIONSHIP TK ON (TK.TASK_RELATIONSHIP = TAS.ID AND TK.TYPE = 'A')
                            WHERE TAS.ORGANIZATION = :organizationId
                              $filterTypeProject
                              $filterLabelIssue)
                        WHERE 1=1
                          $filters
                          GROUP BY  TASK_NAME,ESTIMATED_BEGIN_DATE,NAME_RESOURCE, ESTIMATED_END_DATE,DURATION,LABEL,RESOURCE_T,
                          PERCENT_COMPLETED,TASK_ID,PROJECT_ID,TOTAL_APPROPRIATION,EMAIL_RESOURCE,TIME_PLANNED, PROJECT_NAME,MODEL_ID, CREATED_AT)
          GROUP BY TASK_NAME, ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE, DURATION, TOTAL_APPROPRIATION,TIME_PLANNED,
                    LABEL, PERCENT_COMPLETED, TASK_ID, PROJECT_ID, PROJECT_NAME,NAME_RESOURCE,MODEL_ID,RESOURCE_T
          ORDER BY ORDER_BY
SQL;
        $params = array(
			'organizationId' => $organizationId
        );
        return  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

	public function getStandardTaskTypes($organizationId) {
		$query = <<<SQL
			SELECT 	TT.ID as "column", TT.LABEL as "columnLabel"
			FROM 		TASK_TYPE TT
			WHERE 	TT.ORGANIZATION = :organizationId
			AND (TT.CONTROL = 'T' OR TT.CONTROL = 'I' OR TT.CONTROL = 'D')
SQL;

		$params = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getIssueFrameOrderBasedOnRelatedTasks($issueId){
        $query = <<<SQL
      SELECT MIN(TTYPE.ORDER_TO_FRAME) as ORDER_TO_FRAME
        FROM TASK_RELATIONSHIP TREL
        JOIN TASK T ON (T.ID = TREL.TASK_RELATIONSHIP AND T.USED_IN_FRAMING = :usedInFraming)
        JOIN TASK_STATUS TS ON TS.ID = T.TASK_STATUS
        JOIN TASK_TYPE TTYPE ON TTYPE.ID = TS.TASK_TYPE
       WHERE TREL.TASK = :issueId
         AND TREL.TYPE = :TYPE_ISSUE
SQL;
        $params = array(
            'issueId' => $issueId,
            'TYPE_ISSUE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'usedInFraming' => Entities\Task::USED_IN_FRAMING,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? null : $result[0]['ORDER_TO_FRAME'];
    }

    public function getLabelRelatedUserByTask($taskId,$userId, $meetingProjectIds){
        $query = <<<SQL
        SELECT LAB.ID
         FROM  LABEL LAB
         JOIN  TASK_LABEL TAL ON (TAL.LABEL = LAB.ID AND TAL.TASK = :taskId)
         WHERE LAB.PROJECT IN (:meetingProjectIds)
            OR LAB.RESOURCE_T = :userId
SQL;
        $params = array(
            'userId' => $userId,
            'taskId' => $taskId,
            'meetingProjectIds' => $meetingProjectIds
        );
        $paramTypes = array(
            'meetingProjectIds' => Connection::PARAM_STR_ARRAY,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

	/**
	 * Gets a dataset when you click on a task.
	 * @param $idTask
	 * @param $organization
	 * @return array
	 */
	public function getDataSetForEditTask($idTask,$organization) {
		$query = <<<SQL
			SELECT 	TA.ID, TA.NAME, TO_CHAR(PRO.END_DATE,'DD/MM/YYYY') AS END_PROJECT, TO_CHAR(PRO.BEGIN_DATE,'DD/MM/YYYY') AS BEGIN_PROJECT,
					TO_CHAR(TA.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY') AS "START", TAPR.NAME AS PRECURSOR_NAME, TP.TASK_RELATIONSHIP AS PRECURSOR,
        			TR.TASK AS TASK_GENERATOR, TAGE.NAME AS TASK_GENERATOR_NAME, TR.TYPE AS TYPE_RELATIONSHIP,
        			TO_CHAR(TA.ESTIMATED_END_DATE,'DD/MM/YYYY') AS "END", TAPA.NAME AS PARENT_NAME, TA.PERCENT_COMPLETED, TA.PARENT AS PARENT,
        			TA.PERCENT_COMPLETED as PERCENT, TA.STATUS, TO_CHAR(TA.DATE_EXCEPTION,'DD/MM/YYYY') AS DATE_EXCEPTION, TA.IS_PARENT,
        			TA.PREVIOUS_TIME/3600 as PREVIOUS_TIME, PT.ALLOWENDTASK, TA.URL_TO_TEST, TA.CLASSIFICATION AS TASK_CLASSIFICATION,
					CLA.LABEL AS TASK_CLASSIFICATION_LABEL,CLA.CONTROL AS TASK_CLASSIFICATION_CONTROL,'N' AS NOT_TASK_GENERATE,
					DECODE(TOO.TASK, NULL, 'N', 'Y') AS TASK_GENERATE_ORIGIN, TF.DESCRIPTION AS TASK_FEATURES, TF.ID AS TASK_FEATURES_ID, TA.ERROR_CLASSIFICATION_ID,
					EC.DESCRIPTION AS ERROR_CLASSIFICATION, TF.TIME_PLANNED AS FIRST_TIME_PLANNED,CLA.DIVIDE_DURATION,
					DECODE(PV.ID, NULL, 'Última versão', PV.VERSION || '.' || PV.REVISION) AS VERSION_DEV, TA.PRODUCT_VERSION_ID_DEV AS VERSION_DEV_ID,
                	(
                		SELECT rtrim(xmlagg(xmlelement(e,RP.ID,',').extract('//text()') ORDER BY RP.ID).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                		WHERE TA.ID = AL.TASK
                	) AS "RESOURCE",
                	(
                		SELECT rtrim(xmlagg(xmlelement(e,RP.NAME,',').extract('//text()') ORDER BY RP.ID).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                		WHERE TA.ID = AL.TASK
                	) AS "RESOURCE_NAME", TP.TYPE_PRECURSOR AS TYPE, TA.LATENCY, TA.DESCRIPTION, TS.ID AS TASK_STATUS, TS.LABEL AS TASK_STATUS_NAME,
                	TS.CONTROL AS TASK_STATUS_CONTROL,
					TA.DURATION AS TOTAL_TIME, TT.ID AS TASK_TYPE, TA.DATE_EXCEPTION_TYPE AS DATE_EXCEPTION_ABBREVIATION, TA.DATE_EXCEPTION_TYPE,
					PT.USETIMEPLANNING, TA.TIME_PLANNED, RES.NAME AS CREATED_BY, TO_CHAR(TA.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT,
					(SELECT COUNT(*)
					  FROM TASK_RELATIONSHIP TE
					 WHERE TE.TASK = TA.ID
					   AND TE.TYPE = 'E') AS GENERATED_TASKS_ERROR, TA.RESPONSE AS TASK_RESPONSE,PRO.PROJECT_TYPE,PRO.PROJECT_STATUS
            FROM 		TASK TA
            LEFT JOIN 	PROJECT PRO 			ON (TA.PROJECT = PRO.ID)
            LEFT JOIN 	PROJECT_TYPE PT 		ON (PT.ID = PRO.PROJECT_TYPE)
	        LEFT JOIN 	TASK_RELATIONSHIP TP 	ON (TA.ID = TP.TASK AND TP.TYPE = 'P')
	        LEFT JOIN 	TASK_RELATIONSHIP TR 	ON (TA.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
	        LEFT JOIN 	TASK_RELATIONSHIP TOO 	ON (TA.ID = TOO.TASK AND TOO.TYPE = 'O')
            LEFT JOIN 	TASK_STATUS TS 			ON (TS.ID = TA.TASK_STATUS)
            LEFT JOIN 	TASK_TYPE TT 			ON (TT.ID = TS.TASK_TYPE)
            LEFT JOIN 	TASK TAPR 				ON (TAPR.ID = TP.TASK_RELATIONSHIP)
            LEFT JOIN 	TASK TAGE 				ON (TAGE.ID = TR.TASK)
            LEFT JOIN 	TASK TAPA 				ON (TAPA.ID = TA.PARENT)
            LEFT JOIN 	RESOURCE_T RES			ON (RES.ID = TA.CREATED_BY)
            LEFT JOIN 	CLASSIFICATION CLA		ON (CLA.ID = TA.CLASSIFICATION)
            LEFT JOIN 	TASK_FEATURES TF		ON (TF.ID = TA.TASK_FEATURES)
            LEFT JOIN   TASK_RELATIONSHIP TRF   ON (TA.ID = TRF.TASK AND TRF.TYPE = 'F')
            LEFT JOIN   TASK TF                 ON TRF.TASK_RELATIONSHIP = TF.ID
            LEFT JOIN   ERROR_CLASSIFICATION EC ON EC.ID = TA.ERROR_CLASSIFICATION_ID
            LEFT JOIN   PRODUCT_VERSION PV ON PV.ID = TA.PRODUCT_VERSION_ID_DEV
            WHERE TA.ORGANIZATION = :organization
            AND TA.ID = :idTask
SQL;
		$params = array(
			'organization' => $organization,
			'idTask' => $idTask
		);
		$connection = $this->getEntityManager()->getConnection();
		$tasks = $connection->fetchAll($query, $params);
		foreach($tasks as &$task){
            $task['RESOURCE'] = Utilities::setSelectMultiInQuery($task['RESOURCE']);
			$task['TOTAL_TIME'] = Utilities::secondsToTime($task['TOTAL_TIME']);
			$task['LATENCY'] = Utilities::secondsToTime($task['LATENCY']);
			if(!empty($task['DATE_EXCEPTION_TYPE'])){
				$task['DATE_EXCEPTION_NAME'] = \Service\Calendar\Restriction\Strategy\RestrictionFactory::$ARRAY_RESTRICTION[$task['DATE_EXCEPTION_TYPE']];
			}
            if($this->findOneBy(array('parent' => $idTask))){
                $task['IS_UPPER'] = true;
			} else {
				$task['IS_UPPER'] = false;
			}
            $issueRelationship =  $this->getAllClassificationForIssueRelationship($idTask);
            if(count($issueRelationship) == 1){
                $task['TASK_CLASSIFICATION_ID_ISSUE'] = $issueRelationship[0]['CLASSIFICATION'];
                $task['TASK_CLASSIFICATION_LABEL_ISSUE'] = $issueRelationship[0]['LABEL'];
                $task['TASK_CLASSIFICATION_ISSUE'] = true;
            }else if($task['TASK_CLASSIFICATION_CONTROL']){
                $task['TASK_CLASSIFICATION_ISSUE'] = true;
            }
		}

		return $tasks;
	}
    public function getAllClassificationForIssueRelationship($idTask) {
		$query = <<<SQL
			SELECT T.CLASSIFICATION,C.LABEL
              FROM TASK_RELATIONSHIP TR
              JOIN TASK T ON(T.ID = TR.TASK)
              JOIN CLASSIFICATION C ON(C.ID = T.CLASSIFICATION )
             WHERE TR.TASK_RELATIONSHIP = :idTask
               AND TR.TYPE = 'I'
          GROUP BY T.CLASSIFICATION,C.LABEL
SQL;
		$params = array(
			'idTask' => $idTask
		);
		$connection = $this->getEntityManager()->getConnection();
		return $connection->fetchAll($query, $params);
	}

    public function getPercentCompletedProject($projectId) {
		$query = <<<SQL
		SELECT
                    TRUNC(SUM(TAS.PERCENT_COMPLETED_TIME_PLANNED)*100/SUM(TAS.TIME_PLANNED)) AS PERCENT_COMPLETED
              FROM PROJECT PRO
              LEFT JOIN (
                    SELECT SUM(
                                    DECODE(TAS.PERCENT_COMPLETED,
                                                                0,0,
                                                                TAS.TIME_PLANNED*TAS.PERCENT_COMPLETED/100
                                           )
                                    )
                                AS PERCENT_COMPLETED_TIME_PLANNED,
                    SUM(TAS.TIME_PLANNED) AS TIME_PLANNED,
                    TAS.PROJECT

                    FROM TASK TAS
		       LEFT JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION )
               LEFT JOIN TASK_STATUS TST ON (TAS.TASK_STATUS = TST.ID AND TST.CONTROL = 'B')
                   WHERE TAS.TIME_PLANNED <> 0
                    AND TAS.TIME_PLANNED IS NOT NULL
                    AND TAS.IS_PARENT = 'N'
                    AND TAS.IS_ISSUE = 'N'
                    AND TAS.STATUS = 'A'
                    AND TST.CONTROL IS NULL
                    AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL)
                    GROUP BY TAS.PROJECT
              )TAS ON (TAS.PROJECT = PRO.ID)
          LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
              WHERE PRO.STATUS = 'A'
                AND PT.ALLOWENDTASK = 'Y'
                AND PT.EXTERNALTYPE NOT IN ('M','O','L')
              START WITH PRO.ID = :project
            CONNECT BY PRIOR PRO.ID = PRO.PARENT
SQL;
		$params = array(
			'project' => $projectId
		);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        if($result && $result[0] && $result[0]['PERCENT_COMPLETED']){
            $result = $result[0]['PERCENT_COMPLETED'];
        }else{
            $result = 0;
        }
        return $result;
	}


    public function getPercentCompletedTask($taskId) {
        $query = <<<SQL
          SELECT TRUNC(SUM(
                        DECODE(TAS.PERCENT_COMPLETED,
                                                    0,0,
                                                    TAS.TIME_PLANNED*TAS.PERCENT_COMPLETED/100
                               )
                        )*100/SUM(TAS.TIME_PLANNED)
                    ) AS PERCENT_COMPLETED
            FROM TASK TAS
	   LEFT JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION )
       LEFT JOIN TASK_STATUS TST ON (TAS.TASK_STATUS = TST.ID AND TST.CONTROL = 'B')
           WHERE TAS.TIME_PLANNED <> 0
             AND TAS.TIME_PLANNED IS NOT NULL
             AND TAS.IS_PARENT = 'N'
             AND TAS.IS_ISSUE = 'N'
             AND TAS.STATUS = 'A'
             AND TST.CONTROL IS NULL
             AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL)
            START WITH TAS.PARENT = :task
             CONNECT BY PRIOR TAS.ID = TAS.PARENT
SQL;
        $params = array(
            'task' => $taskId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        if($result && $result[0] && $result[0]['PERCENT_COMPLETED']){
            $result = $result[0]['PERCENT_COMPLETED'];
        }else{
            $result = 0;
        }
        return $result;
    }

    public function getInfoTaskInTaskTracking($idTask,$organization,$resource = null) {

        $resourceFilter = '';
        if($resource){
            $resourceFilter = " AND RP.ID = $resource";
        }
		$query = <<<SQL
			SELECT 	TA.ID AS TASK_ID,TA.ID, TA.NAME,TA.NAME AS TASK_NAME, TO_CHAR(PRO.END_DATE,'YYYY-MM-DD') AS END_PROJECT, TO_CHAR(PRO.BEGIN_DATE,'YYYY-MM-DD') AS BEGIN_PROJECT,
					TO_CHAR(TA.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY') AS "START", TAPR.NAME AS PRECURSOR_NAME, TP.TASK_RELATIONSHIP AS PRECURSOR,
        			TR.TASK AS TASK_GENERATOR, TAGE.NAME AS TASK_GENERATOR_NAME,
        			TR.TYPE AS TYPE_RELATIONSHIP,PRO.ID AS PROJECT_ID,
        			TO_CHAR(TA.ESTIMATED_END_DATE,'DD/MM/YYYY') AS "END", TAPA.NAME AS PARENT_NAME, TA.PERCENT_COMPLETED, TA.PARENT AS PARENT,
        			TA.PERCENT_COMPLETED as PERCENT,(SELECT C.LABEL FROM CLASSIFICATION C WHERE C.ID = TA.CLASSIFICATION) AS TASK_CLASSIFICATION_LABEL,
        			(CASE WHEN TA.STATUS = 'A' THEN 'Ativa' ELSE  'Inativa' END ) AS STATUS,
        			TO_CHAR(TA.DATE_EXCEPTION,'DD/MM/YYYY') AS DATE_EXCEPTION, TA.IS_PARENT,PRO.ID AS PROJECT,PRO.NAME AS PROJECT_NAME,
        			TA.PREVIOUS_TIME/3600 as PREVIOUS_TIME,TA.STATUS AS STATUS_CHECK_BOX,TA.RESPONSE AS TASK_RESPONSE,
        			TA.ERROR_CLASSIFICATION_ID,	EC.DESCRIPTION AS ERROR_CLASSIFICATION,
                    (
                		SELECT rtrim(xmlagg(xmlelement(e,RP.ID,',').extract('//text()') ORDER BY RP.ID).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                		WHERE TA.ID = AL.TASK
                		$resourceFilter
                	) AS "RESOURCE",
                    (
                		SELECT rtrim(xmlagg(xmlelement(e,RP.NAME,',').extract('//text()') ORDER BY RP.NAME).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                		WHERE TA.ID = AL.TASK
                		$resourceFilter
                	) AS "NAME_RESOURCE",
                    (
                		SELECT rtrim(xmlagg(xmlelement(e,TS.LABEL,',').extract('//text()') ORDER BY RP.NAME).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL   ON (RP.ID = AL.RESOURCE_T)
                		LEFT JOIN 	TASK_STATUS TS 	ON (TS.ID = AL.TASK_STATUS)
                		WHERE TA.ID = AL.TASK
                		$resourceFilter
                	) AS TASK_STATUS_ALLOCATION_NAME,
                    (
                           SELECT   rtrim(xmlagg(xmlelement(e,tla.label,',').extract('//text()') ORDER BY tla.id).getclobval(),',')
                           FROM     task_label tla
                           WHERE    ta.id = tla.task) AS TASK_LABEL,
                           TP.TYPE_PRECURSOR AS TYPE, TA.LATENCY, TA.DESCRIPTION, TS.ID AS TASK_STATUS, TS.LABEL AS TASK_STATUS_NAME,
					TA.DURATION AS TOTAL_TIME, TT.ID AS TASK_TYPE, TA.DATE_EXCEPTION_TYPE AS DATE_EXCEPTION_ABBREVIATION, TA.DATE_EXCEPTION_TYPE,
					PT.USETIMEPLANNING,
                    DECODE(TK.ID, NULL, TA.TIME_PLANNED, 0) AS TIME_PLANNED,
					RCB.NAME AS CREATED_BY,TO_CHAR(TA.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT
            FROM 		TASK TA
            LEFT JOIN 	PROJECT PRO 			ON (TA.PROJECT = PRO.ID)
            LEFT JOIN 	PROJECT_TYPE PT 		ON (PT.ID = PRO.PROJECT_TYPE)
	        LEFT JOIN 	TASK_RELATIONSHIP TP 	ON (TA.ID = TP.TASK AND TP.TYPE = 'P')
	        LEFT JOIN 	TASK_RELATIONSHIP TR 	ON (TA.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M','T'))
            LEFT JOIN 	TASK_STATUS TS 			ON (TS.ID = TA.TASK_STATUS)
            LEFT JOIN 	TASK_TYPE TT 			ON (TT.ID = TS.TASK_TYPE)
            LEFT JOIN 	TASK TAPR 				ON (TAPR.ID = TP.TASK_RELATIONSHIP)
            LEFT JOIN 	TASK TAGE 				ON (TAGE.ID = TR.TASK)
            LEFT JOIN 	TASK TAPA 				ON (TAPA.ID = TA.PARENT)
            LEFT JOIN   RESOURCE_T RCB          ON (RCB.ID = TA.CREATED_BY)
            LEFT JOIN   ERROR_CLASSIFICATION EC ON EC.ID = TA.ERROR_CLASSIFICATION_ID
            LEFT JOIN   TASK_RELATIONSHIP TK    ON (TK.TASK_RELATIONSHIP = TA.ID AND TK.TYPE = 'A')
            WHERE TA.ORGANIZATION = :organization
            AND TA.ID = :idTask
SQL;
		$params = array(
			'organization' => $organization,
			'idTask' => $idTask
		);
		$connection = $this->getEntityManager()->getConnection();
		$tasks = $connection->fetchAll($query, $params);
		foreach($tasks as &$task){
			$task['RESOURCE'] = explode(',',$task['RESOURCE']);
			$task['TASK_LABEL'] = explode(',',$task['TASK_LABEL']);
			$task['TOTAL_TIME'] = Utilities::secondsToTime($task['TOTAL_TIME']);
			if(!empty($task['DATE_EXCEPTION_TYPE'])){
				$task['DATE_EXCEPTION_NAME'] =  \Service\Calendar\Restriction\Strategy\RestrictionFactory::$ARRAY_RESTRICTION[$task['DATE_EXCEPTION_TYPE']];
			}
			if($this->findOneBy(array('parent' => $idTask))){
				$task['IS_UPPER'] = true;
			} else {
				$task['IS_UPPER'] = false;
			}
		}


		return $tasks;
	}

	public function getDataSetForEditProjectTask($project, $organization, $filter = '') {
		$query = <<<SQL
		SELECT * FROM(
			SELECT  TA.ID, TA.NAME, TA.DESCRIPTION, TA.URL_TO_TEST, TS.LABEL AS TASK_STATUS_NAME,
			         (CASE WHEN PT.ALLOWENDTASK = 'Y' THEN TA.IN_CALC ELSE  'N' END ) AS IN_CALC,
					 (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(TA.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS BEGIN_DATE,
					 (CASE WHEN PT.ALLOWENDTASK = 'Y' THEN  TA.PERCENT_COMPLETED || '%' ELSE  NULL END ) AS PERCENT,
					 (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(TA.ESTIMATED_END_DATE,'DD/MM/YYYY') ELSE  NULL END ) AS ESTIMATED_DATE,
					 TA.TIME_PLANNED AS TIME_PLANNED,
					 TA.IS_PARENT,
					 DECODE(PV.ID, NULL, 'Última versão', PV.VERSION || '.' || PV.REVISION) AS VERSION_DEV, TA.PRODUCT_VERSION_ID_DEV AS VERSION_DEV_ID,
					 (SELECT CASE WHEN PRT.USETIMEPLANNING != 'Y' THEN 'N'
					 WHEN TO_NUMBER(TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'YYYYMMDD')) NOT BETWEEN TO_NUMBER( TO_CHAR(PRJ.BEGIN_DATE, 'YYYYMMDD')) AND TO_NUMBER( TO_CHAR(PRJ.END_DATE, 'YYYYMMDD'))
                     OR TO_NUMBER( TO_CHAR(TAS.ESTIMATED_END_DATE, 'YYYYMMDD')) NOT BETWEEN TO_NUMBER( TO_CHAR(PRJ.BEGIN_DATE, 'YYYYMMDD')) AND TO_NUMBER( TO_CHAR(PRJ.END_DATE, 'YYYYMMDD'))
                     THEN 'Y' ELSE 'N' END AS EXCEED_MODEL_DATE
                     FROM TASK TAS
                     INNER JOIN PROJECT_RELATIONSHIP PR ON TAS.PROJECT = PR.PROJECT AND PR.TYPE = 'M'
                     INNER JOIN PROJECT PRJ ON PRJ.ID = PR.PROJECT_RELATIONSHIP
                     INNER JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
                     WHERE TAS.PROJECT = TA.PROJECT AND TAS.ID = TA.ID ) AS EXCEED_MODEL_DATE,
					 (CASE WHEN PT.USETIMEPLANNING = 'Y' THEN TA.DURATION ELSE  NULL END ) AS DURATION,
					 TA.ESTIMATED_BEGIN_DATE AS BEGIN_DATE_TASK, TA.ESTIMATED_END_DATE AS END_DATE_TASK,
					TA.TIME_PLANNED AS TOTAL_DURATION, TA.CLASSIFICATION AS TASK_CLASSIFICATION, CLA.CONTROL AS TASK_CLASSIFICATION_CONTROL,
					(SELECT C.LABEL FROM CLASSIFICATION C WHERE C.ID = TA.CLASSIFICATION) AS TASK_CLASSIFICATION_LABEL,
					(
						SELECT LISTAGG(RES.ID, ',') WITHIN GROUP (ORDER BY RES.ID) AS BUSINESS_PARTNER_NAME
						FROM RESOURCE_T RES
						INNER JOIN ALLOCATION ALO ON (RES.ID = ALO.RESOURCE_T)
						WHERE ALO.TASK = TA.ID
					) AS RESOURCES, TA.PERCENT_COMPLETED, TAPA.NAME AS PARENT_NAME, TAPA.ID AS ID_PARENT, TATP.NAME AS PRECURSOR_NAME,
					(CASE WHEN TA.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS STATUS,'0' as HOUR_COMPLETED,PT.USETIMEPLANNING,
					PT.ALLOWENDTASK, TR.TYPE AS TYPE_RELATIONSHIP, TS.ID AS TASK_STATUS,TS.CONTROL AS TASK_STATUS_CONTROL,
					(SELECT COUNT(*)
					  FROM TASK_RELATIONSHIP TE
					 WHERE TE.TASK = TA.ID
					   AND TE.TYPE = 'E') AS GENERATED_TASKS_ERROR,PRO.PROJECT_TYPE,PRO.PROJECT_STATUS
					FROM 		TASK TA
					LEFT JOIN 	TASK_RELATIONSHIP TP ON (TA.ID = TP.TASK AND TP.TYPE = 'P')
	                LEFT JOIN 	TASK_RELATIONSHIP TR ON (TA.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T'))
                    LEFT JOIN 	TASK_STATUS TS 			ON (TS.ID = TA.TASK_STATUS)
					LEFT JOIN 	TASK TATP ON (TATP.ID = TP.TASK_RELATIONSHIP)
					LEFT JOIN 	TASK TAPA ON (TAPA.ID = TA.PARENT)
					LEFT JOIN 	PROJECT PRO ON (PRO.ID = TA.PROJECT)
					LEFT JOIN 	PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
					LEFT  JOIN 	CLASSIFICATION CLA ON (CLA.ID = TA.CLASSIFICATION )
					LEFT JOIN   PRODUCT_VERSION PV ON PV.ID = TA.PRODUCT_VERSION_ID_DEV
                    LEFT JOIN (
                      SELECT ROWNUM AS ORDEM, T.ID
                              FROM TASK T
                              LEFT JOIN TASK_RELATIONSHIP TF ON (T.ID = TF.TASK AND TF.TYPE = 'P')
                              WHERE T.PROJECT = :project
                                AND	 T.STATUS <> 'D'
                                AND	 NVL(T.IS_ISSUE,'N') <> 'Y'
                              START WITH TF.TASK_RELATIONSHIP IS NULL
                                 CONNECT BY PRIOR T.ID = TF.TASK_RELATIONSHIP) O ON (O.ID = TA.ID)
                                WHERE   TA.ORGANIZATION = :organization
                                AND	 TA.PROJECT = :project
                                AND	 TA.STATUS <> 'D'
                                AND	 NVL(TA.IS_ISSUE,'N') <> 'Y'
                                AND (CLA.CONTROL <> :controlClientAccept OR CLA.CONTROL IS NULL)
          ORDER BY TRUNC(CASE WHEN TS.CONTROL = 'B'
                              THEN TAPA.ESTIMATED_END_DATE
                              ELSE TA.ESTIMATED_BEGIN_DATE
                         END), O.ORDEM)
          WHERE 1=1
          $filter

SQL;
		$params = array(
			'organization' => $organization,
			'project' => $project,
			'controlClientAccept' => Entities\Classification::CONTROL_CLIENT_ACCEPT,
		);
		$tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

		foreach ($tasks as &$task) {
            $task['RESOURCES'] = Utilities::setSelectMultiInQuery($task['RESOURCES']);
			$task['DURATION'] = Utilities::secondsToTime((int)$task['DURATION']);

			if($task['USETIMEPLANNING'] == 'N'){
				$task['BEGIN_DATE'] = '';
				$task['ESTIMATED_DATE'] = '';
			}

			if($task['ALLOWENDTASK'] == 'N'){
				$task['PERCENT_COMPLETED'] = '';
			}
            if($task['TASK_STATUS_CONTROL'] == Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG){
				$task['BEGIN_DATE_TASK'] = '';
				$task['END_DATE_TASK'] = '';
                $task['BEGIN_DATE'] = '';
                $task['ESTIMATED_DATE'] = '';
				$task['DURATION'] = '';
				$task['TIME_PLANNED'] = '';
			}
		}
		return $tasks;
	}
	public function getDataSetForEditProjectTaskIssue($project, $organization, $task = 0, $classificationId = null,
                                                      $allFieldsField = '',$taskClassificationIdsString) {

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
		$query = <<<SQL
		SELECT *
		  FROM (SELECT TA.ID, TA.NAME, TO_CHAR(TA.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS BEGIN_DATE, TA.PERCENT_COMPLETED || '%' AS PERCENT,
					   TO_CHAR(TA.ESTIMATED_END_DATE,'DD/MM/YYYY') AS ESTIMATED_DATE, TA.TIME_PLANNED,
                       C.LABEL AS CLASSIFICATION,
                       TA.CLASSIFICATION AS CLASSIFICATION_ID,
					   (
                            SELECT rtrim(xmlagg(Xmlelement(e,RES.NAME,',').EXTRACT('//text()') order BY RES.NAME).getclobval(),',') AS BUSINESS_PARTNER_NAME
                            FROM 		RESOURCE_T RES
                            INNER JOIN 	ALLOCATION ALO ON (RES.ID = ALO.RESOURCE_T)
                            WHERE ALO.TASK = TA.ID
                        ) AS RESOURCES_NAME,
                        DECODE(PV.ID, NULL, 'Última versão', PV.VERSION || '.' || PV.REVISION) AS VERSION_DEV, TA.PRODUCT_VERSION_ID_DEV, PRO.NAME AS PROJECT_NAME
				FROM TASK TA
           LEFT JOIN PROJECT PRO ON(PRO.ID = TA.PROJECT)
           LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
		   LEFT JOIN CLASSIFICATION C  ON ( TA.CLASSIFICATION = C.ID)
		   LEFT JOIN PRODUCT_VERSION PV ON PV.ID = TA.PRODUCT_VERSION_ID_DEV
		       WHERE TA.ORGANIZATION = :organization
				     $filterTypeProject
                 AND TA.PROJECT = :project
                 AND TA.STATUS <> 'D'
                 AND TA.IS_PARENT <> 'Y'
                 AND TA.ID NOT IN ($task)
SQL;

        if($classificationId) {
            $query .= <<<SQL
            AND (TA.CLASSIFICATION = :classificationId OR TA.CLASSIFICATION IS NULL)
SQL;
        }
        if($taskClassificationIdsString){
            $query .= <<<SQL
            AND TA.CLASSIFICATION IN ($taskClassificationIdsString)
SQL;
        }
        $query .= <<<SQL
         	ORDER BY TA.ID DESC )
         	WHERE 1=1
         	$allFieldsField
SQL;
		$params = array(
			'organization' => $organization,
			'project' => $project,
			'task' => $task,
            'classificationId' => $classificationId
		);
		$connection = $this->getEntityManager()->getConnection();
		$tasks = $connection->fetchAll($query, $params);
		return $tasks;
	}

	public function getNotCompletedTasksByUser($resourceId){
		$query = <<<SQL
				SELECT T.PROJECT AS PROJECT_ID, P.NAME AS PROJECT_NAME, T.ID AS TASK_ID, T.NAME, T.DESCRIPTION, T.PERCENT_COMPLETED,
				(T.DURATION / 3600) AS DURATION, T.ESTIMATED_BEGIN_DATE AS BEGIN_DATE, T.ESTIMATED_END_DATE AS END_DATE
					FROM TASK T
					INNER JOIN ALLOCATION A ON (T.ID = A.TASK)
					INNER JOIN PROJECT P ON (T.PROJECT = P.ID)
					INNER JOIN RESOURCE_T R ON R.ID = A.RESOURCE_T
					WHERE R.ID = :resource
					  AND (T.ESTIMATED_END_DATE < TO_DATE(:today, 'DD/MM/YYYY') OR TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') = :today)
						AND T.PERCENT_COMPLETED < 100 AND P.STATUS = 'A' AND T.STATUS = 'A'
						ORDER BY P.ID, T.ID
SQL;
			$today = new \DateTime();
			$params = array(
				'today'	=> $today->format('d/m/Y'),
				'resource' => $resourceId
	  );

	  $connection = $this->getEntityManager()->getConnection();
	  $tasks = $connection->fetchAll($query, $params);
	  return $tasks;
		}

	public function getNotCompletedPerPeriod($resourceId, $beginDate, $endDate){
		$query = <<<SQL
  			SELECT T.PROJECT AS PROJECT_ID, P.NAME AS PROJECT_NAME, T.ID AS TASK_ID, T.NAME, T.DESCRIPTION, T.PERCENT_COMPLETED,
  			(T.DURATION / 3600) AS DURATION, T.ESTIMATED_BEGIN_DATE AS BEGIN_DATE, T.ESTIMATED_END_DATE AS END_DATE
				FROM TASK T
				INNER JOIN ALLOCATION A ON (T.ID = A.TASK)
				INNER JOIN PROJECT P ON (T.PROJECT = P.ID)
				INNER JOIN RESOURCE_T R ON R.ID = A.RESOURCE_T
				WHERE R.ID = :resource
				  AND (T.ESTIMATED_BEGIN_DATE BETWEEN TO_DATE(:beginDate, 'DD/MM/YYYY')
						AND TO_DATE(:endDate, 'DD/MM/YYYY')
						  OR 	:beginDate = TO_CHAR(T.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY')
						  OR 	:endDate = TO_CHAR(T.ESTIMATED_END_DATE,'DD/MM/YYYY')
					  )
					AND P.STATUS = 'A' AND T.STATUS = 'A'
					ORDER BY P.ID, T.ID
SQL;
			$params = array(
				'resource'  => $resourceId,
				'beginDate' => $beginDate,
				'endDate'   => $endDate
	  );
	  $connection = $this->getEntityManager()->getConnection();
	  $tasks = $connection->fetchAll($query, $params);
	  return $tasks;
	}

	public function getTotalNumberTask($projectTree){
		$query = <<<SQL
			SELECT COUNT(ID) AS TOTAL_TASK
			  FROM TASK
			 WHERE TREE_KEY LIKE :projectTree ||'%'
SQL;
		$params = array('projectTree' => $projectTree);

		$connection = $this->getEntityManager()->getConnection();
		$numberTasks = $connection->fetchAll($query, $params);

		return $numberTasks[0]['TOTAL_TASK'];
	}


	public function getRemainingExecutedNumberTask($projectTree, $startDate, $day){
		$query = <<<SQL
			SELECT COUNT(ID) AS REMAINING_EXECUTED
			  FROM TASK T
			 WHERE TREE_KEY LIKE :projectTree ||'%'
			   AND END_DATE BETWEEN TO_DATE(:startDate ,'DD/MM/YYYY') AND TO_DATE(:endDate ,'DD/MM/YYYY')
			   AND NOT EXISTS (SELECT ID FROM TASK WHERE PARENT = T.ID)
SQL;

		$params = array(
			'projectTree'  => $projectTree,
			'endDate' => $day->format('d/m/Y'),
			'startDate' => $startDate->format('d/m/Y')
			);

		$connection = $this->getEntityManager()->getConnection();
		$tasksExecuted = $connection->fetchAll($query, $params);
		return $tasksExecuted[0]['REMAINING_EXECUTED'];
	}


	public function getRemainingPlannedNumberTask($projectTree, $startDate, $day){
		$query = <<<SQL
			SELECT COUNT(ID) AS REMAINING_PLANNED
			  FROM TASK T
			 WHERE TREE_KEY LIKE :projectTree ||'%'
			   AND ESTIMATED_END_DATE BETWEEN TO_DATE(:startDate ,'DD/MM/YYYY') AND TO_DATE(:endDate ,'DD/MM/YYYY')
			   AND NOT EXISTS (SELECT ID FROM TASK WHERE PARENT = T.ID)
SQL;

		$params = array(
			'projectTree'  => $projectTree,
			'endDate' => $day->format('d/m/Y'),
			'startDate' => $startDate->format('d/m/Y')
			);

		$connection = $this->getEntityManager()->getConnection();
		$tasksPlanned = $connection->fetchAll($query, $params);

		return $tasksPlanned[0]['REMAINING_PLANNED'];
	}

	public function getRelationedTasks($taskId, $classificationId = null, $filter = ' '){

        $typeIssue = Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE;
		$query = <<<SQL
			SELECT TTR.ID, TTR.NAME, T.ID AS ISSUE,TTR.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
                   C.LABEL AS CLASSIFICATION, T.CLASSIFICATION AS CLASSIFICATION_ID,P.NAME AS PROJECT_NAME
			FROM 		TASK T
			INNER JOIN 	TASK_RELATIONSHIP TR 	ON TR.TASK = T.ID OR TR.TASK_RELATIONSHIP = T.ID
			INNER JOIN 	TASK TTR 				ON (TR.TASK = TTR.ID OR TR.TASK_RELATIONSHIP = TTR.ID) AND TTR.ID <> :taskId
			 LEFT JOIN  CLASSIFICATION C        ON ( T.CLASSIFICATION = C.ID)
			 JOIN PROJECT P ON (P.ID = TTR.PROJECT)
             WHERE 	T.ID = :taskId
             AND TR.TYPE = :TYPE_ISSUE
             AND TTR.STATUS = 'A'
SQL;
        if($classificationId ){
            $query .= <<<SQL
            AND (T.CLASSIFICATION = :classificationId OR T.CLASSIFICATION IS NULL)
SQL;
        }
        $query = 'SELECT * FROM('.$query.') WHERE 1 = 1'.$filter;
		$params = array(
			'taskId' => $taskId,
            'classificationId' => $classificationId,
            'TYPE_ISSUE' => $typeIssue
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getTaskRelationshipByIssue($taskId, $filter = array()){

        $typeIssue = Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE;
		$query = <<<SQL
			SELECT TTR.ID, TTR.NAME, T.ID AS ISSUE,TTR.PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,
                   C.LABEL AS CLASSIFICATION, C.ID AS CLASSIFICATION_ID,P.NAME AS PROJECT_NAME, TTR.PERCENT_COMPLETED AS PERCENT
			FROM 		TASK T
			INNER JOIN 	TASK_RELATIONSHIP TR 	ON TR.TASK = T.ID OR TR.TASK_RELATIONSHIP = T.ID
			INNER JOIN 	TASK TTR 				ON (TR.TASK = TTR.ID OR TR.TASK_RELATIONSHIP = TTR.ID) AND TTR.ID <> :taskId
			 LEFT JOIN  CLASSIFICATION C        ON ( TTR.CLASSIFICATION = C.ID)
			 JOIN PROJECT P ON (P.ID = TTR.PROJECT)
             WHERE 	T.ID = :taskId
             AND TR.TYPE = :TYPE_ISSUE
             AND TTR.STATUS = :STATUS_ACTIVE
SQL;
        $query = 'SELECT * FROM('.$query.') WHERE 1 = 1'.$filter;
		$params = array(
			'taskId' => $taskId,
            'TYPE_ISSUE' => $typeIssue,
            'STATUS_ACTIVE' => Utilities::STATUS_ACTIVE,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getRelationedIssues($taskId, $filter = ' '){

		$query = <<<SQL
		SELECT * FROM(
			SELECT PRO.ORGANIZATION, TAS.ID,TAS.NAME,PRO.NAME AS PRODUCT_NAME,TTY.CONTROL,TTY.LABEL AS TASK_TYPE_NAME,
			       TRE.TASK AS iSSUE
			 FROM TASK TAS
			 JOIN TASK_RELATIONSHIP TRE ON(TAS.ID = TRE.TASK_RELATIONSHIP AND TRE.TYPE = :TYPE_ISSUE)
			 JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
			 JOIN PROJECT_TYPE PRT ON (PRT.ID = PRO.PROJECT_TYPE AND PRT.EXTERNALTYPE = 'P')
			 JOIN TASK_STATUS TST ON (TST.ID = TAS.TASK_STATUS) JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
			 WHERE TRE.TASK = :taskId
			 AND TAS.IS_ISSUE = 'Y'
			 AND TAS.STATUS = 'A'
			 ORDER BY TAS.ID DESC)
        WHERE 1 =1
        $filter
SQL;

		$params = array(
			'taskId' => $taskId,
            'TYPE_ISSUE' => Entities\TaskRelationship::ISSUE_WITH_ISSUE_TYPE
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}


    public function getAllIssueNeedFramingAutomatic(){

		$query = <<<SQL
		   SELECT TAS.ID,TTY.TASK_STATUS_AUTO_CHANGE_STATUS,TTY.DESCRIPTION_AUTO_CHANGE_STATUS,TAS.ORGANIZATION
             FROM TASK_TYPE TTY
             JOIN TASK_STATUS TST ON (TST.TASK_TYPE = TTY.ID)
             JOIN TASK TAS ON (TAS.TASK_STATUS = TST.ID AND TAS.IS_ISSUE = 'Y' AND STATUS = 'A')
             JOIN TASK_PROPERTIES TPR ON (TPR.ORGANIZATION = TAS.ORGANIZATION AND TAS.TASK_STATUS <> FINAL_TASK_STATUS)
            WHERE TTY.TIME_AUTO_CHANGE_STATUS IS NOT NULL
              AND CURRENT_TASK_STATUS_IN <= TRUNC( SYSDATE - NVL(TTY.TIME_AUTO_CHANGE_STATUS,1) )
              AND TTY.TASK_STATUS_AUTO_CHANGE_STATUS <> TAS.TASK_STATUS

SQL;

		$params = array(
			'IS_ISSUE' => Entities\Task::IS_ISSUE,
            'STATUS' => Entities\Task::ACTIVE
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function validateMergeTaskInSprint($projectId, $taskId) {

        $query = <<<SQL

            SELECT DISTINCT PR.PROJECT_RELATIONSHIP, PJ3.ID, PJ3.NAME
              FROM (SELECT PJ.ID, PJ.NAME
                      FROM (SELECT ID, NAME, STATUS, PROJECT, TASK_STATUS, CLASSIFICATION
                              FROM TASK
                             WHERE PROJECT IN (SELECT ID
                                                 FROM (SELECT PJ1.ID, PJ1.STATUS, PT.EXTERNALTYPE, TS.CONTROL
                                                         FROM PROJECT PJ1
                                                         JOIN PROJECT_TYPE PT ON PJ1.PROJECT_TYPE = PT.ID
                                                         JOIN TASK_STATUS TS ON PJ1.PROJECT_STATUS = TS.ID
                                                   START WITH PJ1.ID IN (:PROJECT_ID)
                                                   CONNECT BY PRIOR PJ1.ID = PJ1.PARENT)
                                                WHERE EXTERNALTYPE = 'B'
                                                  AND STATUS = 'A'
                                                  AND CONTROL NOT IN ('X', 'C')
                                                  AND ID NOT IN (SELECT PROJECT
                                                                   FROM TASK
                                                                  WHERE ID = :TASK_ID))
                               AND STATUS = 'A') TASK
                      JOIN PROJECT PJ ON TASK.PROJECT = PJ.ID
                      JOIN PROJECT_TYPE PT ON PJ.PROJECT_TYPE = PT.ID
                      JOIN TASK_STATUS TS ON TASK.TASK_STATUS = TS.ID
                      JOIN CLASSIFICATION CL ON CL.ID = TASK.CLASSIFICATION
                     WHERE (PT.USEMERGEREQUEST = :IS_USE_MERGE_REQUEST
                            AND TS.CONTROL = :CONTROL
                            AND CL.CONTROL = :MERGE_CLASSIFICATION_CONTROL)
                        OR (PT.USEMERGEREQUEST = :IS_NOT_USE_MERGE_REQUEST
                            AND TS.CONTROL = :CONTROL)) PJ3
              JOIN PROJECT_RELATIONSHIP PR ON PR.PROJECT = PJ3.ID
             WHERE PR.TYPE = :TYPE_MODEL
               AND PR.PROJECT_RELATIONSHIP IN (SELECT ID
                                                 FROM PROJECT
                                                WHERE PARENT = (SELECT PJ.PARENT
                                                                  FROM PROJECT PJ
                                                                 WHERE ID = (SELECT PR.PROJECT_RELATIONSHIP
                                                                               FROM TASK
                                                                               JOIN PROJECT PJ ON TASK.PROJECT = PJ.ID
                                                                               JOIN PROJECT_RELATIONSHIP PR ON PR.PROJECT = PJ.ID
                                                                              WHERE TASK.ID = :TASK_ID
                                                                                AND PR.TYPE = :TYPE_MODEL)))


SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'PROJECT_ID' => $projectId,
            'CONTROL' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'IS_NOT_USE_MERGE_REQUEST' => Entities\ProjectType::IS_NOT_USE_MERGE_REQUEST,
            'IS_USE_MERGE_REQUEST' => Entities\ProjectType::IS_USE_MERGE_REQUEST,
            'TYPE_MODEL' => Entities\ProjectRelationship::TYPE_MODEL_GENERATION,
            'MERGE_CLASSIFICATION_CONTROL' => Entities\Classification::CONTROL_MERGE_REQUEST,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function validateIfProjectIsError($taskId){

        $query = <<<SQL

       SELECT PJ.NAME, PT.LABEL, PT.EXTERNALTYPE FROM TASK TK
                        JOIN PROJECT PJ ON PJ.ID = TK.PROJECT
                        JOIN PROJECT_TYPE PT ON PJ.PROJECT_TYPE = PT.ID
                        WHERE TK.ID = $taskId
                          AND PT.EXTERNALTYPE = 'E'


SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }


    public function validateMergeTaskConclude($taskId){

		$query = <<<SQL

        SELECT PR2.NAME,PR2.ID
          FROM TASK TAS
          JOIN PROJECT PR1 ON (TAS.PROJECT = PR1.ID)
          JOIN SUBRELEASE SU1 ON (SU1.PROJECT_ID = PR1.ID)
          JOIN RELEASE RE1 ON (SU1.RELEASE_ID = RE1.ID)
          JOIN PROJECT_TYPE PT1 ON (PR1.PROJECT_TYPE = PT1.ID AND PT1.USETIMEPLANNING = :USE_TIME_PLANNING)
          JOIN PROJECT_RELATIONSHIP PE4 ON(PE4.PROJECT = PR1.ID AND PE4.TYPE = :TYPE_MODEL)
          JOIN PROJECT PR4 ON (PE4.PROJECT_RELATIONSHIP = PR4.ID AND PR4.STATUS = :IS_ACTIVE)

          JOIN PROJECT_RELATIONSHIP PE1 ON(PE1.PROJECT = PR1.ID AND PE1.TYPE = :TYPE_MODEL_GENERATION)
          JOIN PROJECT PRM ON (PE1.PROJECT_RELATIONSHIP = PRM.ID)
          JOIN PROJECT PRP ON (PRM.PARENT = PRP.PARENT)

          JOIN PROJECT_TYPE PT2 ON (PRP.PROJECT_TYPE = PT2.ID AND PT2.EXTERNALTYPE = :MODEL_RESTRICTED)
          JOIN PROJECT_RELATIONSHIP PE2 ON(PE2.PROJECT_RELATIONSHIP = PRP.ID AND PE2.TYPE = :TYPE_MODEL_GENERATION AND PR1.ID <> PE2.PROJECT )

          JOIN PROJECT PR2 ON (PE2.PROJECT = PR2.ID AND PR2.STATUS = :IS_ACTIVE)
          JOIN PROJECT_TYPE PT3 ON (PR2.PROJECT_TYPE = PT3.ID AND PT3.USETIMEPLANNING = :USE_TIME_PLANNING)
          JOIN TASK_STATUS TS1 ON (PR2.PROJECT_STATUS = TS1.ID AND TS1.CONTROL NOT IN(:CONCLUDED,:CANCELED))
          JOIN SUBRELEASE SU2 ON (SU2.PROJECT_ID = PR2.ID)
          JOIN RELEASE RE2 ON (SU2.RELEASE_ID = RE2.ID AND RE1.PRODUCT_ID = RE2.PRODUCT_ID)

          JOIN PROJECT_RELATIONSHIP PE3 ON(PE3.PROJECT = PR2.ID AND PE3.TYPE = :TYPE_MODEL)
          JOIN PROJECT PR3 ON (PE3.PROJECT_RELATIONSHIP = PR3.ID AND PR3.STATUS =:IS_ACTIVE AND PR4.END_DATE > PR3.END_DATE )

         WHERE TAS.ID = :TASK_ID

SQL;

		$params = array(
            'TASK_ID' => $taskId,
            'USE_TIME_PLANNING' => Entities\ProjectType::IS_USE_TIME_PLANNING,
            'TYPE_MODEL' => Entities\ProjectRelationship::TYPE_MODEL,
            'TYPE_MODEL_GENERATION' => Entities\ProjectRelationship::TYPE_MODEL_GENERATION,
            'CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'MODEL_RESTRICTED' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MODEL_RESTRICTED,
            'IS_ACTIVE' => Entities\Project::IS_ACTIVE,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getIssue($issueId, $userId){
		$query = <<<SQL
		   SELECT 	T.ID, T.RESPONSE, P.ID AS BOARD, T.NAME AS LABEL, COALESCE(AU.IS_RELEVANT, 'N') AS IS_RELEVANT,TS.CONTROL,TT.CONTROL AS CONTROL_TASK_TYPE,
                    DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
                    DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV,
                    PRV.ID AS PRODUCT_VERSION_ID, PROV.ID AS PRODUCT_VERSION_DEV_ID,
                    DECODE((SELECT MAX(TAS.ID)
                             FROM TASK_RELATIONSHIP TRL
                        LEFT JOIN TASK TAS ON (TAS.ID = TRL.TASK_RELATIONSHIP AND TAS.IS_ISSUE = 'N')
                            WHERE TRL.TASK = T.ID
                              AND TRL.TYPE = 'I'), NULL, 'N', 'Y') AS GENERATED_TASK,
             (SELECT COUNT(*) FROM ALLOCATION A WHERE A.IS_RELEVANT = 'Y' AND A.TASK = :issueId) AS RELEVANCE,
             (   SELECT ISS.ID
                   FROM TASK ISS
                   JOIN CLASSIFICATION_BUDGET_ITEM CBI ON (CBI.CLASSIFICATION   = ISS.CLASSIFICATION )
                   JOIN BUDGET_ITEM BUI ON (BUI.ID = CBI.BUDGET_ITEM)
              LEFT JOIN BUDGET BUD ON (BUD.TASK = ISS.ID)
              LEFT JOIN BUDGET_ITEM_BUDGET BIB ON (BIB.BUDGET = BUD.ID AND BIB.BUDGET_ITEM = BUI.ID)
                  WHERE ISS.ID = :issueId
                    AND (BUI.STATUS = 'A' OR BIB.ID IS NOT NULL)
                    AND ROWNUM = 1) AS WITH_BUDGET,
             (    SELECT rtrim(xmlagg(xmlelement(e,CLA.ID,',').extract('//text()') order by CLA.LABEL).GetClobVal(),',')
                    FROM TASK ISS
                    JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                    JOIN TASK_TYPE_CLASSIFICATION TAT ON (TAT.TASK_TYPE_ID = TST.TASK_TYPE AND TAT.TASK_ISSUE_CLASSIF = :typeTask )
                    JOIN CLASSIFICATION CLA ON (CLA.ID = TAT.CLASSIFICATION_ID)
                    WHERE ISS.ID = :issueId
                      AND EXISTS (SELECT *
                                  FROM TASK ISS
                                  JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                  JOIN TASK_TYPE_CLASSIFICATION TAT ON (TAT.TASK_TYPE_ID = TST.TASK_TYPE
                                                                        AND TAT.TASK_ISSUE_CLASSIF = :typeIssue)
                                  WHERE ISS.ID = :issueId AND ISS.CLASSIFICATION = TAT.CLASSIFICATION_ID)) AS TASK_CLASSIFICATION_IDS,
             SP.NAME AS SPONSOR, SP.ID AS SPONSOR_ID, C.ID AS ISSUE_CLASSIFICATION, C.LABEL AS ISSUE_CLASSIFICATION_NAME, TS.ID AS ISSUE_STATUS,
				  	TS.LABEL AS ISSUE_STATUS_NAME, RR.ID AS REPORTER, RR.NAME AS REPORTER_NAME, RR.STATUS AS REPORTER_STATUS, P.ID AS PRODUCT, P.NAME AS PRODUCT_NAME,
				  	TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS DUE_DATE,TO_CHAR(T.END_DATE, 'DD/MM/YYYY') AS END_DATE, T.PRIORITY, PRI.LABEL AS PRIORITY_NAME,T.PERCENT_COMPLETED, T.DESCRIPTION,
				  	(SELECT rtrim(xmlagg(xmlelement(e,id,',').extract('//text()') order by id).GetClobVal(),',')
              FROM (SELECT R.ID
                    FROM  RESOURCE_T R
                    JOIN ALLOCATION A  ON A.TYPE = 'F' AND A.TASK = :issueId AND A.RESOURCE_T = R.ID)) AS FOLLOWERS,
				  	 (SELECT rtrim(xmlagg(xmlelement(e,TL.LABEL,',').extract('//text()') order by TL.LABEL).GetClobVal(),',')
					 FROM TASK_LABEL TL
					 WHERE TL.TASK = T.ID) as LABELS,TT.ALLOW_CLOSE_ISSUE,  RCB.NAME AS CREATED_BY, TO_CHAR(T.CREATED_AT, 'DD/MM/YYYY HH24:MI:SS')  AS CREATED_AT,
					 RUB.NAME AS UPDATED_BY, TO_CHAR(T.UPDATED_AT, 'DD/MM/YYYY HH24:MI:SS')  AS UPDATED_AT
			FROM 		TASK T
			JOIN 	CLASSIFICATION C 	ON C.ID = T.CLASSIFICATION
			JOIN 	TASK_STATUS TS 		ON TS.ID = T.TASK_STATUS
		    JOIN    TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
			LEFT JOIN 	ALLOCATION AR 		ON AR.TYPE = 'R' AND AR.TASK = T.ID
			LEFT JOIN 	ALLOCATION AF 		ON AF.TYPE = 'F' AND AF.TASK = T.ID
			LEFT JOIN   ALLOCATION AL     ON AL.TYPE = 'S' AND AL.TASK = T.ID
            LEFT JOIN 	ALLOCATION AU 		ON  AU.RESOURCE_T = :userId AND AU.TYPE = 'F' AND AU.TASK = T.ID
			LEFT JOIN 	RESOURCE_T RR 		ON AR.RESOURCE_T = RR.ID
			LEFT JOIN 	RESOURCE_T RF 		ON AF.RESOURCE_T = RF.ID
			LEFT JOIN   RESOURCE_T SP       ON AL.RESOURCE_T = SP.ID
            LEFT JOIN   RESOURCE_T RCB      ON(RCB.ID = T.CREATED_BY)
            LEFT JOIN   RESOURCE_T RUB      ON(RUB.ID = T.UPDATED_BY)
			LEFT JOIN 	PROJECT P 			ON T.PROJECT = P.ID
			LEFT JOIN   PRIORITY PRI		ON(PRI.ID = T.PRIORITY)
            LEFT JOIN   PRODUCT_VERSION PRV ON PRV.ID = T.PRODUCT_VERSION_ID
            LEFT JOIN   PRODUCT_VERSION PROV ON PROV.ID = T.PRODUCT_VERSION_ID_DEV
			WHERE T.ID = :issueId
			GROUP BY 	T.ID, T.RESPONSE, T.NAME, C.ID, C.LABEL, TS.ID, TS.LABEL, RR.ID, RR.STATUS, RR.NAME, T.ESTIMATED_END_DATE,T.END_DATE, T.PRIORITY,PRI.LABEL, T.PERCENT_COMPLETED, T.DESCRIPTION, P.ID, P.NAME, SP.NAME, SP.ID,TT.ALLOW_CLOSE_ISSUE, T.CREATED_AT,RCB.NAME, AU.IS_RELEVANT,TS.CONTROL,T.UPDATED_AT,RUB.NAME, PRV.ID, PRV.VERSION, PRV.REVISION, PRV.BUILD,
                PROV.ID, PROV.VERSION, PROV.REVISION, PROV.BUILD, TT.CONTROL


SQL;
		$params = array(
            'issueId' => $issueId,
            'typeTask' => Entities\TaskTypeClassification::TYPE_TASK,
            'typeIssue' => Entities\TaskTypeClassification::TYPE_ISSUE,
            'userId'  => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getIssueReporter($issueId){
		$query = <<<SQL
		  SELECT DISTINCT ORG.IDORGSAAS FROM TASK ISS 
                    JOIN RESOURCE_T RST ON ISS.CREATED_BY = RST.ID 
                    JOIN OPERADOR OPR ON RST.EXTERNAL_CODE = OPR.CDOPERADOR 
                    JOIN CLIENTE CLT ON OPR.CDCLIENTE = CLT.CDCLIENTE 
                    JOIN ORGANIZACAO ORG ON CLT.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
                    WHERE RST.ID = $issueId AND RST.RESOURCES_TYPE = 'W' AND RST.CLASS = 'E'
SQL;

		return $this->getEntityManager()->getConnection()->fetchAll($query);
	}

	public function getActionsOfExits($issueId){
		$query = <<<SQL
		        SELECT TTCT.ID, TTCT.ACTION_LABEL, TTCT.TASK_TYPE_TARGET, COTY.ID AS COMMENT_TYPE_ID,COTY.ATTACHMENT_REQUIRED,
                       NVL(TTCT.COMMENT_DESCRIPTION, COTY.COMMENT_DESCRIPTION) AS COMMENT_DESCRIPTION
                  FROM TASK T
                  JOIN TASK_STATUS TS ON TS.ID = T.TASK_STATUS
                  JOIN TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
                  JOIN TASK_TYPE_COMMENT_TYPE TTCT ON (TT.ID = TTCT.TASK_TYPE_ID)
             LEFT JOIN TASK_TYPE_COMMENT_TYPE_CLASSIF TCTC ON (TCTC.TASK_TYPE_COMMENT_TYPE_ID = TTCT.ID)
             LEFT JOIN COMMENT_TYPE COTY ON (COTY.ID = TTCT.COMMENT_TYPE_ID )
                 WHERE T.ID = :issueId
                   AND (TCTC.CLASSIFICATION_ID = T.CLASSIFICATION OR TCTC.CLASSIFICATION_ID IS NULL)
                   AND TTCT.IN_OUT_COMMENT = :OUT_COMMENT
SQL;
		$params = array(
            'issueId' => $issueId,
            'OUT_COMMENT' => Entities\TaskTypeCommentType::OUT_COMMENT,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getAllIssuesGrid($organizationId, $userId, $conditions, $filterComments = '', $filterRelevance = '', $filterProductVersion = '')
    {
        $query = <<<SQL
        SELECT *
          FROM (
			  SELECT I.ID as "id", I.NAME as "label", TO_CHAR(I.ESTIMATED_END_DATE, 'DD/MM/YYYY') as "due-date1",
			         TO_CHAR(I.END_DATE, 'DD/MM/YYYY') as "DUE_DATE", I.DESCRIPTION as "description", './images/user.jpg' as "photo",
			         (SELECT COUNT(*) FROM ALLOCATION A WHERE A.IS_RELEVANT = 'Y' AND A.TASK = I.ID) AS RELEVANCE,
					 TT.ORDER_TO_VIEW as "column", TT.LABEL as "columnLabel", TT.ORDER_TO_FRAME as ORDER_TO_FRAME, TT.ID AS TASK_STATUS_ID, PRI.COLOR,RA.RESOURCE_T AS REPORTER,
					 RR.NAME AS REPORTER_NAME, P.NAME AS PROJECT_NAME, C.LABEL AS CLASSIFICATION, TS.LABEL AS STATUS, TO_CHAR(I.CREATED_AT, 'DD/MM/YYYY') as CREATED_AT,
					 PRI.LABEL AS PRIORITY, SR.NAME AS SPONSOR,
                     DECODE(PRV.ID, NULL, 'Última versão', PRV.VERSION || '.' || PRV.REVISION || '.' || PRV.BUILD) AS VERSION,
                     DECODE(PROV.ID, NULL, 'Última versão', PROV.VERSION || '.' || PROV.REVISION) AS VERSION_DEV
				FROM TASK I
		  INNER JOIN TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
		  INNER JOIN TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
		   LEFT JOIN PRIORITY PRI ON (I.PRIORITY = PRI.ID)
		   LEFT JOIN CLASSIFICATION C ON C.ID = I.CLASSIFICATION
		   LEFT JOIN ALLOCATION RA ON RA.TASK = I.ID  AND RA.TYPE = 'R'
		   LEFT JOIN ALLOCATION SA ON SA.TASK = I.ID  AND SA.TYPE = 'S'
		   LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
		   LEFT JOIN LABEL L ON TL.LABEL = L.ID
		   LEFT JOIN ALLOCATION FA ON FA.TASK = I.ID  AND FA.TYPE = 'F'
		   LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
           LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
           LEFT JOIN PROJECT P ON (P.ID = I.PROJECT)
           LEFT JOIN RESOURCE_T RR 		ON RA.RESOURCE_T = RR.ID
           LEFT JOIN RESOURCE_T SR 		ON SA.RESOURCE_T = SR.ID
           LEFT JOIN PRODUCT_VERSION PRV ON PRV.ID = I.PRODUCT_VERSION_ID
           LEFT JOIN PRODUCT_VERSION PROV ON PROV.ID = I.PRODUCT_VERSION_ID_DEV
		       WHERE I.ORGANIZATION = :organizationId
				     AND I.IS_ISSUE = 'Y'
				     AND I.STATUS <> 'D'
                     $filterComments
                     $conditions
            GROUP BY I.ID,I.NAME, SR.NAME ,C.LABEL,RR.NAME,P.NAME, I.CREATED_AT, I.END_DATE , PRI.LABEL,TS.LABEL, TT.ID,
                     I.ESTIMATED_END_DATE, I.DESCRIPTION, TT.ID, TT.LABEL, TT.ORDER_TO_VIEW, TT.ORDER_TO_FRAME, PRI.COLOR, RA.RESOURCE_T, PRV.ID, PRV.VERSION, PRV.REVISION, PRV.BUILD, PROV.ID, PROV.VERSION, PROV.REVISION, PROV.BUILD
            ORDER BY I.ID DESC) I
        WHERE 1=1
              $filterRelevance
              $filterProductVersion
SQL;

        $params = array(
            'organizationId' => $organizationId,
            'userId' => $userId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


	/**
	 * Gets all task parameters
	 * @param int $organizationId
	 * @return array
	 */

	 /* MEXER AQUI */
	public function getTaskOptionsParameters($organizationId){
		$query = <<<SQL
			SELECT 		TS1.LABEL AS INITIAL_TASK_STATUS_NAME,
						TS1.ID AS INITIAL_TASK_STATUS_ID,
						TS2.LABEL AS  FINAL_TASK_STATUS_NAME,
						TS2.ID AS FINAL_TASK_STATUS_ID,
						TS3.LABEL AS  CONCLUDED_TASK_STATUS_NAME,
						TS3.ID AS CONCLUDED_TASK_STATUS_ID,
						TP.ID AS PROPERTY_ID
			FROM		TASK_PROPERTIES TP
			LEFT JOIN	TASK_STATUS TS1			ON TS1.ID = TP.INITIAL_TASK_STATUS
			LEFT JOIN	TASK_STATUS TS2			ON TS2.ID = TP.FINAL_TASK_STATUS
			LEFT JOIN	TASK_STATUS TS3			ON TS3.ID = TP.CONCLUDED_TASK_STATUS
			WHERE 		TP.ORGANIZATION = :organizationId
SQL;
		$params = array(
			'organizationId' => $organizationId
		);

		# Forcing to return a null value if no records are found
		$result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		if(empty($result)){
			$result = array(array(
				'INITIAL_TASK_STATUS_NAME' => null,
				'INITIAL_TASK_STATUS_ID' => null,
				'FINAL_TASK_STATUS_NAME' => null,
				'FINAL_TASK_STATUS_ID' => null,
				'PROPERTY_ID' => null
			));
		}
		return $result;
	}

	public function getTasksAppropriatedForUserInRange($userId, $organizationId, $begin, $end, $range, $showWeekendQuery, $weekendQueryDate){
        $filterOrganization     = $organizationId ? " AND A.ORGANIZATION = $organizationId" : '';
        $filterOrganizationTask = $organizationId ? " AND T.ORGANIZATION = $organizationId" : '';
		$query = <<<SQL
			SELECT D.ID, D.ID || ' ' || D.NAME AS TASK_NAME, T.USE_TIME_PLANNING,
                   SUM(NVL(T.TOTAL, 0)) AS HOURS,
                   D.DAY, D.DAY_LABEL, T.ALLOWENDTASK
              FROM (SELECT T.ID, T.NAME AS TASK_NAME, A.TOTAL, PT.USETIMEPLANNING AS USE_TIME_PLANNING, PT.ALLOWENDTASK, TRUNC(A.BEGIN_HOUR) AS TASK_DATE
                      FROM TASK T
                     INNER JOIN APPROPRIATION A ON A.TASK = T.ID AND A.RESOURCE_T = :userId
                           JOIN PROJECT P ON T.PROJECT = P.ID
                      LEFT JOIN PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
                           JOIN CLASSIFICATION CL ON CL.ID = T.CLASSIFICATION
                     WHERE A.RESOURCE_T = :userId
                       AND A.BEGIN_HOUR $range
                       AND T.STATUS <> 'D'
                       AND T.IS_ISSUE = 'N'
                       AND CL.USED_IN_TIME_CONTROL = 'Y'
                       $filterOrganization
					   $showWeekendQuery
                     ORDER BY A.BEGIN_HOUR DESC) T,
                   (SELECT T.ID, T.NAME, D.DATA, TO_CHAR(D.DATA, 'D DD/MM/YYYY') AS DAY, TO_CHAR(D.DATA, 'D DD/MM/YYYY') AS DAY_LABEL
                      FROM TASK T, PROJECT P, PROJECT_TYPE PT, CLASSIFICATION CL,
                           (SELECT TRUNC(TO_DATE('$begin', 'DD/MM/YYYY')) + LEVEL - 1 AS DATA
                              FROM DUAL
                           CONNECT BY LEVEL <= TO_DATE('$end', 'DD/MM/YYYY') - TO_DATE('$begin', 'DD/MM/YYYY') + 1) D
                     WHERE T.STATUS <> 'D'
                       AND T.IS_ISSUE = 'N'
                       AND T.PROJECT = P.ID
                       AND P.PROJECT_TYPE = PT.ID
                       AND CL.ID = T.CLASSIFICATION
                       AND CL.USED_IN_TIME_CONTROL = 'Y'
                       AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
                       $filterOrganizationTask
                       $weekendQueryDate
                       AND T.ID IN (SELECT A.TASK
                                      FROM APPROPRIATION A
                                     WHERE A.RESOURCE_T = :userId
                                       AND A.BEGIN_HOUR $range
                                       $filterOrganization)) D
             WHERE D.ID = T.ID(+)
               AND D.DATA = T.TASK_DATE(+)
             GROUP BY D.ID, D.NAME, T.USE_TIME_PLANNING, D.DAY, D.DAY_LABEL, T.ALLOWENDTASK, D.DATA
             ORDER BY D.ID, D.DATA
SQL;
		$params = array(
			'userId' => $userId,
			'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function getTotalAppropriatedForDayInRange($userId, $range, $organizationId){

		$query = <<<SQL
			SELECT SUM(TOTAL) AS HOURS, DAY,BEGIN_HOUR
			FROM (	SELECT A.TOTAL, TO_CHAR(A.BEGIN_HOUR, 'DD/MM/YYYY') AS DAY, TO_CHAR(A.BEGIN_HOUR, 'YYYYMMDD') AS BEGIN_HOUR
					FROM TASK T
					INNER JOIN APPROPRIATION A 	ON A.TASK = T.ID AND A.RESOURCE_T = :userId
					WHERE A.RESOURCE_T = :userId
					AND A.BEGIN_HOUR $range
					AND T.STATUS <> 'D'
					AND T.IS_ISSUE = 'N'
					AND T.ORGANIZATION = :ORGANIZATION
					ORDER BY A.BEGIN_HOUR DESC)
			GROUP BY BEGIN_HOUR, DAY
            ORDER BY BEGIN_HOUR DESC
SQL;
		$params = array(
			'userId' => $userId,
			'ORGANIZATION' => $organizationId,
		);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
	}

	public function getTasksNotAppropriatedForUser($userId, $range){
		$query = <<<SQL
			SELECT 	T.ID, T.PARENT,
					T.NAME AS TASK_NAME, P.ID AS PROJECT_ID, P.NAME AS PROJECT_NAME,
					(CASE
						WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY')
						ELSE NULL
					END) AS ESTIMATED_BEGIN_DATE,
					T.PROJECT,
					(CASE WHEN PT.ALLOWENDTASK = 'Y' THEN T.PERCENT_COMPLETED ELSE NULL END) AS PERCENT_COMPLETED,
					(CASE
						WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY')
						ELSE NULL
					END) AS ESTIMATED_END_DATE, PT.ALLOWENDTASK, PT.USETIMEPLANNING AS USE_TIME_PLANNING
			FROM 		TASK T
			LEFT JOIN 	APPROPRIATION A ON A.TASK = T.ID AND A.RESOURCE_T = :userId
			INNER JOIN 	ALLOCATION AL 	ON AL.TASK = T.ID AND AL.RESOURCE_T = :userId
			INNER JOIN 	PROJECT P 		ON P.ID = T.PROJECT
			INNER JOIN 	PROJECT_TYPE PT ON PT.ID = P.PROJECT_TYPE AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
			JOIN 	CLASSIFICATION CLA ON (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> :CONTROL_CLIENT_ACCEPT OR CLA.CONTROL IS NULL))
			WHERE 	T.IS_ISSUE = 'N'
			AND 	T.STATUS <> 'D'
			AND 	P.STATUS <> 'D'
			AND 	T.PERCENT_COMPLETED <= 99
			AND 	(A.ID IS NULL OR A.END_HOUR NOT $range)
			GROUP BY T.ID, T.PARENT, T.NAME, P.ID, P.NAME, T.ESTIMATED_BEGIN_DATE, T.ESTIMATED_END_DATE, T.PROJECT, T.PERCENT_COMPLETED, PT.USETIMEPLANNING, PT.ALLOWENDTASK
			ORDER BY T.ID ASC
SQL;
		$params = array(
			'userId' => $userId,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
            'CONTROL_CLIENT_ACCEPT' => Entities\Classification::CONTROL_CLIENT_ACCEPT
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getTasksForCurrentUser($userId){
		$query = <<<SQL
			SELECT 	T.ID, (CASE WHEN TP.ID IS NULL THEN P.ID ELSE TP.ID END) AS PARENT, T.NAME AS TASK_NAME,P.ID AS PROJECT, (CASE
						WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY')
						ELSE NULL
					END) AS ESTIMATED_BEGIN_DATE, T.PROJECT,
					(CASE WHEN PT.USETIMEPLANNING = 'Y' THEN T.PERCENT_COMPLETED ELSE NULL END) AS PERCENT_COMPLETED,
					(CASE
						WHEN PT.USETIMEPLANNING = 'Y' THEN TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY')
						ELSE NULL
					END) AS ESTIMATED_END_DATE
			FROM 		TASK T
			LEFT JOIN 	ALLOCATION AL 	ON AL.TASK = T.ID AND AL.RESOURCE_T = :userId
			LEFT JOIN 	PROJECT P 		ON P.ID = T.PROJECT
			LEFT JOIN 	PROJECT_TYPE PT ON PT.ID = P.PROJECT_TYPE
      LEFT JOIN (SELECT TP.ID
                    FROM 		TASK TP
                    JOIN    ALLOCATION ALP ON (ALP.TASK = TP.ID AND ALP.RESOURCE_T = :userId)
                    JOIN 	PROJECT P 		ON P.ID = TP.PROJECT
                    WHERE 	(TP.IS_ISSUE = 'N' OR TP.IS_ISSUE IS NULL )
                    AND 	   TP.STATUS = 'A'
                    AND 	   P.STATUS = 'A'
                    AND        TP.PERCENT_COMPLETED <= 99)TP ON(TP.ID = T.PARENT)
			WHERE 	(T.IS_ISSUE = 'N' OR T.IS_ISSUE IS NULL )
			AND 	T.STATUS = 'A'
			AND 	P.STATUS = 'A'
            AND     AL.ID IS NOT NULL
			GROUP BY P.ID ,T.ID, TP.ID, T.NAME, T.ESTIMATED_BEGIN_DATE, T.ESTIMATED_END_DATE, T.PROJECT, T.PERCENT_COMPLETED, PT.USETIMEPLANNING
      UNION
      SELECT 	P.ID, NULL AS PARENT, P.NAME AS TASK_NAME,P.ID AS PROJECT,
          NULL AS ESTIMATED_BEGIN_DATE,
          NULL AS PROJECT,
					NULL AS PERCENT_COMPLETED,
					NULL AS ESTIMATED_END_DATE
			FROM 		TASK T
			LEFT JOIN 	ALLOCATION AL 	ON AL.TASK = T.ID AND AL.RESOURCE_T = :userId
			LEFT JOIN 	PROJECT P 		ON P.ID = T.PROJECT
			LEFT JOIN 	PROJECT_TYPE PT ON PT.ID = P.PROJECT_TYPE
			WHERE 	(T.IS_ISSUE = 'N' OR T.IS_ISSUE IS NULL )
			AND 	T.STATUS = 'A'
			AND 	P.STATUS = 'A'
            AND     AL.ID IS NOT NULL
			GROUP BY P.ID, P.NAME

SQL;
		$params = array(
			'userId' => $userId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function filterTypeResponse(){
        return "'".implode("','",\Service\TaskPrecursor::getGeneratedTypRelationship())."'";
    }

	public function getTaskDataForTimeControl($taskId,$userId){
        $managerType = \Service\ResourceProject::MANAGER;
        $filterTypeResponse = $this->filterTypeResponse();
		$query = <<<SQL
			SELECT 	T.ID,T.ID AS TASK_ID,T.NAME AS TASK_NAME, TT.LABEL AS TASK_TYPE_LABEL, TSA.LABEL AS TASK_STATUS_LABEL, T.DURATION, T.DESCRIPTION,
					TO_CHAR(T.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,T.RESPONSE,
					TO_CHAR(T.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE,PT.USETIMEPLANNING AS USE_TIME_PLANNING,
                    TT.CONTROL AS TASK_TYPE,AL.ID AS ALLOCATION,PR.NAME AS PROJECT_NAME, PR.ID AS PROJECT_ID,TSP.TASK_CAN_START,AL.RESOURCE_T,
                    TSA.CONTROL AS TASK_STATUS,PT.ALLOWENDTASK, T.URL_TO_TEST, PT.EXTERNALTYPE, TR.TYPE AS TYPE_RELATIONSHIP,
                    ALO.RESOURCE_T AS RESOURCE_GENERATOR_TASK, RP.RESOURCE_C AS MANAGER_PROJECT, RPR.NAME AS MANAGER_PROJECT_NAME,
                    TSM.CONTROL AS CONTROL_TASK_MR, RCB.NAME AS CREATED_BY, T.CLASSIFICATION,T.PARENT,
                    CASE WHEN TR.TYPE = 'M' AND TRC.TYPE = 'C' THEN 'Y' ELSE 'N' END AS MR_GENERATED_BY_CONFLICT,
                    CASE WHEN (TR.TYPE = 'M' AND TRC.TYPE = 'R') OR TOR.TASK IS NOT NULL THEN 'Y' ELSE 'N' END AS MR_GENERATED_BY_REPLICATION,
                    CASE WHEN TR.TYPE = 'G' AND TGR.TASK IS NOT NULL THEN 'Y' ELSE 'N' END AS ERROR_MR_GEN_BY_REPLICATION,
                    SR.ID AS SUBRELEASE_ID, SR.NAME AS SUBRELEASE_NAME,T.PERCENT_COMPLETED,
                    PT.EXTERNALTYPE AS PROJECT_EXTERNALTYPE, TRE.RESPONSE AS TASK_RESPONSE,
                    PRL.PROJECT AS ORIGIN_TEST_PROJECT_ID, PRO.NAME AS ORIGIN_TEST_PROJECT_NAME, ERR.DESCRIPTION AS ERROR_CLASSIFICATION,
                    TA.PROJECT AS ORIGIN_PROJECT_ID, PR.IS_LTS_REPLICATION AS PROJECT_IS_LTS_REPLICATION, T.TEST_BLOCKED,
                    CASE WHEN TA.PRODUCT_VERSION_ID_DEV IS NOT NULL
                         THEN TA.PRODUCT_VERSION_ID_DEV
                         WHEN TAC.PRODUCT_VERSION_ID_DEV IS NOT NULL AND PTP.USEMERGEREQUEST = 'Y'
                         THEN TAC.PRODUCT_VERSION_ID_DEV
                         WHEN TAR.PRODUCT_VERSION_ID_DEV IS NOT NULL AND PTP.USEMERGEREQUEST = 'Y'
                         THEN TAR.PRODUCT_VERSION_ID_DEV
                         WHEN TGA.PRODUCT_VERSION_ID_DEV IS NOT NULL AND PTP.USEMERGEREQUEST = 'Y'
                         THEN TGA.PRODUCT_VERSION_ID_DEV
                         ELSE NULL
                     END AS ORIGIN_PRODUCT_VERSION_ID_DEV,
                    PA.NAME AS ORIGIN_PROJECT_NAME,CLA.CONTROL AS CLASSIFICATION_CONTROL,
                    TO_CHAR(T.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT,TF.TIME_PLANNED AS FIRST_TIME_PLANNED,
                    DECODE(PRV.ID, NULL, NULL, PRV.VERSION || '.' || PRV.REVISION) AS VERSION, PRV.ID AS PRODUCT_VERSION_ID,
                    NVL(PRV.IS_LTS, 'Y') AS IS_LTS,
                    DECODE(TK.ID, NULL, T.TIME_PLANNED, 0) AS TIME_PLANNED,
                    (CASE WHEN (TR.TYPE  = 'C' AND TRS.TYPE = 'R') THEN 'Y' ELSE 'N' END) AS CONFLICT_REPLICATION,
                    (CASE WHEN ((TSP.CONTROL = 'O' AND CLA.CONTROL IS NOT NULL ) OR TSP.CONTROL <> 'O') THEN 'N' ELSE 'Y' END  ) AS IN_HOMOLOGATION,
                    CASE WHEN EXISTS (SELECT TAS.ID
                                        FROM TASK TAS
                                        JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                        JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                        JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                        JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E' AND TAS.CLASSIFICATION = TTC.CLASSIFICATION_ID)
                                        JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                       WHERE TAS.ID = T.ID
                                         AND ISS.STATUS = 'A'
                                         AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                         AND TAS.STATUS = 'A')
                         THEN 'Y'
                         ELSE (SELECT  TTY.ALLOW_PLAY_TASK
                                 FROM TASK TAS
                                 JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                 JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                 JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                 JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                WHERE TAS.ID = T.ID
                                  AND ISS.STATUS = 'A'
                                  AND TAS.STATUS = 'A'
                                  AND ROWNUM = 1)
                     END AS ALLOW_PLAY_TASK, TSR.RESPONSE AS FIRST_TASK_RESPONSE, PR.IS_LTS_REPLICATION
			FROM TASK T
       LEFT JOIN TASK_STATUS TS ON T.TASK_STATUS = TS.ID
       LEFT JOIN TASK_TYPE TT ON TT.ID = TS.TASK_TYPE
       LEFT JOIN TASK_RELATIONSHIP TR ON (T.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
       LEFT JOIN TASK_RELATIONSHIP TRM ON TR.TASK = TRM.TASK AND TRM.TYPE = 'M'
       LEFT JOIN TASK_RELATIONSHIP TRC ON TR.TASK = TRC.TASK_RELATIONSHIP AND (TRC.TYPE = 'C' OR TRC.TYPE = 'R')
       LEFT JOIN TASK_RELATIONSHIP TRO ON T.ID = TRO.TASK_RELATIONSHIP AND TRO.TYPE = 'O'
       LEFT JOIN TASK_RELATIONSHIP TOR ON TRO.TASK = TOR.TASK_RELATIONSHIP AND TOR.TYPE = 'R'
       LEFT JOIN TASK TAR ON TOR.TASK = TAR.ID
       LEFT JOIN TASK_RELATIONSHIP TGM ON TR.TASK = TGM.TASK_RELATIONSHIP AND TGM.TYPE = 'M'
       LEFT JOIN TASK_RELATIONSHIP TGR ON TGM.TASK = TGR.TASK_RELATIONSHIP AND TGR.TYPE = 'R'
       LEFT JOIN TASK_RELATIONSHIP TGO ON TGR.TASK = TGO.TASK_RELATIONSHIP AND TGO.TYPE = 'M'
       LEFT JOIN TASK_RELATIONSHIP TRL ON TRL.TASK_RELATIONSHIP = TR.TASK
       LEFT JOIN TASK_RELATIONSHIP TRS ON TRS.TASK_RELATIONSHIP = TRL.TASK
       LEFT JOIN TASK TM ON (TM.ID = TRM.TASK_RELATIONSHIP AND TRM.TYPE IN ('C', 'E', 'R', 'M', 'G', 'T', 'D'))
       LEFT JOIN TASK_STATUS TSM ON TSM.ID = TM.TASK_STATUS
       LEFT JOIN TASK TA ON TR.TASK = TA.ID
       LEFT JOIN TASK TAC ON TRC.TASK = TAC.ID
       LEFT JOIN TASK TGA ON TGO.TASK = TGA.ID
       LEFT JOIN ALLOCATION ALO ON ALO.TASK = TA.ID
       LEFT JOIN PROJECT PA ON PA.ID = TA.PROJECT
            JOIN PROJECT PR ON (PR.ID = T.PROJECT)
            JOIN PROJECT_TYPE PTP ON PTP.ID = PR.PROJECT_TYPE
       LEFT JOIN CLASSIFICATION CLA ON(T.CLASSIFICATION = CLA.ID)
       LEFT JOIN PROJECT_RELATIONSHIP PRL ON (PR.ID = PRL.PROJECT_RELATIONSHIP AND PRL.TYPE = 'T')
       LEFT JOIN PROJECT PRO ON (PRO.ID = PRL.PROJECT)
       LEFT JOIN RESOURCE_PROJECT RP ON (RP.PROJECT = PR.ID AND RP.TYPE = '$managerType')
       LEFT JOIN RESOURCE_T RPR ON (RPR.ID = RP.RESOURCE_C)
       LEFT JOIN TASK_STATUS TSP ON(TSP.ID = PR.PROJECT_STATUS)
       LEFT JOIN PROJECT_TYPE PT ON(PR.PROJECT_TYPE = PT.ID)
       LEFT JOIN ALLOCATION AL ON(T.ID = AL.TASK AND AL.RESOURCE_T = :userId)
       LEFT JOIN TASK_STATUS TSA ON(TSA.ID = AL.TASK_STATUS)
       LEFT JOIN RESOURCE_T RCB ON(RCB.ID = T.CREATED_BY)
       LEFT JOIN SUBRELEASE SR ON SR.PROJECT_ID = PR.ID
       LEFT JOIN TASK_RELATIONSHIP TRR ON( T.ID = TRR.TASK_RELATIONSHIP AND TRR.TYPE IN ($filterTypeResponse))
       LEFT JOIN TASK TRE ON( TRE.ID = TRR.TASK AND TRR.TYPE in ($filterTypeResponse))
       LEFT JOIN TASK_RELATIONSHIP TRF ON (T.ID = TRF.TASK AND TRF.TYPE = 'F')
       LEFT JOIN TASK TF ON TRF.TASK_RELATIONSHIP = TF.ID
       LEFT JOIN ERROR_CLASSIFICATION ERR ON T.ERROR_CLASSIFICATION_ID = ERR.ID
       LEFT JOIN PRODUCT_VERSION PRV ON PRV.ID = T.PRODUCT_VERSION_ID_DEV
       LEFT JOIN TASK_RELATIONSHIP TK ON (TK.TASK_RELATIONSHIP = T.ID AND TK.TYPE = 'A')
       LEFT JOIN TASK_RELATIONSHIP TRES ON TRES.TASK = T.ID AND TRES.TYPE = 'F' AND TR.TYPE = 'M'
       LEFT JOIN TASK TSR ON TSR.ID = TRES.TASK_RELATIONSHIP
		   WHERE T.ID = :taskId
SQL;
		$params = array(
			'taskId' => $taskId,
			'userId' => $userId,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	/**
	 * @param $filters
	 * @return string
	 */

	protected function createdFilters($filters)
	{
		$filterSQL = '';
		foreach ($filters as $filter) {
			$name = $filter['columnName'];
			$operator = $filter['operator'];
			$values = $filter['value'];
			if (is_array($values)) {
				$stringValues ='('. implode($values,',').')';
				$values = $stringValues;
			}
			if($operator == 'LIKE'){
				$values = "'".$values."'";

				// Prevents case-sensitive search
				$values = strtolower($values);
				$name = 'LOWER('.$name.') ';
			}
			$filterSQL .= <<<SQL
			AND $name $operator $values
SQL;
		}
		return $filterSQL;
	}

    public function findLabelsByProject($projectId = array(0),$organizationId, $userId,$filters = ''){
        if(empty($projectId)){
            $projects = '0';
        }else{
            if(is_array($projectId)){
                $projects = implode(',',$projectId);
            }else{
                $projects =  $projectId;
            }
        }
        $query = <<<SQL
		SELECT * FROM (
			SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,PRT.LABEL AS PROJECT_TYPE_LABEL
			FROM LABEL LAB
			LEFT JOIN PROJECT PRO ON(PRO.ID = LAB.PROJECT)
			LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
			WHERE LAB.ORGANIZATION = :organizationId
			AND (LAB.RESOURCE_T = :userId OR LAB.RESOURCE_T IS NULL)
			AND (LAB.PROJECT IN ( $projects ) OR LAB.PROJECT IS NULL))
			WHERE 1=1
			$filters
SQL;
        $params = array(
            'organizationId' => $organizationId,
            'userId' => $userId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

	public function findLabelByBoardIssue($projectId = array(0),$organizationId, $userId,$filters = '', $limit = null, $offset = null){

        $projects = implode(',',$projectId);

		$query = <<<SQL
		SELECT *
         FROM (SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,PRT.LABEL AS PROJECT_TYPE_LABEL
			     FROM LABEL LAB
			LEFT JOIN PROJECT PRO ON(PRO.ID = LAB.PROJECT)
			LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
			    WHERE LAB.ORGANIZATION = :organizationId
			      AND (LAB.RESOURCE_T = :userId OR LAB.RESOURCE_T IS NULL)
			      AND (LAB.PROJECT IN ( $projects ) OR LAB.PROJECT IS NULL))
		 WHERE 1=1
		$filters
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);


		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}
    public function findLabelsByFilter($organizationId, $userId){

		$query = <<<SQL
		SELECT * FROM(
			SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,PRT.LABEL AS PROJECT_TYPE_LABEL
			FROM LABEL LAB
            LEFT JOIN PROJECT PRO ON (PRO.ID = LAB.PROJECT)
			LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
			WHERE LAB.ORGANIZATION = :organizationId
			AND (LAB.RESOURCE_T = :userId OR LAB.RESOURCE_T IS NULL))
			WHERE 1 = 1
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}


	/************************************************************
	 * Gets all issues related to a specified task.
	 * @param int $taskId
	 * @return array $issues
	 ************************************************************/
	public function findIssuesRelatedToTask($taskId, $organizationId, $conditions){
		$query = <<<SQL
    		SELECT *
    		  FROM (SELECT TI.ID, TI.NAME AS ISSUE_NAME, TT.LABEL AS ISSUE_STATUS, CLA.LABEL AS ISSUE_CLASSIFICATION, CLA.ID AS ISSUE_CLASSIFICATION_ID,TI.PRODUCT_VERSION_ID_DEV
            		  FROM TASK_RELATIONSHIP TR
            	INNER JOIN TASK TI ON (TR.TASK = TI.ID)
            	INNER JOIN TASK_STATUS TS ON (TI.TASK_STATUS = TS.ID)
            	INNER JOIN TASK_TYPE TT ON (TS.TASK_TYPE = TT.ID)
            	 LEFT JOIN CLASSIFICATION CLA ON(TI.CLASSIFICATION = CLA.ID)
            		 WHERE TR.TASK_RELATIONSHIP = :taskId
            		   AND TR.TYPE = 'I'
            		   AND TR.ORGANIZATION = :organizationId)
            		 WHERE 1 = 1
               $conditions
    ORDER BY ID
SQL;
		$queryParams = array(
			'taskId' => $taskId,
			'organizationId' => $organizationId
		);

		$issues = $this->getEntityManager()->getConnection()->fetchAll($query, $queryParams);
		return $issues;
	}


	/************************************************************
	 * Gets all comments related to a specified issue.
	 * @param int 		$issueId
	 * @return array 	$comments
	 ************************************************************/
	public function findCommentsRelatedToIssue($issueId, $organizationId){
		$query = <<<SQL
		SELECT      COM.ID AS COMMENT_ID,
					COM.DESCRIPTION AS COMMENT_DESC,
					RES.ID AS RESOURCE_ID,
					RES.NAME AS RESOURCE_NAME,
					COM.CREATED_AT AS CREATION_DATE,
					TO_CHAR(COM.CREATED_AT, 'YYYY-MM-DD HH24:MI:SS') AS CREATED_AT_FORMATTED
		FROM        COMMENTS COM
		INNER JOIN  RESOURCE_T RES  ON (RES.ID = COM.CREATED_BY)
		WHERE       COM.ORGANIZATION = :organizationId
		AND         COM.TASK = :issueId
		GROUP BY    COM.ID, COM.DESCRIPTION, RES.ID, RES.NAME,
					COM.CREATED_AT
        ORDER BY    COM.CREATED_AT DESC
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'issueId' => $issueId
		);
		$comments = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
		return $comments;
	}

    /**
     * Gets all generated tasks not concluded related to a specified task.
     * @param $task
     * @return array
     */
    public function getTasksGeneratedNotConcluded($task){
		$query = <<<SQL
		SELECT COUNT(*) AS COUNT_TASKS
          FROM TASK T
    INNER JOIN TASK_RELATIONSHIP TR ON T.ID = TR.TASK
    INNER JOIN TASK TG ON TG.ID = TR.TASK_RELATIONSHIP
    INNER JOIN TASK_STATUS TS ON TS.ID = TG.TASK_STATUS
         WHERE T.ID = :TASK
           AND TS.CONTROL <> :CONCLUDED_STATUS
           AND TR.TYPE IN ('C', 'G', 'E', 'R')
SQL;
		$params = array(
			'TASK' => $task,
			'CONCLUDED_STATUS' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED
		);

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getStartedAllocationsByTaskId($taskId, $resource) {
        $query = <<<SQL
		SELECT A.ID, A.TASK, A.RESOURCE_T, A.TASK_STATUS,
		       TS.LABEL AS TASK_STATUS_LABEL, R.NAME AS RESOURCE_NAME
          FROM ALLOCATION A
          LEFT JOIN TASK_STATUS TS ON TS.ID = A.TASK_STATUS
          LEFT JOIN RESOURCE_T R ON R.ID = A.RESOURCE_T
         WHERE A.TASK = :TASK
           AND A.RESOURCE_T <> :RESOURCE
           AND TS.CONTROL = 'D'
SQL;

        $params = array(
            'TASK' => $taskId,
            'RESOURCE' => $resource
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getProjectTasksInitialized($resourceId, $projectId){
        $query = <<<SQL
       SELECT T.ID, T.NAME, P.ID AS PROJECT_ID, P.NAME AS PROJECT_NAME
         FROM TASK T
        INNER JOIN PROJECT P ON T.PROJECT = P.ID
        INNER JOIN ALLOCATION A ON A.TASK = T.ID
        INNER JOIN TASK_STATUS S ON A.TASK_STATUS = S.ID
        WHERE A.RESOURCE_T = :RESOURCE_ID
          AND T.PROJECT = :PROJECT
          AND P.STATUS = 'A'
          AND S.CONTROL = :CONTROL
          AND T.STATUS <> 'D'
          AND T.IS_ISSUE = 'N'
SQL;
        $param = array(
            'RESOURCE_ID' => $resourceId,
            'PROJECT'     => $projectId,
            'CONTROL'     => self::CONTROL_TASK_STATUS_IN_DEVELOPMENT
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function getRelatedResourcesEmailsByIssueId($issueId, $currentUserId){
        $query = <<<SQL
        SELECT DISTINCT RES.EMAIL
        FROM            ALLOCATION ALLOC
        LEFT JOIN       RESOURCE_T RES ON (ALLOC.RESOURCE_T = RES.ID AND ALLOC.TYPE IN (:follower, :reporter))
        WHERE           ALLOC.TASK = :issueId
        AND             RES.ID <> :currentUserId
        AND             RES.STATUS = 'A'
        AND             RES.EMAIL IS NOT NULL
SQL;

        $param = array(
            'issueId' => $issueId,
            'currentUserId' => $currentUserId,
            'follower' => self::RESOURCE_FOLLOWER,
            'reporter' => self::RESOURCE_REPORTER
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }
    public function getManagersEmailsByIssueId($issueId, $currentUserId){
        $query = <<<SQL
        SELECT DISTINCT RES.EMAIL
        FROM            TASK TAS
        JOIN            PROJECT PRO ON (PRO.ID = TAS.PROJECT)
        LEFT JOIN       RESOURCE_PROJECT REP ON (REP.PROJECT = PRO.ID AND REP.TYPE = :managerType)
        LEFT JOIN       RESOURCE_T RES ON (RES.ID = REP.RESOURCE_C)
        WHERE           TAS.ID = :issueId
        AND             RES.ID <> :currentUserId
        AND             RES.STATUS = 'A'
        AND             RES.EMAIL IS NOT NULL
SQL;

        $param = array(
            'issueId' => $issueId,
            'currentUserId' => $currentUserId,
            'managerType' => Entities\ResourceProject::MANAGER,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }

    public function findTasksNotConcludedByProjectDevOps($projectId){
        $query = <<<SQL
        SELECT TA.NAME, TA.ID, PO.ID AS PROJECT_ID, PO.NAME AS PROJECT_NAME
          FROM TASK TA, PROJECT PO
         WHERE TA.PERCENT_COMPLETED < 100
           AND TA.STATUS = 'A'
           AND TA.IS_PARENT = 'N'
           AND TA.PROJECT = :projectId
           AND PO.ID = TA.PROJECT
           AND PO.STATUS = 'A'
         UNION
        SELECT TT.NAME, TT.ID, PJ.ID AS PROJECT_ID, PJ.NAME AS PROJECT_NAME
          FROM PROJECT_RELATIONSHIP PR, TASK TT, PROJECT PJ
         WHERE PR.PROJECT = :projectId
           AND PR.TYPE = :type
           AND TT.PROJECT = PR.PROJECT_RELATIONSHIP
           AND TT.PERCENT_COMPLETED < 100
           AND TT.STATUS = 'A'
           AND TT.IS_PARENT = 'N'
           AND PJ.ID = TT.PROJECT
           AND PJ.STATUS = 'A'
SQL;

        $param = array(
            'projectId' => $projectId,
            'type' => Entities\ProjectRelationship::TYPE_TEST
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }

    public function findTaskBranchesByProject($projectId){
        $query = <<<SQL
        SELECT *
          FROM (SELECT NVL(TB.ID, TC.ID) AS ID, NVL(TB.NAME, TC.NAME) AS NAME
                  FROM TASK TA
                  LEFT JOIN TASK_BRANCHES TB ON TB.TASK = TA.ID
                  LEFT JOIN TASK_RELATIONSHIP TR ON (TR.TASK = TA.ID OR TR.TASK_RELATIONSHIP = TA.ID)
                                                AND (TR.TYPE = :RELATED_TYPE)
                  LEFT JOIN TASK_BRANCHES TC ON TC.TASK = TR.TASK
                 WHERE TA.STATUS = :ACTIVE
                   AND TA.PROJECT = :PROJECT)
         WHERE ID IS NOT NULL
SQL;

        $param = array(
            'PROJECT' => $projectId,
            'RELATED_TYPE' => Entities\TaskRelationship::RELATED_TASK_TYPE,
            'ACTIVE' => Entities\Task::ACTIVE,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }

    /***********************************************************************
     * Gets the specified issue logs.
     ***********************************************************************/
    public function findIssueLogByIssueId($issueId, $organizationId, $filter = ' ')
    {
        $query = <<<SQL
        SELECT      TL.ID,
                    RES.NAME AS MODIFIED_BY,
                    ( CASE  WHEN TL.LOG_TYPE LIKE 'UPDATE' THEN 'Modificação' END ) LOG_TYPE,
                    ( CASE
                      WHEN TL.COLUMN_NAME LIKE 'DESCRIPTION'  THEN 'Descrição'
                      WHEN TL.COLUMN_NAME LIKE 'TASK_STATUS'  THEN 'Status'
                      WHEN TL.COLUMN_NAME LIKE 'REPORTER'     THEN 'Reportado por'
                    END) COLUMN_NAME,
                    ( CASE
                      WHEN TL.COLUMN_NAME LIKE 'REPORTER'
                        THEN (SELECT R.NAME FROM RESOURCE_T R WHERE R.ID = TL.OLD_VALUE)
                      WHEN TL.COLUMN_NAME LIKE 'TASK_STATUS'
                        THEN (SELECT TS.LABEL FROM TASK_STATUS TS WHERE TS.ID = TL.OLD_VALUE)
                      ELSE TL.OLD_VALUE
                    END) OLD_VALUE,
                    ( CASE
                      WHEN TL.COLUMN_NAME LIKE 'REPORTER'
                        THEN (SELECT R.NAME FROM RESOURCE_T R WHERE R.ID = TL.NEW_VALUE)
                      WHEN TL.COLUMN_NAME LIKE 'TASK_STATUS'
                        THEN (SELECT TS.LABEL FROM TASK_STATUS TS WHERE TS.ID = TL.NEW_VALUE)
                      ELSE TL.NEW_VALUE
                    END) NEW_VALUE,
                    TO_CHAR(TL.UPDATED_AT, 'DD/MM/YYYY HH24:MI:SS') AS MODIFIED_AT,
                    TO_CHAR(TL.UPDATED_AT, 'YYYYMMDDHH24MISS') AS ORDERBY
        FROM        TASK_LOG TL
        LEFT JOIN   RESOURCE_T RES ON (RES.ID = TL.UPDATED_BY)
        WHERE       TL.ORGANIZATION = :organizationId
        AND         TL.TASK = :issueId
        ORDER BY    ORDERBY DESC
SQL;
        $params = array(
            'organizationId' => $organizationId,
            'issueId' => $issueId
        );
        $query = 'SELECT * FROM('.$query.') WHERE 1 = 1'.$filter;
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function findTasksByBranches($strBranches, $projectId,$taskNotIsCompletedOfIssues = array(0), $notInTaskNotIsCompletedOfIssues = true) {
        $not = '';
        if($notInTaskNotIsCompletedOfIssues){
            $not = 'NOT';
        }
        if(empty($taskNotIsCompletedOfIssues)){
            $taskNotIsCompletedOfIssues = array(0);
        }
        $query = <<<SQL
        SELECT TA.ID, TA.NAME, TA.DESCRIPTION, TA.CLASSIFICATION AS TASK_CLASSIFICATION
          FROM TASK TA,
               (SELECT TR.TASK, TA.PROJECT
                  FROM TASK_RELATIONSHIP TR
                 INNER JOIN TASK TA ON TA.ID = TR.TASK
                 WHERE TR.TYPE = :RELATED_TYPE
                   AND TA.STATUS <> 'D'
                   AND TR.TASK IN (SELECT TB.TASK
                                     FROM TASK_BRANCHES TB
                                    WHERE TB.NAME IN ($strBranches))
                UNION
                SELECT TR.TASK_RELATIONSHIP, TA.PROJECT
                  FROM TASK_RELATIONSHIP TR
                 INNER JOIN TASK TA ON TA.ID = TR.TASK_RELATIONSHIP
                 WHERE TR.TYPE = :RELATED_TYPE
                   AND TA.STATUS <> 'D'
                   AND TR.TASK IN (SELECT TB.TASK
                                     FROM TASK_BRANCHES TB
                                    WHERE TB.NAME IN ($strBranches))
                 UNION
                 SELECT TB.TASK, TA.PROJECT
                   FROM TASK_BRANCHES TB
                  INNER JOIN TASK TA ON TA.ID = TB.TASK
                  WHERE TB.NAME IN ($strBranches)
                    AND TA.STATUS <> 'D') TR
         WHERE TA.STATUS = 'A'
           AND TA.ID NOT IN (SELECT TRL.TASK
                               FROM TASK_RELATIONSHIP TRL
                               JOIN (SELECT TASK , MAX(ID) AS ID
                                       FROM TASK_RELATIONSHIP
                                      WHERE TYPE IN ('T', 'E', 'D')
                                   GROUP BY TASK) TAR ON (TAR.ID = TRL.ID)
                               JOIN TASK TS ON TS.ID = TRL.TASK_RELATIONSHIP
                              WHERE TRL.TYPE <> 'D'
                                AND TS.STATUS <> 'D')
           AND TA.ID = TR.TASK
           AND TR.PROJECT = :PROJECT
           AND TA.ID $not IN (:NOT_IN_TASK_ID)
SQL;
        $params = array(
            'PROJECT'        => $projectId,
            'RELATED_TYPE'   => Entities\TaskRelationship::RELATED_TASK_TYPE,
            'NOT_IN_TASK_ID' => $taskNotIsCompletedOfIssues,
        );
        $paramTypes = array(
            'NOT_IN_TASK_ID' => Connection::PARAM_INT_ARRAY
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params,$paramTypes);
        return $result;
    }

    public function findConflictTaskGeneratedByBranch($branch) {
        $query = <<<SQL
        SELECT TC.ID, TC.NAME
          FROM TASK TA
         INNER JOIN TASK_BRANCHES TB ON TB.TASK = TA.ID
         INNER JOIN TASK_RELATIONSHIP TR ON TR.TASK = TA.ID AND TR.TYPE = 'D'
         INNER JOIN TASK TC ON TC.ID = TR.TASK_RELATIONSHIP
         WHERE TA.STATUS = 'A'
           AND TC.PERCENT_COMPLETED < 100
           AND TB.NAME = '$branch'
SQL;
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }

    public function getIssuesPartialCompletedBySubrelease($subreleaseId, $organizationId = null){
        $query = <<<SQL
        SELECT TSI.NAME AS ISSUE_NAME, TSI.ID AS ISSUE_ID
          FROM SUBRELEASE SRE
         INNER JOIN BRANCH BRA ON (BRA.SUBRELEASE_ID = SRE.ID)
         INNER JOIN TASK_BRANCHES TBR ON (TBR.NAME = BRA.NAME)
         INNER JOIN TASK_RELATIONSHIP TRL ON TRL.TASK_RELATIONSHIP = TBR.TASK
                                         AND TRL.TYPE = 'I'
         INNER JOIN TASK_RELATIONSHIP TRI ON TRI.TASK = TRL.TASK
                                         AND TRI.TYPE = 'I'
         INNER JOIN TASK TSI ON TSI.ID = TRI.TASK
         INNER JOIN TASK TSK ON TSK.ID = TRI.TASK_RELATIONSHIP
         WHERE SRE.ID = :SUBRELEASE
           AND SRE.ORGANIZATION_ID = :ORGANIZATION
           AND TSK.STATUS = 'A'
           AND TSK.PERCENT_COMPLETED < 100
         GROUP BY TSI.NAME, TSI.ID
         ORDER BY TSI.ID
SQL;
        $params = array(
            'SUBRELEASE' => $subreleaseId,
            'ORGANIZATION' => $organizationId
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function getRelatedIssuesNotHomologatedBySubrelease($subreleaseId){
        $query = <<<SQL
        SELECT TAK.NAME AS ISSUE_NAME, TAK.ID AS ISSUE_ID
          FROM SUBRELEASE SRE
         INNER JOIN BRANCH BRA ON (BRA.SUBRELEASE_ID = SRE.ID)
         INNER JOIN TASK_BRANCHES TBR ON (TBR.NAME = BRA.NAME)
         INNER JOIN TASK_RELATIONSHIP TRL ON TRL.TASK_RELATIONSHIP = TBR.TASK
                                         AND TRL.TYPE = :RELATIONSHIP_ISSUE_TYPE
         INNER JOIN TASK_RELATIONSHIP TRI ON TRI.TASK = TRL.TASK
                                         AND TRI.TYPE = :ISSUE_WITH_ISSUE_TYPE
         INNER JOIN TASK TAK ON TAK.ID        = TRI.TASK
         INNER JOIN TASK TAS ON TAS.ID        = TRI.TASK_RELATIONSHIP
         INNER JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
         INNER JOIN TASK_TYPE TTY ON TTY.ID   = TST.TASK_TYPE
         WHERE SRE.ID = :SUBRELEASE
           AND TAS.STATUS = :TASK_ACTIVE
           AND TAS.IS_ISSUE = :TASK_IS_ISSUE
           AND TTY.CONTROL <> :HOMOLOGATED_CONTROL
         GROUP BY TAK.NAME, TAK.ID
         ORDER BY TAK.ID
SQL;
        $params = array(
            'RELATIONSHIP_ISSUE_TYPE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'ISSUE_WITH_ISSUE_TYPE' => Entities\TaskRelationship::ISSUE_WITH_ISSUE_TYPE,
            'SUBRELEASE' => $subreleaseId,
            'TASK_ACTIVE' => Entities\Task::ACTIVE,
            'TASK_IS_ISSUE' => Entities\Task::IS_ISSUE,
            'HOMOLOGATED_CONTROL' => Entities\TaskType::CONTROL_HOMOLOGATED,
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }


    public function getBudgetProduct($issueId){
        $query = <<<SQL
        SELECT pr.id, pr.name as BUDGET_PRODUCT
          FROM CP_CALL CP
          JOIN PROJECT PR ON CP.PRODUCT = PR.ID
          WHERE CALL_RELATIONSHIP = $issueId
SQL;


        $result = $this->getEntityManager()->getConnection()->fetchAll($query)[0];
        return $result;
    }

    public function findTaskByBranch($branchName){
        $query = <<<SQL
        SELECT ID, NAME, BRANCH_NAME, TASK_CLASSIFICATION, PROJECT, PROJECT_NAME, PARENT, PARENT_NAME,
               LISTAGG(RESOURCE_T, ', ') WITHIN GROUP (ORDER BY RESOURCE_T) AS RESOURCES
          FROM (SELECT TA.ID, AL.RESOURCE_T, TA.NAME, TB.NAME AS BRANCH_NAME, TA.CLASSIFICATION AS TASK_CLASSIFICATION,
                       TA.PROJECT, PR.NAME AS PROJECT_NAME, TA.PARENT, TS.NAME AS PARENT_NAME
                  FROM TASK TA
            INNER JOIN TASK_BRANCHES TB ON TB.TASK = TA.ID
             LEFT JOIN ALLOCATION AL ON AL.TASK = TA.ID
            INNER JOIN PROJECT PR ON PR.ID = TA.PROJECT
             LEFT JOIN TASK TS ON TS.ID = TA.PARENT
                 WHERE TA.STATUS = 'A'
                   AND TB.NAME = '$branchName'
              GROUP BY TA.ID, TA.NAME, TB.NAME, TA.CLASSIFICATION, AL.RESOURCE_T, TA.PROJECT, PR.NAME, TA.PARENT, TS.NAME)
      GROUP BY ID, NAME, BRANCH_NAME, TASK_CLASSIFICATION, PROJECT, PROJECT_NAME, PARENT, PARENT_NAME
SQL;

        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }

    public function getTestTasksByProject($projectId){
        $query = <<<SQL
        SELECT TT.ID, TT.NAME, TT.PARENT AS ID_PARENT, TP.NAME AS PARENT_NAME,
               TT.PROJECT AS PROJECT_ID, PR.NAME AS PROJECT_NAME
          FROM TASK TA
         INNER JOIN TASK_RELATIONSHIP TR ON TR.TASK = TA.ID
                                        AND TR.TYPE = :TYPE
         INNER JOIN TASK TT ON TT.ID = TR.TASK_RELATIONSHIP
          LEFT JOIN TASK TP ON TP.ID = TT.PARENT
         INNER JOIN PROJECT PR ON PR.ID = TT.PROJECT
         WHERE TA.PROJECT = :PROJECT
           AND TA.STATUS = 'A'
           AND TT.STATUS = 'A'
SQL;
        $params = array(
            'TYPE' => Entities\TaskRelationship::TEST_TYPE,
            'PROJECT' => $projectId
        );

        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function findProjectTypesForFraming($issueId){
        $query = <<<SQL
         SELECT TTY.INITIAL_TASK_STATUS,TTY.ID,TTY.ORDER_TO_FRAME
         FROM TASK ISS
         JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK = ISS.ID AND TAR.TYPE = :typeIssue)
         JOIN TASK TAS ON (TAS.ID = TAR.TASK_RELATIONSHIP AND TAS.USED_IN_FRAMING = :usedInFraming)
         JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
         JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
         JOIN TASK_TYPE_PROJECT_TYPE TTC ON ( PRO.PROJECT_TYPE = TTC.PROJECT_TYPE_ID )
         JOIN TASK_TYPE TTY ON (TTC.TASK_TYPE_ID = TTY.ID)
        WHERE ISS.ID = :issueId
          AND ISS.STATUS = :statusActive
          AND TAS.STATUS = :statusActive
          ORDER BY TTY.ORDER_TO_FRAME DESC
SQL;
        $params = array(
            'issueId' => $issueId,
            'typeIssue' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'statusActive' => Utilities::STATUS_ACTIVE,
            'usedInFraming' => Entities\Task::USED_IN_FRAMING,

        );
        $return = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        return empty($return) ? null : $return[0];
    }

    public function findLastCommentDateRelatedToIssuesExceptCommentedBy($issuesIds, $exceptions = '') {

        $query = <<<SQL
            SELECT I.ID, TO_CHAR(MAX(CM.CREATED_AT), 'DD/MM/YYYY') as LASTDATE
            FROM TASK I
            LEFT JOIN COMMENTS CM on CM.TASK = I.ID
            WHERE I.ID IN ($issuesIds)
            GROUP BY I.ID
SQL;

        $params = array(
            'EXCEPTION_IDS' => $exceptions
        );

        $lastDates = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($lastDates) ? array() : $lastDates;
    }

    public function findLastAppropriationDateRelatedToIssuesAndItsTasks($issuesIds) {
        $query = <<<SQL
            SELECT I.ID, TO_CHAR(MAX(AP.UPDATED_AT), 'DD/MM/YYYY') AS LASTDATE
            FROM TASK I
            LEFT JOIN TASK_RELATIONSHIP TR ON TR.TASK = I.ID
            LEFT JOIN APPROPRIATION AP ON AP.TASK = TR.TASK_RELATIONSHIP
            WHERE I.ID IN ($issuesIds)
            GROUP BY I.ID
SQL;

        $params = array(
            'IDS' => $issuesIds,
        );

        $lastDates = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($lastDates) ? array() : $lastDates;
    }

    public function getPointsDeliveredInMonthByCurrentUser($userId, $beginDate, $endDate) {
        $query = <<<SQL
            SELECT ROUND(SUM(POINTS)/3600, 2) AS POINTS, LABEL
              FROM (
                SELECT POINT*(RATING*100/TOTAL_RESOURCES_RATING)/100 AS POINTS, LABEL
                  FROM (
                    SELECT CLA.LABEL, (SELECT SUM(RATING) FROM ALLOCATION WHERE TASK=TAS.ID) AS TOTAL_RESOURCES_RATING, RES.RATING,
                           CASE WHEN TAS.TIME_SCORE IS NULL THEN TAS.TIME_PLANNED ELSE TAS.TIME_SCORE END AS POINT
                      FROM TASK TAS
                      JOIN TASK_STATUS TST    ON TAS.TASK_STATUS = TST.ID
                      JOIN ALLOCATION ALO     ON ALO.TASK = TAS.ID
                      JOIN RESOURCE_T RES     ON RES.ID = ALO.RESOURCE_T
                      JOIN CLASSIFICATION CLA ON CLA.ID = TAS.CLASSIFICATION
                      JOIN PROJECT P          ON P.ID = TAS.PROJECT
                      JOIN PROJECT_TYPE PT    ON P.PROJECT_TYPE = PT.ID
                                             AND PT.USETIMEPLANNING = :useTimePlanning
                                             AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
                     WHERE TAS.IS_ISSUE = :notIsIssue
                       AND TST.CONTROL = :statusConcluded
                       AND RES.ID = :userId
                       AND (CLA.CONTROL IS NULL OR CLA.CONTROL IN (:classificationTestAndMerge))
                       AND (TAS.END_DATE BETWEEN  TO_DATE('$beginDate', 'dd/mm/yyyy') AND TO_DATE('$endDate', 'dd/mm/yyyy'))
                       AND NOT EXISTS (SELECT * FROM TASK_RELATIONSHIP WHERE TASK_RELATIONSHIP=TAS.ID AND TYPE=:relatedTaskType))
              GROUP BY LABEL, RATING, TOTAL_RESOURCES_RATING, POINT)
            GROUP BY LABEL
SQL;

        $params = array(
            'userId' => $userId,
            'notIsIssue' => Entities\Task::NOT_IS_ISSUE,
            'statusConcluded' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'relatedTaskType' => Entities\TaskRelationship::RELATED_TASK_TYPE,
            'useTimePlanning' => Entities\ProjectType::IS_USE_TIME_PLANNING,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
            'classificationTestAndMerge' => array(
                Entities\Classification::CONTROL_TEST,
                Entities\Classification::CONTROL_MERGE_REQUEST
            )
        );

        $paramTypes = array(
            'classificationTestAndMerge' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function getPointsToDeliveryByCurrentUser($userId)
    {
        $query = <<<SQL
         SELECT ROUND(SUM(POINTS)/3600, 2) AS POINTS, LABEL
           FROM (
            SELECT ID, POINT*(RATING*100/TOTAL_RESOURCES_RATING)/100 AS POINTS, LABEL
              FROM (
                SELECT TAS.ID, CLA.LABEL, (SELECT SUM(RATING) FROM ALLOCATION WHERE TASK=TAS.ID) AS TOTAL_RESOURCES_RATING, RES.RATING,
                       CASE WHEN TAS.TIME_SCORE IS NULL THEN TAS.TIME_PLANNED ELSE TAS.TIME_SCORE END AS POINT
                  FROM TASK TAS
            RIGHT JOIN TASK_STATUS TST     ON TAS.TASK_STATUS = TST.ID
                  JOIN ALLOCATION ALO      ON ALO.TASK = TAS.ID
                  JOIN CLASSIFICATION CLA  ON CLA.ID = TAS.CLASSIFICATION
                  JOIN RESOURCE_T RES      ON RES.ID = ALO.RESOURCE_T
            INNER JOIN RESOURCE_PROJECT RP ON RP.RESOURCE_C = RES.ID
            INNER JOIN PROJECT P           ON RP.PROJECT = P.ID AND P.ID = TAS.PROJECT
            INNER JOIN PROJECT_TYPE PT     ON P.PROJECT_TYPE = PT.ID
                                          AND PT.USETIMEPLANNING = :useTimePlanning
                                          AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
            INNER JOIN TASK_STATUS TSP 	   ON P.PROJECT_STATUS = TSP.ID
                 WHERE TAS.IS_ISSUE = :notIsIssue
                   AND TAS.IS_PARENT <> :isParent
                   AND RES.ID = :userId
                   AND PT.ALLOWENDTASK = :isAllowEndTask
                   AND TAS.PERCENT_COMPLETED < 100
                   AND TSP.CONTROL <> :statusCanceled
                   AND TST.CONTROL NOT IN (:statusCanceledAndConcluded)
                   AND (CLA.CONTROL IS NULL OR CLA.CONTROL IN (:classificationTestAndMerge))
                   AND PT.EXTERNALTYPE NOT IN (:externalTypeMeeting)
                   AND NOT EXISTS (SELECT * FROM TASK_RELATIONSHIP WHERE TASK_RELATIONSHIP=TAS.ID AND TYPE=:relatedTaskType)
                   )
           GROUP BY ID, LABEL, RATING, TOTAL_RESOURCES_RATING, POINT)
            GROUP BY LABEL
SQL;

        $params = array(
            'userId'     => $userId,
            'notIsIssue' => Entities\Task::NOT_IS_ISSUE,
            'isParent'   => Entities\Task::IS_PARENT,
            'statusCanceled' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'statusCanceledAndConcluded' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED
            ),
            'classificationTestAndMerge' => array(
                Entities\Classification::CONTROL_TEST,
                Entities\Classification::CONTROL_MERGE_REQUEST
            ),
            'externalTypeMeeting' => array(
                'meeting' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                'meetingOccurrences' => Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES
            ),
            'useTimePlanning' => Entities\ProjectType::IS_USE_TIME_PLANNING,
            'isAllowEndTask' => Entities\ProjectType::IS_ALLOW_END_TASK,
            'relatedTaskType' => Entities\TaskRelationship::RELATED_TASK_TYPE,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
        );

        $paramTypes = array(
            'statusCanceledAndConcluded' => Connection::PARAM_STR_ARRAY,
            'classificationTestAndMerge' => Connection::PARAM_STR_ARRAY,
            'externalTypeMeeting' => Connection::PARAM_STR_ARRAY

        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function getHoursDeliveredInTheMonthByCurrentUser($userId, $beginDate, $endDate)
    {
        $query = <<<SQL
            SELECT SUM(HOURS) AS HOURS, LABEL
              FROM (SELECT CLA.LABEL AS LABEL, TAS.TIME_PLANNED AS HOURS
              FROM TASK TAS
              JOIN TASK_STATUS TST    ON TAS.TASK_STATUS = TST.ID
              JOIN ALLOCATION ALO     ON ALO.TASK = TAS.ID
              JOIN RESOURCE_T RES     ON RES.ID = ALO.RESOURCE_T
              JOIN CLASSIFICATION CLA ON CLA.ID = TAS.CLASSIFICATION
              JOIN PROJECT P          ON P.ID = TAS.PROJECT
              JOIN PROJECT_TYPE PT    ON P.PROJECT_TYPE = PT.ID
             WHERE TAS.IS_ISSUE = :notIsIssue
               AND TST.CONTROL = :statusConcluded
               AND RES.ID = :userId
               AND TAS.STATUS = :activeTask
               AND PT.ALLOWENDTASK = :isAllowEndTask
               AND (CLA.CONTROL IS NULL OR CLA.CONTROL IN (:classificationTestAndMerge))
               AND PT.EXTERNALTYPE NOT IN (:externalTypeMeeting)
               AND (TAS.END_DATE BETWEEN  TO_DATE(:beginDate, 'dd/mm/yyyy') AND TO_DATE(:endDate, 'dd/mm/yyyy'))
               AND NOT EXISTS (SELECT * FROM TASK_RELATIONSHIP WHERE TASK_RELATIONSHIP=TAS.ID AND TYPE = :relatedTaskType))
          GROUP BY LABEL
SQL;

        $params = array(
            'userId' => $userId,
            'beginDate' => $beginDate,
            'endDate' => $endDate,
            'activeTask' => Entities\Task::ACTIVE,
            'notIsIssue' => Entities\Task::NOT_IS_ISSUE,
            'isAllowEndTask' => Entities\ProjectType::IS_ALLOW_END_TASK,
            'statusConcluded' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'relatedTaskType' => Entities\TaskRelationship::RELATED_TASK_TYPE,
            'useTimePlanning' => Entities\ProjectType::IS_USE_TIME_PLANNING,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
            'externalTypeMeeting' => array(
                Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
            ),
            'classificationTestAndMerge' => array(
                Entities\Classification::CONTROL_TEST,
                Entities\Classification::CONTROL_MERGE_REQUEST
            )
        );

        $paramTypes = array(
            'classificationTestAndMerge' => Connection::PARAM_STR_ARRAY,
            'externalTypeMeeting' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);


    }

    public function getPlannedHoursByCurrentUser($userId)
    {
        $query = <<<SQL
            SELECT SUM(PLANNED_HOURS) HOURS, LABEL
              FROM (SELECT TAS.TIME_PLANNED AS PLANNED_HOURS, CLA.LABEL AS LABEL
                      FROM TASK TAS
                RIGHT JOIN TASK_STATUS TST     ON TAS.TASK_STATUS = TST.ID
                      JOIN ALLOCATION ALO      ON ALO.TASK = TAS.ID
                      JOIN CLASSIFICATION CLA  ON CLA.ID = TAS.CLASSIFICATION
                      JOIN RESOURCE_T RES      ON RES.ID = ALO.RESOURCE_T
                INNER JOIN RESOURCE_PROJECT RP ON RP.RESOURCE_C = RES.ID
                INNER JOIN PROJECT P           ON RP.PROJECT = P.ID AND P.ID = TAS.PROJECT
                INNER JOIN PROJECT_TYPE PT     ON P.PROJECT_TYPE = PT.ID
                INNER JOIN TASK_STATUS TSP     ON P.PROJECT_STATUS = TSP.ID
                     WHERE TAS.IS_ISSUE = :notIsIssue
                       AND TAS.IS_PARENT <> :isParent
                       AND TAS.STATUS = :activeTask
                       AND RES.ID = :userId
                       AND PT.ALLOWENDTASK = :isAllowEndTask
                       AND TAS.PERCENT_COMPLETED < 100
                       AND TSP.CONTROL NOT IN (:statusCanceledAndConcluded)
                       AND TST.CONTROL NOT IN (:statusCanceledAndConcluded)
                       AND (CLA.CONTROL IS NULL OR CLA.CONTROL IN (:classificationTestAndMerge))
                       AND PT.EXTERNALTYPE NOT IN (:externalTypeMeeting)
                       AND NOT EXISTS (SELECT * FROM TASK_RELATIONSHIP WHERE TASK_RELATIONSHIP = TAS.ID AND TYPE = :relatedTaskType))
                  GROUP BY LABEL
SQL;

        $params = array(
            'userId'     => $userId,
            'activeTask' => Entities\Task::ACTIVE,
            'notIsIssue' => Entities\Task::NOT_IS_ISSUE,
            'isParent'   => Entities\Task::IS_PARENT,
            'statusCanceledAndConcluded' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED
            ),
            'classificationTestAndMerge' => array(
                Entities\Classification::CONTROL_TEST,
                Entities\Classification::CONTROL_MERGE_REQUEST
            ),
            'externalTypeMeeting' => array(
                Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
            ),
            'isAllowEndTask' => Entities\ProjectType::IS_ALLOW_END_TASK,
            'relatedTaskType' => Entities\TaskRelationship::RELATED_TASK_TYPE,
        );

        $paramTypes = array(
            'statusCanceledAndConcluded' => Connection::PARAM_STR_ARRAY,
            'classificationTestAndMerge' => Connection::PARAM_STR_ARRAY,
            'externalTypeMeeting' => Connection::PARAM_STR_ARRAY

        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function getTotalBackLogIssuesByCurrentUser($userId){
        $stringProjectByManager = Utilities::getStringSelectAllProjectByUser($userId);
        $query = <<<SQL
        SELECT PRODUCT_IDS,
                (   SELECT COUNT (TAS.ID) AS TOTAL
                  FROM PROJECT P
                  JOIN PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID
                  JOIN PROJECT B ON B.PARENT = P.ID AND B.STATUS <> :deleted
                  JOIN PROJECT_TYPE BT ON BT.ID = B.PROJECT_TYPE AND BT.EXTERNALTYPE = :typeIssue
                  JOIN TASK_STATUS TS ON TS.ID = P.PROJECT_STATUS
                  JOIN TASK TAS ON (P.ID = TAS.PROJECT AND TAS.IS_ISSUE = :IS_ISSUE)
                  JOIN TASK_STATUS TTS ON (TTS.ID = TAS.TASK_STATUS AND TTS.CONTROL = :CONTROL_BACKLOG)
                 WHERE ($stringProjectByManager)
                   AND PT.EXTERNALTYPE = :product
                   AND P.STATUS = :active
                   AND TS.CONTROL = :IN_PROGRESS_STATUS
                   AND P.PRODUCT_TYPE <> 'G') AS TOTAL
         FROM (
                SELECT RTRIM(XMLAGG(XMLELEMENT(E,P.ID,',').EXTRACT('//text()') ORDER BY P.ID).GETCLOBVAL(),',') AS PRODUCT_IDS
                      FROM PROJECT P
                      JOIN	PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID
                      JOIN	PROJECT B ON B.PARENT = P.ID AND B.STATUS <> :deleted
                      JOIN	PROJECT_TYPE BT ON BT.ID = B.PROJECT_TYPE AND BT.EXTERNALTYPE = :typeIssue
                      JOIN TASK_STATUS TS ON TS.ID = P.PROJECT_STATUS
                 LEFT JOIN RESOURCE_T RES ON(RES.ID = P.CREATED_BY )
                     WHERE ($stringProjectByManager)
                       AND PT.EXTERNALTYPE = :product
                       AND P.STATUS = :active
                       AND TS.CONTROL = :IN_PROGRESS_STATUS
                       AND P.PRODUCT_TYPE <> 'G'
              )
SQL;
        $params = array(
            'active' =>  Entities\Project::IS_ACTIVE,
            'product' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_PRODUCT,
            'deleted' =>  Entities\Project::STATUS_LOGICAL_DELETED,
            'IN_PROGRESS_STATUS' =>  Entities\Project::PROJECT_IN_PROGRESS_STATUS,
            'CONTROL_BACKLOG' =>  Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG,
            'IS_ISSUE' =>  Entities\Task::IS_ISSUE,
            'typeIssue' =>  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE
        );
        $dataSet = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        foreach ($dataSet as &$row) {
            $row['PRODUCT_IDS'] = Utilities::setSelectMultiInQuery($row['PRODUCT_IDS']);
        }

        return empty($dataSet) ? null : $dataSet[0];
    }

    public function getTotalDelayedTasksByCurrentUser($userId, $dontCount = false) {
        $today = new \DateTime();
        $todayString = $today->format('d/m/Y');

        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $start = $dontCount ? "*" : "COUNT(*) AS TOTAL";
        $query = <<<SQL
        SELECT $start FROM (
		    	SELECT T.ID
                  FROM TASK_TYPE TT
            RIGHT JOIN TASK_STATUS TS 		ON TT.ID = TS.TASK_TYPE
            RIGHT JOIN TASK T 				ON TS.ID = T.TASK_STATUS
            INNER JOIN ALLOCATION A 		ON A.TASK = T.ID
            INNER JOIN RESOURCE_T RT 		ON A.RESOURCE_T = RT.ID
            INNER JOIN RESOURCE_PROJECT RP 	ON RP.RESOURCE_C = RT.ID
            INNER JOIN PROJECT P 			ON RP.PROJECT = P.ID AND P.ID = T.PROJECT
            INNER JOIN PROJECT_TYPE PT 		ON P.PROJECT_TYPE = PT.ID AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
            INNER JOIN TASK_STATUS TSP 		ON P.PROJECT_STATUS = TSP.ID
             LEFT JOIN CLASSIFICATION CLA   ON (CLA.ID = T.CLASSIFICATION)
                 WHERE RT.ID = :userId
                   AND TO_DATE('$todayString', 'DD/MM/YYYY') > T.ESTIMATED_END_DATE
                   AND T.PERCENT_COMPLETED < 100
                   AND T.IS_ISSUE = :notIsIssue
                   AND T.IS_PARENT <> :isParent
                   AND T.STATUS = 'A'
                   AND TS.CONTROL <> :statusConcluded
                   AND PT.ALLOWENDTASK = 'Y'
                   AND TSP.CONTROL <> :statusCanceled
                   AND 	(PT.USETIMEPLANNING = 'Y' AND T.TIME_PLANNED > 0)
                   AND NOT EXISTS (  SELECT TAT.ID
                                  FROM TASK TAS
                                  JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                  JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
                                  JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                  JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                  JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL = 'D' )
                                  JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                  JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
                                  JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                  JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
                                 WHERE TAS.ID = T.ID
                                   AND ISS.STATUS = 'A'
                                   AND TAT.STATUS = 'A'
                                   AND TAS.STATUS = 'A'
                                   AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                   AND TAT.PERCENT_COMPLETED < 100
                                   AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
                                   AND TAS.CLASSIFICATION NOT IN  ( SELECT TTC.CLASSIFICATION_ID
                                                                      FROM  TASK ISS
                                                                      JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                                      JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                                     WHERE ISS.ID = TAR.TASK)
                                   )
                    AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
                   $filterTypeProject
                    AND NOT EXISTS (SELECT TAR.ID, TAR.NAME
                                      FROM TASK_RELATIONSHIP TR
                                      LEFT JOIN TASK TAR ON TAR.ID = TR.TASK_RELATIONSHIP
                                      JOIN TASK TAS ON TAS.ID = TR.TASK
                                     WHERE (TR.TYPE IN ('C', 'G', 'E')
                                       AND TR.TASK = T.ID
                                       AND TAR.PERCENT_COMPLETED < 100)
                                       OR(
                                          TR.TASK_RELATIONSHIP = T.ID
                                          AND TR.TYPE = 'M'
                                          AND TAS.PERCENT_COMPLETED < 100
                                       ))
                   AND ((TSP.CONTROL = 'O' AND (CLA.CONTROL IS NOT NULL OR TS.CONTROL = 'D') ) OR TSP.CONTROL <> 'O' )
            ORDER BY T.PARENT ASC NULLS FIRST)
SQL;
        $params = array(
            'userId' => $userId,
            'notIsIssue' => Entities\Task::NOT_IS_ISSUE,
            'isParent' => Entities\Task::IS_PARENT,
            'statusCanceled' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
            'statusConcluded' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getTotalPendingTasksByCurrentUser($userId) {
        $today = new \DateTime();
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);

        $notIsIssue = Entities\Task::NOT_IS_ISSUE;
        $isParent = Entities\Task::IS_PARENT;
        $statusCanceled = Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED;
        $statusConcluded = Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED;
        $active = Entities\Task::ACTIVE;
        $PROJECT_TYPE_TIME_LINE = Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE;

        $query = <<<SQL
        SELECT COUNT(*) AS TOTAL
          FROM (
            SELECT T.ID
              FROM TASK_TYPE TT
        RIGHT JOIN TASK_STATUS TS ON TT.ID = TS.TASK_TYPE
        RIGHT JOIN TASK T ON TS.ID = T.TASK_STATUS
        INNER JOIN ALLOCATION A ON A.TASK = T.ID
        INNER JOIN RESOURCE_T RT ON A.RESOURCE_T = RT.ID
        INNER JOIN RESOURCE_PROJECT RP ON RP.RESOURCE_C = RT.ID
        INNER JOIN PROJECT P ON RP.PROJECT = P.ID AND P.ID = T.PROJECT
        INNER JOIN PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID AND PT.EXTERNALTYPE <> '$PROJECT_TYPE_TIME_LINE'
        INNER JOIN TASK_STATUS TSP ON P.PROJECT_STATUS = TSP.ID
         LEFT JOIN CLASSIFICATION CLA ON (CLA.ID = T.CLASSIFICATION)
             WHERE RT.ID = $userId
               AND T.IS_PARENT <> '$isParent'
               AND T.STATUS ='$active'
               AND T.PERCENT_COMPLETED < 100
               AND TSP.CONTROL NOT IN ('$statusCanceled','$statusConcluded')
               AND T.IS_ISSUE = '$notIsIssue'
               AND TS.CONTROL <> '$statusConcluded'
               AND PT.ALLOWENDTASK = 'Y'
               AND (PT.USETIMEPLANNING = 'N' OR T.TIME_PLANNED > 0)
               AND NOT EXISTS (  SELECT TAT.ID
                                  FROM TASK TAS
                                  JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
                                  JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
                                  JOIN TASK ISS ON (ISS.ID = TAR.TASK)
                                  JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                  JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL = 'D' )
                                  JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                  JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
                                  JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
                                  JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
                                 WHERE TAS.ID = T.ID
                                   AND ISS.STATUS = 'A'
                                   AND TAT.STATUS = 'A'
                                   AND TAS.STATUS = 'A'
                                   AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
                                   AND TAT.PERCENT_COMPLETED < 100
                                   AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
                                    AND TAS.CLASSIFICATION NOT IN  ( SELECT TTC.CLASSIFICATION_ID
                                                                      FROM  TASK ISS
                                                                      JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                                      JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                                     WHERE ISS.ID = TAR.TASK)
                               )

			   AND (T.CLASSIFICATION = CLA.ID AND (CLA.CONTROL <> 'A' OR CLA.CONTROL IS NULL))
               $filterTypeProject
                    AND NOT EXISTS (SELECT TAR.ID, TAR.NAME
                                      FROM TASK_RELATIONSHIP TR
                                      LEFT JOIN TASK TAR ON TAR.ID = TR.TASK_RELATIONSHIP
                                      JOIN TASK TAS ON TAS.ID = TR.TASK
                                     WHERE (TR.TYPE IN ('C', 'G', 'E')
                                       AND TR.TASK = T.ID
                                       AND TAR.PERCENT_COMPLETED < 100)
                                       OR(
                                          TR.TASK_RELATIONSHIP = T.ID
                                          AND TR.TYPE = 'M'
                                          AND TAS.PERCENT_COMPLETED < 100
                                       ))
               AND ((TSP.CONTROL = 'O' AND (CLA.CONTROL IS NOT NULL OR TS.CONTROL = 'D') ) OR TSP.CONTROL <> 'O' ))
SQL;

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function getTotalIssuesWithPendingConclusion($organizationId, $userId) {
		$query = <<<SQL
		SELECT count(*) AS TOTAL
         FROM (
            SELECT *
             FROM (
                SELECT I.ID
                  FROM TASK I
            INNER JOIN ALLOCATION A 	ON A.TASK = I.ID AND A.RESOURCE_T = :userId AND A.TYPE = 'R'
            INNER JOIN TASK_STATUS TS 	ON TS.ID = I.TASK_STATUS
            INNER JOIN TASK_TYPE TT 	ON TT.ID = TS.TASK_TYPE
            INNER JOIN PROJECT P 		ON P.ID = I.PROJECT
             FULL JOIN ALLOCATION AL 	ON AL.TASK = I.ID AND AL.TYPE = 'S'
             LEFT JOIN PRIORITY PRI ON(I.PRIORITY = PRI.ID)
             LEFT JOIN TASK_LABEL TL ON TL.TASK = I.ID
             LEFT JOIN COMMENTS COM ON (I.ID = COM.TASK)
             LEFT JOIN ALLOCATION ALO ON(ALO.TASK = I.ID AND ALO.RESOURCE_T = :userId)
                 WHERE I.ORGANIZATION = :organizationId
                   AND I.IS_ISSUE = 'Y'
                   AND TS.CONTROL = :statusHomologated
              GROUP BY I.ID)
            WHERE 1=1)
SQL;

		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId,
			'statusHomologated' => Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getTaskNotIsCompletedOfIssues($projectId) {
		$query = <<<SQL
		  SELECT TAS.ID

            FROM SUBRELEASE SUB
            JOIN TASK TAS ON (SUB.PROJECT_ID = TAS.PROJECT )
            JOIN TASK_RELATIONSHIP TAR ON(TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = :ISSUE_TYPE)
            JOIN TASK ISS ON(TAR.TASK = ISS.ID)
            JOIN TASK_RELATIONSHIP TAI ON(TAI.TASK = ISS.ID AND TAI.TYPE = :ISSUE_TYPE)
            JOIN TASK TA2 ON(TAI.TASK_RELATIONSHIP = TA2.ID)
            JOIN PROJECT PRO ON (PRO.ID = TA2.PROJECT)
            JOIN CLASSIFICATION CLA ON(TAS.CLASSIFICATION = CLA.ID)
            JOIN CLASSIFICATION CA2 ON(TA2.CLASSIFICATION = CA2.ID)

            WHERE SUB.PROJECT_ID = :PROJECT_ID
            AND TAS.STATUS = :STATUS
            AND TAS.IS_ISSUE = :IS_ISSUE
            AND TAS.IS_PARENT = :IS_PARENT
            AND TA2.STATUS = :STATUS
            AND TA2.IS_ISSUE = :IS_ISSUE
            AND TA2.IS_PARENT = :IS_PARENT
            AND SUB.PROJECT_ID <> TA2.PROJECT
            AND TA2.PERCENT_COMPLETED <> 100
            AND (CA2.CONTROL <> :TEST_CONTROL OR CA2.CONTROL IS NULL)
            GROUP BY TAS.ID
SQL;

		$params = array(
			'PROJECT_ID' => $projectId,
			'STATUS' => Utilities::STATUS_ACTIVE,
			'IS_ISSUE' => Entities\Task::NOT_IS_ISSUE,
			'IS_PARENT' => Entities\Task::NOT_IS_PARENT,
			'TEST_CONTROL' => Entities\Classification::CONTROL_TEST,
			'ISSUE_TYPE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
		);
		return Utilities::arrayAssociativeToArray($this->getEntityManager()->getConnection()->fetchAll($query, $params));
	}

	public function getTasksFromIssue($issueId) {

        $typeIssue = Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE;
		$query = <<<SQL
		  SELECT TTR.ID
			FROM 		TASK T
			INNER JOIN 	TASK_RELATIONSHIP TR 	ON TR.TASK = T.ID OR TR.TASK_RELATIONSHIP = T.ID
			INNER JOIN 	TASK TTR 				ON (TR.TASK = TTR.ID OR TR.TASK_RELATIONSHIP = TTR.ID) AND TTR.ID <> :issueId
			 LEFT JOIN  CLASSIFICATION C        ON ( TTR.CLASSIFICATION = C.ID)
			 JOIN PROJECT P ON (P.ID = TTR.PROJECT)
             WHERE 	T.ID = :issueId
             AND TR.TYPE = :TYPE_ISSUE
             AND TTR.STATUS = :STATUS_ACTIVE

SQL;

		$params = array(
			'issueId' => $issueId,
            'TYPE_ISSUE' => $typeIssue,
            'STATUS_ACTIVE' => Utilities::STATUS_ACTIVE,
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

    public function getLastTaskId($taskParentId)
    {

        $query = <<<SQL
		  SELECT TAS.ID
            FROM TASK TAS
       LEFT JOIN TASK_STATUS TST ON (TAS.TASK_STATUS = TST.ID AND TST.CONTROL = 'B')
           WHERE PARENT = :TASK_PARENT_ID
             AND TAS.STATUS = :STATUS_ACTIVE
             AND TST.CONTROL IS NULL
        ORDER BY TAS.ESTIMATED_END_DATE DESC, (TAS.PREVIOUS_TIME + TAS.DURATION) DESC
SQL;

        $params = array(
            'TASK_PARENT_ID' => $taskParentId,
            'STATUS_ACTIVE' => Utilities::STATUS_ACTIVE,
        );
        $return = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        return empty($return) ? null : $return[0]['ID'];
    }

    public function getFistTaskIdFromParentTask($taskParentId)
    {

        $query = <<<SQL
		  SELECT TAS.ID
            FROM TASK TAS
       LEFT JOIN TASK_STATUS TST ON (TAS.TASK_STATUS = TST.ID AND TST.CONTROL = 'B')
           WHERE PARENT = :TASK_PARENT_ID
             AND TAS.STATUS = :STATUS_ACTIVE
             AND TST.CONTROL IS NULL
        ORDER BY TAS.ESTIMATED_BEGIN_DATE, (TAS.PREVIOUS_TIME + TAS.DURATION)
SQL;

        $params = array(
            'TASK_PARENT_ID' => $taskParentId,
            'STATUS_ACTIVE' => Utilities::STATUS_ACTIVE,
        );
        $return = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        return empty($return) ? null : $return[0]['ID'];
    }

    public function validateTaskToBeStarted($taskId)
    {
        $query = <<<SQL
            SELECT 'TASK_PARENT' AS ERROR
              FROM TASK
             WHERE ID = :TASK_ID
               AND IS_PARENT = 'Y'
             UNION
            SELECT 'MERGE_REQUEST' AS ERROR
              FROM TASK_RELATIONSHIP TR
              LEFT JOIN TASK TAR ON TAR.ID = TR.TASK_RELATIONSHIP
              JOIN TASK TAS ON TAS.ID = TR.TASK
             WHERE (TR.TYPE IN ('C', 'G', 'E')
               AND TR.TASK = :TASK_ID
               AND TAR.PERCENT_COMPLETED < 100)
                OR (TR.TASK_RELATIONSHIP = :TASK_ID AND TR.TYPE = 'M' AND TAS.PERCENT_COMPLETED < 100)
             UNION
            SELECT 'PROJECT_IN_HOMOLOGATION' AS ERROR
              FROM TASK         TAS
              JOIN PROJECT      PRO ON PRO.ID = TAS.PROJECT
              JOIN TASK_STATUS  TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN CLASSIFICATION CLA ON CLA.ID = TAS.CLASSIFICATION
             WHERE TAS.ID = :TASK_ID
               AND TSP.CONTROL = 'O'
               AND CLA.CONTROL IS NULL
             UNION
            SELECT 'TIME_PLANNED' AS ERROR
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN PROJECT_TYPE PRT ON  PRT.ID = PRO.PROJECT_TYPE
             WHERE PRT.USETIMEPLANNING = 'Y'
               AND (TAS.TIME_PLANNED = 0 OR TAS.TIME_PLANNED IS NULL)
               AND TAS.ID = :TASK_ID
             UNION
            SELECT 'SPECIFICATION_PENDENCES' AS ERROR
              FROM TASK TAS
              JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP = TAS.ID AND TAR.TYPE = 'I')
              JOIN TASK_STATUS AST ON (AST.ID = TAS.TASK_STATUS AND AST.CONTROL <> 'D' )
              JOIN TASK ISS ON (ISS.ID = TAR.TASK)
              JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
              JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
              JOIN TASK_RELATIONSHIP TRT ON (TRT.TASK = ISS.ID AND TRT.TYPE = 'I')
              JOIN TASK_TYPE TTY ON (TTY.ID = TST.TASK_TYPE)
              JOIN TASK TAT ON (TRT.TASK_RELATIONSHIP = TAT.ID )
             WHERE TAS.ID = :TASK_ID
               AND ISS.STATUS = 'A'
               AND TAT.STATUS = 'A'
               AND TAS.STATUS = 'A'
               AND TTY.ALLOW_PLAY_TASK_IN_FRAME = 'N'
               AND TAT.PERCENT_COMPLETED < 100
               AND TAT.CLASSIFICATION = TTC.CLASSIFICATION_ID
               AND TAS.CLASSIFICATION NOT IN  ( SELECT TTC.CLASSIFICATION_ID
                                                  FROM  TASK ISS
                                                  JOIN TASK_STATUS TST ON (TST.ID = ISS.TASK_STATUS)
                                                  JOIN TASK_TYPE_CLASSIFICATION TTC ON (TTC.TASK_TYPE_ID = TST.TASK_TYPE AND TTC.TASK_ISSUE_CLASSIF = 'E')
                                                 WHERE ISS.ID = TAR.TASK)
SQL;

        $params = array(
            'IS_PARENT' => Entities\Task::IS_PARENT,
            'TASK_ID'   => $taskId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getScheduleRelationshipFromTask($taskId,$meetingId)
    {

        $query = <<<SQL
		    SELECT  TAR.*
            FROM  TASK_RELATIONSHIP TAR
            JOIN  TASK TRE   ON (TRE.ID = TAR.TASK_RELATIONSHIP AND TRE.PROJECT =  :MEETING_ID)
           WHERE  TAR.TYPE = 'W'
            AND TAR.TASK = :TASK_ID
            AND TRE.STATUS <> 'D'
SQL;

        $params = array(
            'TASK_ID' => $taskId,
            'MEETING_ID' => $meetingId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findTaskInformation($idTask, $date){
        $date = $date == NULL ? date("d/m/Y") : $date ;
        $query = <<<SQL
        SELECT  P.NAME AS PROJECT_NAME,
                T.NAME AS TASK_NAME,
                P.BEGIN_DATE AS ESTIMATED_BEGIN_DATE,
                T.TIME_PLANNED,
                (SELECT SUM(A.TOTAL) FROM APPROPRIATION A WHERE A.TASK = :idTask AND TO_CHAR(A.BEGIN_HOUR, 'DD/MM/YYYY') = :date) AS TIME_APPROPRIATE,
                T.PERCENT_COMPLETED,
                A.DESCRIPTION,
                A.ID AS ID_APPROPRIATION
        FROM TASK T
            INNER JOIN PROJECT P ON T.PROJECT = P.ID
            LEFT JOIN APPROPRIATION A ON (A.TASK = :idTask AND TO_CHAR(A.BEGIN_HOUR, 'DD/MM/YYYY') = :date)
        WHERE T.ID = :idTask
SQL;

		$params = array(
		    'idTask' => $idTask,
		    'date' => $date
		);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findReletedProjectIdByIssue ($issueId,$projects = '0' )
    {
        $query = <<<SQL
            SELECT TAK.PROJECT
              FROM TASK TAS
              JOIN TASK_RELATIONSHIP TRE ON TRE.TASK = TAS.ID AND TRE.TYPE = :RELATIONSHIP_ISSUE_TYPE
              JOIN TASK TAK              ON TAK.ID   = TRE.TASK_RELATIONSHIP
              JOIN PROJECT           PRO ON PRO.ID   = TAK.PROJECT
              JOIN TASK_STATUS       TST ON TST.ID   = PRO.PROJECT_STATUS
              JOIN CLASSIFICATION CLT ON (CLT.ID = TAK.CLASSIFICATION )
             WHERE TAS.ID = :ISSUE_ID
               AND TAS.STATUS = 'A'
               AND PRO.STATUS = 'A'
               AND TAK.STATUS = 'A'
               AND PRO.ID <> ($projects)
               AND ( CLT.CONTROL NOT IN (:CONTROL_CLIENT_ACCEPT,:CONTROL_SPECIFICATION, :CONTROL_TEST) OR TAK.PERCENT_COMPLETED <> 100)
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED,:TASK_STATUS_CONTROL_CANCELED)
SQL;
        $params = array(
            'ISSUE_ID' => $issueId,
            'RELATIONSHIP_ISSUE_TYPE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            'CONTROL_CLIENT_ACCEPT' => Entities\Classification::CONTROL_CLIENT_ACCEPT,
            'CONTROL_SPECIFICATION' => Entities\Classification::CONTROL_SPECIFICATION,
            'CONTROL_TEST' => Entities\Classification::CONTROL_TEST,
            'TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findVersionByIssueInRelease($issueId,$releaseId,$version)
    {
        $query = <<<SQL
            SELECT DISTINCT PRD.NAME,
             (CASE WHEN REL.ID = :RELEASE AND :VERSION IS NOT NULL
                   THEN :VERSION
                   ELSE SRE.VERSION ||'.'||SRE.REVISION||'.'||SRE.BUILD END) AS VERSION
              FROM TASK TAS
              JOIN TASK_RELATIONSHIP TRE ON TRE.TASK = TAS.ID AND TRE.TYPE = :RELATIONSHIP_ISSUE_TYPE
              JOIN TASK TAK              ON TAK.ID   = TRE.TASK_RELATIONSHIP
              JOIN PROJECT           PRO ON (PRO.ID  = TAK.PROJECT AND PRO.STATUS = 'A')
              JOIN SUBRELEASE SRE ON (SRE.PROJECT_ID = PRO.ID)
              JOIN RELEASE REL ON (REL.ID = SRE.RELEASE_ID)
              JOIN PRODUCT PRD ON (PRD.ID = REL.PRODUCT_ID)
             WHERE TAS.ID = :ISSUE_ID
               AND TAS.STATUS = 'A'
SQL;
        $params = array(
            'ISSUE_ID' => $issueId,
            'RELEASE' => $releaseId,
            'VERSION' => $version,
            'RELATIONSHIP_ISSUE_TYPE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    public function findVersionByIssueInSubRelease($issueId,$subReleaseId,$version)
    {
        $query = <<<SQL
             SELECT PRD.NAME,
                  (CASE WHEN MIN(SRE.ID) = :SUBRELEASE AND :VERSION IS NOT NULL
                        THEN :VERSION
                        ELSE MAX(SRE.VERSION) ||'.'||MAX(SRE.REVISION)||'.'||MAX(SRE.BUILD) END) AS VERSION
              FROM TASK TAS
              JOIN TASK_RELATIONSHIP TRE ON TRE.TASK = TAS.ID AND TRE.TYPE = :RELATIONSHIP_ISSUE_TYPE
              JOIN TASK TAK              ON TAK.ID   = TRE.TASK_RELATIONSHIP
              JOIN PROJECT           PRO ON (PRO.ID  = TAK.PROJECT AND PRO.STATUS = 'A')
              JOIN SUBRELEASE SRE ON (SRE.PROJECT_ID = PRO.ID)
              JOIN RELEASE REL ON (REL.ID = SRE.RELEASE_ID)
              JOIN PRODUCT PRD ON (PRD.ID = REL.PRODUCT_ID)
             WHERE TAS.ID = :ISSUE_ID
               AND TAS.STATUS = 'A'
          GROUP BY PRD.NAME
SQL;
        $params = array(
            'ISSUE_ID' => $issueId,
            'SUBRELEASE' => $subReleaseId,
            'VERSION' => $version,
            'RELATIONSHIP_ISSUE_TYPE' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE,
            'TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUserTasksGroupedByProject($userId, $currentDate)
    {
        $currentDateTime = $currentDate . ' 23:59:59';
        $query = <<<SQL
			SELECT * FROM (
            SELECT PRO.ID, 'P'||PRO.ID AS ID_TREE, PRO.NAME, '' AS PROJECT_TREE,
            	   NULL AS PERCENT_COMPLETED, NULL AS PERCENT_COMPLETED_ORIGIN,
                   'Y' AS PROJECT, 'N' AS IS_RECURRING, 'N' AS CONTRACT, 'N' AS CDA,
                   NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
             WHERE (TAS.ESTIMATED_BEGIN_DATE <= TO_DATE(:CURRENT_DATE_TIME, 'DD/MM/YYYY HH24:MI:SS')
                    OR
                    TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE)
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
          GROUP BY PRO.ID, PRO.NAME, TAS.PERCENT_COMPLETED
             UNION
            SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                   TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED, TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN,
                   'N' AS PROJECT,
                   CASE WHEN (PRT.EXTERNALTYPE = 'C')
                        THEN 'Y'
                        ELSE 'N'
                 END AS IS_RECURRING, 'N' AS CONTRACT, 'N' AS CDA,
                   NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
             WHERE ((TAS.ESTIMATED_BEGIN_DATE <= TO_DATE(:CURRENT_DATE_TIME, 'DD/MM/YYYY HH24:MI:SS')
                    AND TAS.PERCENT_COMPLETED < 100)
                    OR
                    (TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE))
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
          GROUP BY TAS.ID, TAS.NAME, TAS.PROJECT, TAS.PERCENT_COMPLETED, PRT.EXTERNALTYPE)
          UNION
      SELECT * FROM (
         SELECT CO.NRSEQCONTRATO AS ID, 'C'||CO.NRSEQCONTRATO AS ID_TREE, CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS NAME,
                NULL AS PROJECT_TREE, NULL AS PERCENT_COMPLETED, NULL AS PERCENT_COMPLETED_ORIGIN,
                'N' AS PROJECT, 'N' AS IS_RECURRING, 'Y' AS CONTRACT, 'Y' AS CDA,
                NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
           FROM RESOURCE_T            RE
           JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
           JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
           JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
           JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                                        AND TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') = :CURRENT_DATE
           JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
           JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
          WHERE RE.ID = :USER_ID
			AND RA.IDSITUARECGEND = '1'
       GROUP BY CO.NRSEQCONTRATO, CO.NRCONTRATOFCT, CO.DSREFERENCIACONT
          UNION
         SELECT IC.NRITEMOCONTRATO AS ID, 'S'||IC.NRITEMOCONTRATO AS ID_TREE, IC.NMITEMCONTRATO AS NAME,
                'C'||IC.NRSEQCONTRATO AS PROJECT_TREE,
                round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED,
                round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED_ORIGIN,
                'N' AS PROJECT, 'N' AS IS_RECURRING, 'N' AS CONTRACT, 'Y' AS CDA,
                RA.NRRECURSOSAGENDCONT, AG.NRAPROPAGENDAMENTO
           FROM RESOURCE_T            RE
           JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
           JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
           JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
           JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                                        AND TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') = :CURRENT_DATE
           JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
           JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
           JOIN (SELECT SUM(AA.QTHORASAPROPRIADAS) AS TOTAL_APPROPRIATION, AA.NRRECURSOSAGENDCONT
                   FROM FCT_APROPAGENDAMENTO AA
                  GROUP BY AA.NRRECURSOSAGENDCONT) AA ON (AA.NRRECURSOSAGENDCONT = AG.NRRECURSOSAGENDCONT)
          WHERE RE.ID = :USER_ID
            AND RA.IDSITUARECGEND = '1')
SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':CURRENT_DATE_TIME' => $currentDateTime,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findUserAppropriationTasksGroupedByProject($userId, $filter)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                           TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED, TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN,
                           'N' AS PROJECT, TAS.TASK_STATUS, PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME, TAS.ESTIMATED_BEGIN_DATE,
                           TAS.NAME AS TASK_NAME, CASE WHEN (PRT.EXTERNALTYPE = 'C')
                                                       THEN 'Y'
                                                       ELSE 'N'
                                                END AS IS_RECURRING
                      FROM TASK TAS
                      JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                      JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
                      JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
                      JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
                      JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
                     WHERE TAS.STATUS = :TASK_STATUS_ACTIVE
                       AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
                       AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
                       AND TAS.IS_ISSUE = 'N'
                       AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN))
             WHERE 1 = 1
            $filter
SQL;

        $query = "
            SELECT NULL AS ID,PROJECT_TREE AS ID_TREE,PROJECT_NAME AS NAME, '' AS PROJECT_TREE, NULL AS PERCENT_COMPLETED,
                   NULL AS PERCENT_COMPLETED_ORIGIN, 'Y' AS PROJECT, NULL AS TASK_STATUS, NULL AS PROJECT_ID,NULL AS PROJECT_NAME,
                   NULL AS ESTIMATED_BEGIN_DATE, NULL AS TASK_NAME, 'N' AS IS_RECURRING
              FROM ( $query )
          GROUP BY PROJECT_TREE, PROJECT_NAME
             UNION
            $query";

          $params = array(
            ':USER_ID' => $userId,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findUserCompletedTasksGroupedByProject($userId, $currentDate)
    {
        $currentDateTime = $currentDate . ' 23:59:59';
        $query = <<<SQL
            SELECT * FROM (
            SELECT PRO.ID, 'P'||PRO.ID AS ID_TREE, PRO.NAME, '' AS PROJECT_TREE,
                   NULL AS PERCENT_COMPLETED, NULL AS PERCENT_COMPLETED_ORIGIN,
                   'Y' AS PROJECT, 'N' AS CONTRACT, 'N' AS CDA,
                   NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
             WHERE (TAS.ESTIMATED_BEGIN_DATE <= TO_DATE(:CURRENT_DATE_TIME, 'DD/MM/YYYY HH24:MI:SS')
                    OR
                    TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE)
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
          GROUP BY PRO.ID, PRO.NAME, TAS.PERCENT_COMPLETED
             UNION
            SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                   TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED,
                   TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN, 'N' AS PROJECT,
                   'N' AS CONTRACT, 'N' AS CDA, NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
             WHERE (TAS.ESTIMATED_BEGIN_DATE <= TO_DATE(:CURRENT_DATE_TIME, 'DD/MM/YYYY HH24:MI:SS')
                    OR
                    TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE)
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
          GROUP BY TAS.ID, TAS.NAME, TAS.PROJECT, TAS.PERCENT_COMPLETED)
          UNION
          SELECT * FROM (
            SELECT CO.NRSEQCONTRATO AS ID, 'C'||CO.NRSEQCONTRATO AS ID_TREE, CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS NAME,
                NULL AS PROJECT_TREE, NULL AS PERCENT_COMPLETED, NULL AS PERCENT_COMPLETED_ORIGIN,
                'N' AS PROJECT, 'Y' AS CONTRACT, 'Y' AS CDA,
                NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
           FROM RESOURCE_T            RE
           JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
           JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
           JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
           JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                                        AND TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') = :CURRENT_DATE
           JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
           JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
          WHERE RE.ID = :USER_ID
			AND RA.IDSITUARECGEND = '1'
       GROUP BY CO.NRSEQCONTRATO, CO.NRCONTRATOFCT, CO.DSREFERENCIACONT
          UNION
         SELECT IC.NRITEMOCONTRATO AS ID, 'S'||IC.NRITEMOCONTRATO AS ID_TREE, IC.NMITEMCONTRATO AS NAME,
                'C'||IC.NRSEQCONTRATO AS PROJECT_TREE,
                round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED,
                round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED_ORIGIN,
                'N' AS PROJECT, 'N' AS CONTRACT, 'Y' AS CDA,
                RA.NRRECURSOSAGENDCONT, AG.NRAPROPAGENDAMENTO
           FROM RESOURCE_T            RE
           JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
           JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
           JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
           JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                                        AND TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') = :CURRENT_DATE
           JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
           JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
           JOIN (SELECT SUM(AA.QTHORASAPROPRIADAS) AS TOTAL_APPROPRIATION, AA.NRRECURSOSAGENDCONT
                   FROM FCT_APROPAGENDAMENTO AA
                  GROUP BY AA.NRRECURSOSAGENDCONT) AA ON (AA.NRRECURSOSAGENDCONT = AG.NRRECURSOSAGENDCONT)
          WHERE RE.ID = :USER_ID
            AND RA.IDSITUARECGEND = '1'
            AND DECODE(IC.QTMEDIDACONTR, 0, 100,(TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR) < 100)
SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':CURRENT_DATE_TIME' => $currentDateTime,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findUserTasksAppropriationCompletedTasks($userId, $filter)
    {

        $query = <<<SQL
            SELECT *
              FROM (SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                           TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED, TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN,
                           'N' AS PROJECT, TAS.TASK_STATUS, PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME, TAS.ESTIMATED_BEGIN_DATE,
                           TAS.NAME AS TASK_NAME, CASE WHEN (PRT.EXTERNALTYPE = 'C')
                                                       THEN 'Y'
                                                       ELSE 'N'
                                                END AS IS_RECURRING
                      FROM TASK TAS
                      JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                      JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
                      JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
                      JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
                      JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
                     WHERE TAS.STATUS = :TASK_STATUS_ACTIVE
                       AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
                       AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
                       AND TAS.IS_ISSUE = 'N'
                       AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN))
             WHERE 1 = 1
            $filter
SQL;

        $query = "
            SELECT NULL AS ID,PROJECT_TREE AS ID_TREE,PROJECT_NAME AS NAME, '' AS PROJECT_TREE, NULL AS PERCENT_COMPLETED,
                   NULL AS PERCENT_COMPLETED_ORIGIN, 'Y' AS PROJECT, NULL AS TASK_STATUS, NULL AS PROJECT_ID,NULL AS PROJECT_NAME,
                   NULL AS ESTIMATED_BEGIN_DATE, NULL AS TASK_NAME, 'N' AS IS_RECURRING
              FROM ( $query )
          GROUP BY PROJECT_TREE, PROJECT_NAME
             UNION
            $query";

	      $params = array(
            ':USER_ID' => $userId,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findRecurringTasksUserByDayGroupedByProject($userId, $currentDate)
    {
        $currentDateTime = $currentDate . ' 23:59:59';
        $query = <<<SQL
            SELECT PAP.ID, 'P'||PAP.ID AS ID_TREE, PAP.NAME, '' AS PROJECT_TREE, NULL AS PERCENT_COMPLETED,
                   'Y' AS PROJECT, NULL AS TIME_APPROPRIATE
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
              JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
              JOIN PROJECT PAP ON PAP.ID = APP.PROJECT_ID
              JOIN TASK_STATUS PSP ON PSP.ID = PAP.PROJECT_STATUS AND PSP.CONTROL <> :TASK_STATUS_CONTROL_CANCELED
             WHERE TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE = :TYPE_PROJECT_CONTROL_RECURRING_TASKS
               AND APP.TOTAL > 0
          GROUP BY PAP.ID, PAP.NAME, TAS.PERCENT_COMPLETED, APP.TOTAL
             UNION
            SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||PAP.ID AS PROJECT_TREE,
                   TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED,
                   'N' AS PROJECT, APP.TOTAL AS TIME_APPROPRIATE
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
              JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
              JOIN PROJECT PAP ON PAP.ID = APP.PROJECT_ID
              JOIN TASK_STATUS PSP ON PSP.ID = PAP.PROJECT_STATUS AND PSP.CONTROL <> :TASK_STATUS_CONTROL_CANCELED
             WHERE TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE = :TYPE_PROJECT_CONTROL_RECURRING_TASKS
               AND APP.TOTAL > 0
          GROUP BY TAS.ID, TAS.NAME, PAP.ID, TAS.PERCENT_COMPLETED, APP.TOTAL
SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':CURRENT_DATE_TIME' => $currentDateTime,
            ':TYPE_PROJECT_CONTROL_RECURRING_TASKS' => Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUserTasksAppropriationRecurringTasks($userId, $filter)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                           TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED, TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN,
                           'N' AS PROJECT, TAS.TASK_STATUS, PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME, TAS.ESTIMATED_BEGIN_DATE,
                           TAS.NAME AS TASK_NAME
                      FROM TASK TAS
                      JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                      JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
                      JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
                      JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
                      JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
                     WHERE TAS.STATUS = :TASK_STATUS_ACTIVE
                       AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
                       AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
                       AND TAS.IS_ISSUE = 'N'
                       AND PRT.EXTERNALTYPE = :TYPE_PROJECT_CONTROL_RECURRING_TASKS)
             WHERE 1 = 1
            $filter
SQL;

        $query = "
            SELECT NULL AS ID,PROJECT_TREE AS ID_TREE,PROJECT_NAME AS NAME, '' AS PROJECT_TREE, NULL AS PERCENT_COMPLETED,
                   NULL AS PERCENT_COMPLETED_ORIGIN, 'Y' AS PROJECT, NULL AS TASK_STATUS, NULL AS PROJECT_ID,NULL AS PROJECT_NAME,
                   NULL AS ESTIMATED_BEGIN_DATE, NULL AS TASK_NAME
              FROM ( $query )
          GROUP BY PROJECT_TREE, PROJECT_NAME
             UNION
            $query";

          $params = array(
            ':USER_ID' => $userId,
            ':TYPE_PROJECT_CONTROL_RECURRING_TASKS' => Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUserTasksNotCompletedGroupedByProject($userId, $currentDate, $taskFilter, $serviceFilter)
    {
        $currentDateTime = $currentDate . ' 23:59:59';
        $query = <<<SQL
            SELECT PRO.ID, 'P'||PRO.ID AS ID_TREE, PRO.NAME, '' AS PROJECT_TREE,
                   NULL AS PERCENT_COMPLETED, NULL AS PERCENT_COMPLETED_ORIGIN,
                   'Y' AS PROJECT, 'N' AS APPROPRIATED_DAY, 'N' AS CONTRACT, 'N' AS CDA,
                   NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID
             WHERE (TAS.ESTIMATED_BEGIN_DATE <= TO_DATE(:CURRENT_DATE_TIME, 'DD/MM/YYYY HH24:MI:SS')
                    OR
                    TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE)
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
               $taskFilter
          GROUP BY PRO.ID, PRO.NAME, TAS.PERCENT_COMPLETED
             UNION
            SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                   TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED,
                   TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN, 'N' AS PROJECT,
                   CASE WHEN TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
                        THEN 'Y'
                        ELSE 'N'
                    END AS APPROPRIATED_DAY, 'N' AS CONTRACT, 'N' AS CDA,
                   NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
              JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND APP.RESOURCE_T = :USER_ID AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE
             WHERE (TAS.ESTIMATED_BEGIN_DATE <= TO_DATE(:CURRENT_DATE_TIME, 'DD/MM/YYYY HH24:MI:SS')
                    OR
                    APP.ID IS NOT NULL)
               AND TAS.PERCENT_COMPLETED < 100
               AND TAS.STATUS = :TASK_STATUS_ACTIVE
               AND TST.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED)
               AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
               AND TAS.IS_ISSUE = 'N'
               AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN)
               $taskFilter
          GROUP BY TAS.ID, TAS.NAME, TAS.PROJECT, TAS.PERCENT_COMPLETED, APP.BEGIN_HOUR

          UNION

          SELECT * FROM (
            SELECT CO.NRSEQCONTRATO AS ID, 'C'||CO.NRSEQCONTRATO AS ID_TREE, CO.NRCONTRATOFCT || ' - ' || CO.DSREFERENCIACONT AS NAME,
                NULL AS PROJECT_TREE, NULL AS PERCENT_COMPLETED, NULL AS PERCENT_COMPLETED_ORIGIN,
                'N' AS PROJECT, 'N' AS APPROPRIATED_DAY, 'Y' AS CONTRACT, 'Y' AS CDA,
                NULL AS NRRECURSOSAGENDCONT, NULL AS NRAPROPAGENDAMENTO
           FROM RESOURCE_T            RE
           JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
           JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
           JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
           JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                                        AND TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') = '27/04/2020'
           JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
           JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
          WHERE RE.ID = :USER_ID
			AND RA.IDSITUARECGEND = '1'
            $serviceFilter
       GROUP BY CO.NRSEQCONTRATO, CO.NRCONTRATOFCT, CO.DSREFERENCIACONT
          UNION
         SELECT IC.NRITEMOCONTRATO AS ID, 'S'||IC.NRITEMOCONTRATO AS ID_TREE, IC.NMITEMCONTRATO AS NAME,
                'C'||IC.NRSEQCONTRATO AS PROJECT_TREE,
                round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED,
                round(((TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR), 2) || '%' AS PERCENT_COMPLETED_ORIGIN,
                'N' AS PROJECT, 'N' AS APPROPRIATED_DAY, 'N' AS CONTRACT, 'Y' AS CDA,
                RA.NRRECURSOSAGENDCONT, AG.NRAPROPAGENDAMENTO
           FROM RESOURCE_T            RE
           JOIN OPERADOR              OP ON OP.CDOPERADOR = RE.EXTERNAL_CODE
           JOIN PARCNEGOCIO           PN ON PN.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
           JOIN FCT_RECURSOSAGENDCONT RA ON RA.NRPARCNEPESSOA = OP.NRPARCNEGOCIO
           JOIN FCT_APROPAGENDAMENTO  AG ON AG.NRRECURSOSAGENDCONT = RA.NRRECURSOSAGENDCONT
                                        AND TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') = '27/04/2020'
           JOIN FCT_ITEMCONTRATO      IC ON IC.NRITEMOCONTRATO = RA.NRITEMOCONTRATO
           JOIN FCT_CONTRATO          CO ON CO.NRSEQCONTRATO = IC.NRSEQCONTRATO
           JOIN (SELECT SUM(AA.QTHORASAPROPRIADAS) AS TOTAL_APPROPRIATION, AA.NRRECURSOSAGENDCONT
                   FROM FCT_APROPAGENDAMENTO AA
                  GROUP BY AA.NRRECURSOSAGENDCONT) AA ON (AA.NRRECURSOSAGENDCONT = AG.NRRECURSOSAGENDCONT)
          WHERE RE.ID = :USER_ID
            AND RA.IDSITUARECGEND = '1'
            AND DECODE(IC.QTMEDIDACONTR, 0, 100,(TOTAL_APPROPRIATION * 100) / IC.QTMEDIDACONTR) < 100
            $serviceFilter
            )

SQL;
        $params = array(
            ':USER_ID' => $userId,
            ':CURRENT_DATE' => $currentDate,
            ':CURRENT_DATE_TIME' => $currentDateTime,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findUserTasksAppropriationTasksNotCompleted($userId, $filter)
    {

        $query = <<<SQL
            SELECT *
              FROM (SELECT TAS.ID, 'T'||TAS.ID AS ID_TREE, TAS.NAME, 'P'||TAS.PROJECT AS PROJECT_TREE,
                           TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED, TAS.PERCENT_COMPLETED || '%' AS PERCENT_COMPLETED_ORIGIN,
                           'N' AS PROJECT, TAS.TASK_STATUS, PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME, TAS.ESTIMATED_BEGIN_DATE,
                           TAS.NAME AS TASK_NAME
                      FROM TASK TAS
                      JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
                      JOIN ALLOCATION ALO ON ALO.TASK = TAS.ID AND ALO.RESOURCE_T = :USER_ID
                      JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
                      JOIN TASK_STATUS TSP ON TSP.ID = PRO.PROJECT_STATUS
                      JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
                     WHERE TAS.STATUS = :TASK_STATUS_ACTIVE
                       AND PRO.STATUS = :PROJECT_STATUS_ACTIVE
                       AND TSP.CONTROL NOT IN (:TASK_STATUS_CONTROL_CONCLUDED, :TASK_STATUS_CONTROL_CANCELED)
                       AND TAS.IS_ISSUE = 'N'
                       AND PRT.EXTERNALTYPE NOT IN (:ARRAY_EXTERNAL_TYPE_NOT_IN))
             WHERE 1 = 1
            $filter
SQL;

        $query = "
            SELECT NULL AS ID,PROJECT_TREE AS ID_TREE,PROJECT_NAME AS NAME, '' AS PROJECT_TREE, NULL AS PERCENT_COMPLETED,
                   NULL AS PERCENT_COMPLETED_ORIGIN, 'Y' AS PROJECT, NULL AS TASK_STATUS, NULL AS PROJECT_ID,NULL AS PROJECT_NAME,
                   NULL AS ESTIMATED_BEGIN_DATE, NULL AS TASK_NAME
              FROM ( $query )
          GROUP BY PROJECT_TREE, PROJECT_NAME
             UNION
            $query";

          $params = array(
            ':USER_ID' => $userId,
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_RECURRING_TASKS,
                                                  Entities\ProjectType::TYPE_PROJECT_CONTROL_ISSUE),
            ':TASK_STATUS_ACTIVE'    => Entities\Task::ACTIVE,
            ':PROJECT_STATUS_ACTIVE' => Entities\Project::IS_ACTIVE,
            ':TASK_STATUS_CONTROL_CONCLUDED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );

        $paramTypes = array(
            ':ARRAY_EXTERNAL_TYPE_NOT_IN' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findTaskInfoByPN($taskId, $currentDate, $userId)
    {
        $query = <<<SQL
            SELECT TAS.ID, TAS.NAME AS TASK_NAME, PRO.ID AS PROJECT_ID, PRO.NAME AS PROJECT_NAME, TAS.TASK_STATUS AS TASK_STATUS_ID,
                   TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE, TAS.TIME_PLANNED,
                   SUM(APP.TOTAL) AS TIME_APPROPRIATE, TAS.PERCENT_COMPLETED, TAS.DESCRIPTION AS DESCRIPTION_TASK,
                   APP.ID AS APPROPRIATION_ID, APP.DESCRIPTION, PAP.ID AS PROJECT_APPROPRIATION_ID, PAP.NAME AS PROJECT_APPROPRIATION_NAME,
                   CASE WHEN (PRT.EXTERNALTYPE = 'C') THEN 'Y' ELSE 'N' END AS IS_RECURRING,
                   CASE WHEN (TST.CONTROL = 'C')      THEN 'Y' ELSE 'N' END AS IS_FINALIZED
              FROM TASK TAS
              JOIN PROJECT PRO ON PRO.ID = TAS.PROJECT
              JOIN PROJECT_TYPE PRT ON PRT.ID = PRO.PROJECT_TYPE
              JOIN TASK_STATUS TST ON TST.ID = TAS.TASK_STATUS
         LEFT JOIN APPROPRIATION APP ON APP.TASK = TAS.ID AND TO_CHAR(APP.BEGIN_HOUR, 'DD/MM/YYYY') = :CURRENT_DATE AND APP.RESOURCE_T = :USER_ID
         LEFT JOIN PROJECT PAP ON PAP.ID = APP.PROJECT_ID
         LEFT JOIN TASK_STATUS PSP ON PSP.ID = PAP.PROJECT_STATUS AND PSP.CONTROL <> :TASK_STATUS_CONTROL_CANCELED
             WHERE TAS.ID = :TASK_ID
             GROUP BY TAS.ID, TAS.NAME, PRO.ID, PRO.NAME, TAS.TASK_STATUS, TAS.ESTIMATED_BEGIN_DATE, TAS.TIME_PLANNED,
                   TAS.PERCENT_COMPLETED, TAS.DESCRIPTION, APP.ID, APP.DESCRIPTION, PRT.EXTERNALTYPE, PAP.ID, PAP.NAME, TST.CONTROL
SQL;

        $params = array(
            ':USER_ID' => $userId,
            ':TASK_ID' => $taskId,
            ':CURRENT_DATE' => $currentDate,
            ':TASK_STATUS_CONTROL_CANCELED' => Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getSumOfDurationOfTasksById($taskIdsFilter)
    {
        $query = <<<SQL
            SELECT SUM(DURATION) AS TOTAL_DURATION
              FROM TASK
             WHERE 1 = 1
             $taskIdsFilter
SQL;
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result[0]['TOTAL_DURATION'] ? $result[0]['TOTAL_DURATION'] : 0;
    }

    public function findTaskByProject($projectId, $filter)
    {
        $query = <<<SQL
            SELECT *
              FROM TASK T
             WHERE T.PROJECT = :PROJECT_ID
               AND T.STATUS <> :DELETED
            $filter
SQL;

        $params = array(
            'PROJECT_ID'    => $projectId,
            'DELETED'       => Entities\Task::DELETED,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    public function findErrorTaskFromErrorMerge($task_id){

        $query = <<<SQL
		  SELECT TRT.TASK , TRT.TYPE
            FROM TASK_RELATIONSHIP TRG
            JOIN TASK_RELATIONSHIP TRO ON TRO.TASK_RELATIONSHIP = TRG.TASK AND TRO.TYPE = 'O'
            JOIN TASK_RELATIONSHIP TRE ON TRE.TASK_RELATIONSHIP = TRO.TASK AND TRE.TYPE = 'E'
            JOIN TASK_RELATIONSHIP TRT ON TRT.TASK_RELATIONSHIP = TRE.TASK AND TRT.TYPE = 'T'
           WHERE TRG.TASK_RELATIONSHIP = :TASK_ID


SQL;
        $params = array(
            'TASK_ID' => $task_id
        );
        $tasks = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        return $tasks;
    }

    public function getBaselineTaskFromOriginBaselineProject($taskId, $projectId)
    {
        $query = <<<SQL
		  SELECT T.ID
            FROM TASK_RELATIONSHIP TR
            JOIN TASK T ON T.ID = TR.TASK_RELATIONSHIP
            JOIN PROJECT P ON P.ID = T.PROJECT
            JOIN PROJECT_RELATIONSHIP PR ON PR.PROJECT_RELATIONSHIP = P.ID AND PR.TYPE = 'L' AND PR.PROJECT = :PROJECT_ID
           WHERE TR.TASK = :TASK_ID AND TR.TYPE = :BASELINE_TYPE
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'PROJECT_ID' => $projectId,
            'BASELINE_TYPE' => Entities\TaskRelationship::BASELINE_TYPE
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function getBaselineProjectIdsFromTaskId($taskId)
    {
        $query = <<<SQL
		  SELECT DISTINCT PR.PROJECT
            FROM TASK T
            JOIN TASK_RELATIONSHIP TR ON TR.TASK = T.ID
            JOIN TASK TA ON TA.ID = TR.TASK_RELATIONSHIP
            JOIN PROJECT P ON P.ID = TA.PROJECT
            JOIN PROJECT_RELATIONSHIP PR ON PR.PROJECT_RELATIONSHIP = P.ID
           WHERE T.ID = :TASK_ID
SQL;
        $params = array(
            'TASK_ID' => $taskId,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getBaselineTask($taskId, $baselineProject)
    {
        $query = <<<SQL
		  SELECT TA.*
            FROM TASK T
            JOIN TASK_RELATIONSHIP TR ON TR.TASK = T.ID
            JOIN TASK TA ON TA.ID = TR.TASK_RELATIONSHIP AND TA.PROJECT = :PROJECT_ID
           WHERE T.ID = :TASK_ID
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'PROJECT_ID' => $baselineProject
        );
        $task =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return (!empty($task)) ? $task[0] : $task;
    }

    public function findBaselineTaskOfTaskCurrentProject($taskId, $baselineProjectId)
    {
        $query = <<<SQL
		  SELECT T.ID
            FROM TASK_RELATIONSHIP TR
            JOIN TASK T ON T.ID = TR.TASK_RELATIONSHIP
           WHERE TR.TASK = :TASK_ID AND TR.TYPE = :TR_BASELINE_TYPE AND T.PROJECT = :PROJECT_ID
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'PROJECT_ID' => $baselineProjectId,
            'TR_BASELINE_TYPE' => Entities\TaskRelationship::BASELINE_TYPE
        );

        $task = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return (!empty($task)) ? $task[0] : $task;
    }

    public function hasRelationshipWithBaselineTaskInThisProject($taskId, $baselineProject)
    {
        $query = <<<SQL
            SELECT TA.*
              FROM TASK T
              JOIN TASK_RELATIONSHIP TR ON (T.ID = TR.TASK AND TR.TYPE = :TR_BASELINE_TYPE)
              JOIN TASK TA ON (TA.ID = TR.TASK_RELATIONSHIP AND TA.PROJECT = :PROJECT_ID)
              WHERE T.ID = :TASK_ID
SQL;
        $params = array(
            'TASK_ID' => $taskId,
            'PROJECT_ID' => $baselineProject,
            'TR_BASELINE_TYPE' => Entities\TaskRelationship::BASELINE_TYPE
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getChildrenTasks($filterInTasks)
    {
        $query = <<<SQL
            SELECT ID, NAME
              FROM TASK T
             WHERE T.STATUS = 'A'
        START WITH T.ID IN $filterInTasks
  CONNECT BY PRIOR T.ID = T.PARENT
SQL;

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    /**
     * @param array $subReleaseIds
     * @param $organizationId
     * @param $filter
     * @return array
     */
    public function findResponsesOfIssuesAndTasks(array $subReleaseIds, $organizationId, $filter)
    {
        $query = <<<SQL
        SELECT NOME.*, DECODE(RESPONSE, NULL, 'Y', 'N') AS SHOW_BUTTON
          FROM (SELECT DISTINCT DECODE(TI.ID, NULL, TA.ID, TI.ID) AS ID,
                       DECODE(TI.ID, NULL, 'T', 'I') AS TYPE,
                       DECODE(TI.ID, NULL, 'Task', 'Issue') AS LABEL_TYPE,
                       DECODE(TI.NAME, NULL, TA.NAME, TI.NAME) AS NAME,
                       DECODE(TI.ID, NULL, TA.ID, TI.ID) || ' - ' || DECODE(TI.NAME, NULL, TA.NAME, TI.NAME) AS ID_NAME,
                       CASE WHEN (SELECT COUNT(TA.ID)
                                     FROM TASK_RELATIONSHIP TR
                                     JOIN TASK TA ON TA.ID = TR.TASK_RELATIONSHIP
                                     JOIN CLASSIFICATION CL ON CL.ID = TA.CLASSIFICATION AND CL.CONTROL IS NULL
                                    WHERE TYPE = 'I'
                                      AND TASK = DECODE(TI.ID, NULL, TA.ID, TI.ID)) > 1
                            THEN NULL
                            ELSE DECODE(TT.RESPONSE, NULL, TA.RESPONSE, TT.RESPONSE)
                             END AS RESPONSE
                  FROM BRANCH BR
            INNER JOIN SUBRELEASE SR ON BR.SUBRELEASE_ID = SR.ID
            INNER JOIN TASK_BRANCHES TB ON BR.NAME = TB.NAME
            INNER JOIN TASK TA ON TB.TASK = TA.ID
            INNER JOIN CLASSIFICATION CL ON CL.ID = TA.CLASSIFICATION AND CL.CONTROL IS NULL
             LEFT JOIN TASK_RELATIONSHIP TR ON TR.TASK_RELATIONSHIP = TA.ID AND TR.TYPE = 'I'
             LEFT JOIN TASK TI ON TI.ID = TR.TASK
             LEFT JOIN TASK_RELATIONSHIP TRE ON TRE.TASK = TA.ID AND TRE.TYPE = 'M'
             LEFT JOIN TASK TT ON TT.ID = TRE.TASK_RELATIONSHIP
                 WHERE SR.ID IN (:SUBRELEASE_IDS)
                   AND SR.ORGANIZATION_ID = :ORGANIZATION_ID
                   AND TA.STATUS = 'A'
                   AND (TI.STATUS = 'A' OR TI.STATUS IS NULL)) NOME

     WHERE 1 = 1
     $filter
SQL;

        $params = array (
            'SUBRELEASE_IDS' => $subReleaseIds,
            'ORGANIZATION_ID' => $organizationId
        );

        $paramTypes = array(
            'SUBRELEASE_IDS' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findTaskResponsesRelatedSameIssue($issueId)
    {
        $query = <<<SQL
        SELECT TA.ID, TA.NAME,
               DECODE(TT.RESPONSE, NULL, TA.RESPONSE, TT.RESPONSE) AS RESPONSE
         FROM  TASK_RELATIONSHIP TR
         JOIN  TASK TA ON TA.ID = TR.TASK_RELATIONSHIP
         JOIN  CLASSIFICATION CL ON CL.ID = TA.CLASSIFICATION AND CL.CONTROL IS NULL
    LEFT JOIN  TASK_RELATIONSHIP TRE ON TRE.TASK = TA.ID AND TRE.TYPE = 'M'
    LEFT JOIN  TASK TT ON TT.ID = TRE.TASK_RELATIONSHIP
        WHERE  TR.TYPE = 'I'
          AND  TR.TASK = :ISSUE_ID
SQL;

        $params = array (
            'ISSUE_ID' => $issueId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function searchForSingleComments($issueIds){
        $query = <<<SQL
           SELECT TASK AS ISSUE_ID
             FROM (SELECT TASK, COUNT(*)
                     FROM COMMENTS
                    WHERE TASK IN ($issueIds)
                 GROUP BY TASK
                   HAVING COUNT(*) < 2)
SQL;
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }

    public function findTestTasksBySubreleases($ids){
        $query = <<<SQL
            SELECT TAS.ID, TAS.NAME, TAS.TEST_BLOCKED
              FROM SUBRELEASE SBR
              JOIN BRANCH BRA ON BRA.SUBRELEASE_ID = SBR.ID
              JOIN TASK_BRANCHES TBR ON TBR.NAME = BRA.NAME
              JOIN TASK_RELATIONSHIP TRL ON TRL.TASK = TBR.TASK
                                        AND TRL.TYPE = 'T'
              JOIN TASK TAS ON TAS.ID = TRL.TASK_RELATIONSHIP
             WHERE SBR.ID IN (:SUBRELEASE_IDS)
               AND BRA.IS_ACTIVE = 'Y'
               AND TAS.STATUS = 'A'
               AND TAS.IS_PARENT = 'N'
               AND TAS.PERCENT_COMPLETED < 100
SQL;
        $params = array (
            'SUBRELEASE_IDS' => $ids
        );
        $paramTypes = array(
            'SUBRELEASE_IDS' => Connection::PARAM_STR_ARRAY,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    public function findNotConcludedTasksBySubrelease($subReleaseId){
        $query = <<<SQL
              SELECT TAS.ID, TAS.NAME, PRO.ID AS PROJECT_ID, PRO.NAME AS PROJECT_NAME
                FROM SUBRELEASE SBR
                JOIN PROJECT PRO ON PRO.ID = SBR.PROJECT_ID
                JOIN TASK TAS    ON TAS.PROJECT = PRO.ID
           LEFT JOIN TASK_RELATIONSHIP TRL ON TRL.TASK_RELATIONSHIP = TAS.ID
               WHERE SBR.ID IN (:SUBRELEASE_ID)
                 AND TAS.STATUS = 'A'
                 AND TAS.IS_PARENT = 'N'
                 AND TAS.PERCENT_COMPLETED < 100
                 AND (TRL.ID IS NULL OR TRL.TYPE IN ('C', 'D', 'E', 'G', 'M', 'R', 'T'))

               UNION

              SELECT TAS.ID, TAS.NAME, PJP.ID AS PROJECT_ID, PJP.NAME AS PROJECT_NAME
                FROM SUBRELEASE SBR
                JOIN PROJECT PRO      ON PRO.ID = SBR.PROJECT_ID
                JOIN PROJECT PJP      ON PRO.PARENT = PJP.PARENT
                JOIN PROJECT_TYPE PRT ON PRT.ID = PJP.PROJECT_TYPE AND PRT.EXTERNALTYPE = 'T'
                JOIN TASK TAS         ON TAS.PROJECT = PJP.ID
           LEFT JOIN TASK_RELATIONSHIP TRL ON TRL.TASK_RELATIONSHIP = TAS.ID
               WHERE (SBR.ID IN (:SUBRELEASE_ID)
                 AND TAS.STATUS = 'A'
                 AND TAS.IS_PARENT = 'N'
                 AND TAS.PERCENT_COMPLETED < 100
                 AND (TRL.ID IS NULL OR TRL.TYPE IN ('C', 'D', 'E', 'G', 'M', 'R', 'T')))
SQL;
        $params = array (
            'SUBRELEASE_ID' => $subReleaseId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findNotConcludedProjectTestsBySubrelease($subReleaseId){
        $query = <<<SQL
             SELECT PJP.NAME, PJP.ID AS PROJECT_ID  FROM SUBRELEASE SBR
            JOIN PROJECT PRO ON PRO.ID = SBR.PROJECT_ID
            JOIN PROJECT PJP ON PRO.PARENT = PJP.PARENT
            JOIN PROJECT_TYPE PTY ON PJP.PROJECT_TYPE = PTY.ID AND PTY.EXTERNALTYPE = 'T'
            WHERE SBR.ID = :SUBRELEASE_ID
              AND PJP.PERCENT_COMPLETED < 100
SQL;
        $params = array (
            'SUBRELEASE_ID' => $subReleaseId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findTaskWithPendentTestBySubrelease($subReleaseId){
        $query = <<<SQL
            SELECT TA.ID, TA.NAME, TA.DESCRIPTION, TA.CLASSIFICATION AS TASK_CLASSIFICATION
              FROM TASK TA,
                   (SELECT TR.TASK, TA.PROJECT
                      FROM TASK_RELATIONSHIP TR
                     INNER JOIN TASK TA ON TA.ID = TR.TASK
                     WHERE TR.TYPE = 'A'
                       AND TA.STATUS <> 'D'
                       AND TR.TASK IN (SELECT TB.TASK
                                         FROM TASK_BRANCHES TB
                                        WHERE TB.NAME IN (SELECT NAME
                                                            FROM BRANCH
                                                           WHERE SUBRELEASE_ID = :SUBRELEASE_ID
                                                             AND NAME NOT LIKE 'TASK_BKP_%'))
                     UNION
                    SELECT TR.TASK_RELATIONSHIP, TA.PROJECT
                      FROM TASK_RELATIONSHIP TR
                     INNER JOIN TASK TA ON TA.ID = TR.TASK_RELATIONSHIP
                     WHERE TR.TYPE = 'A'
                       AND TA.STATUS <> 'D'
                       AND TR.TASK IN (SELECT TB.TASK
                                         FROM TASK_BRANCHES TB
                                        WHERE TB.NAME IN (SELECT NAME
                                                            FROM BRANCH
                                                           WHERE SUBRELEASE_ID = :SUBRELEASE_ID
                                                             AND NAME NOT LIKE 'TASK_BKP_%'))
                      UNION
                     SELECT TB.TASK, TA.PROJECT
                       FROM TASK_BRANCHES TB
                      INNER JOIN TASK TA ON TA.ID = TB.TASK
                      WHERE TB.NAME IN (SELECT NAME
                                          FROM BRANCH
                                         WHERE SUBRELEASE_ID = :SUBRELEASE_ID
                                           AND NAME NOT LIKE 'TASK_BKP_%')
                        AND TA.STATUS <> 'D') TR
             WHERE TA.STATUS = 'A'
               AND TA.ID NOT IN (SELECT TRL.TASK
                                   FROM TASK_RELATIONSHIP TRL
                                   JOIN (SELECT TASK , MAX(ID) AS ID
                                           FROM TASK_RELATIONSHIP
                                          WHERE TYPE IN ('T', 'E', 'D')
                                       GROUP BY TASK) TAR ON (TAR.ID = TRL.ID)
                                   JOIN TASK TS ON TS.ID = TRL.TASK_RELATIONSHIP
                                  WHERE TRL.TYPE <> 'D'
                                    AND TS.STATUS <> 'D')
               AND TA.ID = TR.TASK
               AND TA.IS_PARENT = 'N'
               AND TR.PROJECT = (SELECT PROJECT_ID
                                   FROM SUBRELEASE
                                  WHERE ID = :SUBRELEASE_ID)
SQL;
        $params = array (
            'SUBRELEASE_ID' => $subReleaseId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}
